// OBJModel.h: interface for the COBJModel class.
//
//////////////////////////////////////////////////////////////////////

#if !defined(AFX_OBJMODEL_H__C6456F01_C5A2_11D4_9161_0020183AD048__INCLUDED_)
#define AFX_OBJMODEL_H__C6456F01_C5A2_11D4_9161_0020183AD048__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000

#define WIN32_LEAN_AND_MEAN

#include <windows.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <string.h>
#include <stdio.h>
#include "glext.h"
#include <iostream>
#include <string>
#include <fstream>

#define TEX_NONE		0x0001 // 1
#define TEX_DEFAULT		0x0002 // 2
#define TEX_S3TC		0x0004 // 4
#define TEX_PALETTED	0x0008 // 8
#define TEX_LOWRES		0x0010 // 16


// Holds info on a Material
// sizeof(tagOBJMaterial) = 80 bytes
typedef struct tagOBJMaterial{
	char Name[32];
	GLuint map_Kd;		// Diffuse Map (normal Texture)
	GLuint map_Ks;		// Specular Map
	GLuint map_Ka;		// Ambient Map
	GLuint map_Bump;	// Bump Map
	GLuint map_d;		// Opacity Map
	float Ambient[4];	// Ambient Color
	float Diffuse[4];	// Diffuse Color
	float Specular[4];	// Specular Color
	char  illum;		// Lighting Model (0 = None, 1 = Amb & Diff, 2 = Full Lighting) 
	short Ns;			// Phong Specular Component (0 - 1000)
	short Ni;			// Refraction Index (1+ )
	char  padding;
} OBJMaterial;

class COBJModel
{
public:
	void NukeData();

	void LockPosition();
	BYTE TranzColor[3]; // Colour which is to be seen as Transparent
	void __fastcall SetOBJPos(float xPos,float yPos,float zPos);
	void __fastcall SetOBJHeading(float Pitch,float Yaw,float Roll);
	BYTE LOCKED; // Boolean 1/0 Position Pre-Transformed?
	float Pos[3];
	float Heading[3];
	int texsUsed;
	int ModelNumber;
	// Filenames
	char* ObjectFilename;
	char* OBJDirName;
	char* MaterialFilename;
	//Texture 
	int UseTextures;
	int TexturesLoaded;
	int Transparent;
	// BoundingBox {xmin,xmax,ymin,ymax,zmin,zmax}
	float Box[6];
	
	// Vertices
	int numVertices;
	float *xVertex;
	float *yVertex;
	float *zVertex;
	// Faces
	int numFaces;
	int (*Faces)[9]; // vx/vt/vn vx/vt/vn vx/vt/vn
	// Methods

	void SetVertexArrays();
	int VTX;
	GLfloat *VertexArray;
	GLfloat *NormalArray;
	GLfloat *TexCoordArray;

	int ReadMTL(int Flags=0x0002);
	int ReadMTL2(int Flags=0x0002);
	void DrawModel();
	void BuildList();
	COBJModel();
	void LoadCOBJModel(char *dirname,char *filename);
	void LoadCOBJModel2(std::string dirname,std::string filename);
	virtual ~COBJModel();

//TEMP
float Facing[3];
//END_TEMP
	
	// Texture Coords
	int numTextureCoords;
	float *uCoord, *vCoord;
	// Normals
	int numNormals;
	float *iNormal, *jNormal, *kNormal;
protected:
	void DrawModel2(); 
	
	
	// Display List
	GLuint OBJList;
	int* FaceTexNum;
	BYTE PALLETEDEXT;		// Boolean 1/0 Pal Ext Available?
	BYTE S3TCEXT;			// Boolean 1/0 S3TC Ext Available?
	BYTE LOWRES;			// Boolean 1/0 
	BYTE LISTUSED;			// Boolean 1/0 Display List Created?
	OBJMaterial *Materials; // List of Material Structs
	GLuint *texture;		// Model's Texture IDs
};

#endif // !defined(AFX_OBJMODEL_H__C6456F01_C5A2_11D4_9161_0020183AD048__INCLUDED_)
