//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAux2DArray.h                                       //
//   Description: Class for handling 2D arrays                  //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_2DARRAY_H_
#define _GF_C_2DARRAY_H_


#include "gfDefines.h"
#include <malloc.h>
#define NULL 0


template<class T>
class gfAux2DArray
{
private:
    T* m_array;
    gfInt m_i_size;
    gfInt m_j_size;

public:
    gfAux2DArray(void)
    {
        m_array = NULL;
        resize(0, 0);
    }

    gfAux2DArray(gfInt i_size, gfInt j_size)
    {
        m_array = NULL;
        resize(i_size, j_size);
        memset(m_array, 0, i_size * j_size * sizeof(T));
    }

    ~gfAux2DArray(void)
    {
        resize(0, 0);
    }

    gfAux2DArray& operator=(const gfAux2DArray<T> *a)
    {
        resize(a->GetISize(), a->GetJSize());

        for (gfInt i = 0;i < m_i_size; i++)
            for (gfInt j = 0;j < m_j_size; j++)
                Set(i, a->at(i, j));

        return *this;
    }

    GFRESULT resize(gfInt i_size, gfInt j_size)
    {
        if (i_size < 0 || j_size < 0)
            return GF_E_BADARG;

        if (i_size * j_size > 0)
        {
            if (m_array)
            {
                m_array = (T*) realloc(m_array, i_size * j_size * sizeof(T));
            }
            else
            {
                m_array = (T*) malloc(i_size * j_size * sizeof(T));
            }
        }
        else
        {
            free(m_array);
            m_array = NULL;
        }

        // null all newly alocated parts
        /*if (i_size > m_i_size)
        {
            for ()
        }*/

        m_i_size = i_size;
        m_j_size = j_size;

        return GF_S_OK;
    }

    T at(gfInt i, gfInt j)
    {
        if(i < 0 || j < 0)
            return m_array[0];
        if(i > m_i_size - 1 || j > m_j_size - 1)
            return m_array[m_i_size * m_j_size];

        return m_array[i + j * m_i_size];
    }

    GFRESULT Set(gfInt i, gfInt j, T element)
    {
        if(i < 0 || i >= m_i_size || j < 0 || j >= m_j_size)
            return GF_E_BADARG;

        m_array[i + j * m_i_size] = element;

        return GF_S_OK;
    }

    gfInt GetISize(void)
    {
        return m_i_size;
    }

    gfInt GetJSize(void)
    {
        return m_j_size;
    }

    T* GetDataPtr(void)
    {
        return m_array;
    }

    /*GFRESULT push_back(T element)
    {
        return resize(m_size + 1, element);
    }

    GFRESULT Remove(void)
    {
        return resize(m_size - 1);
    }

    GFRESULT erase(gfInt index)
    {
        if(index < 0 || index >= m_size)
            return GF_E_BADARG;

        for (gfInt i = index;i < m_size - 1; i++)
                Set(i, at(i + 1));

        return resize(m_size - 1);
    }*/

    GFRESULT clear(void)
    {
        return resize(0, 0);
    }
};


#endif