//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAux4DArray.h                                       //
//   Description: Class for handling 4D arrays                  //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_4DARRAY_H_
#define _GF_C_4DARRAY_H_


#include "gfDefines.h"
#include <malloc.h>
#define NULL 0


template<class T>
class gfAux4DArray
{
private:
    T* m_array;
    gfInt m_i_size;
    gfInt m_j_size;
    gfInt m_k_size;
    gfInt m_l_size;

public:
    gfAux4DArray(void)
    {
        m_array = NULL;
        resize(0, 0, 0, 0);
    }

    gfAux4DArray(gfInt i_size, gfInt j_size, gfInt k_size, gfInt l_size)
    {
        m_array = NULL;
        resize(i_size, j_size, k_size, l_size);
        memset(m_array, 0, i_size * j_size * k_size * l_size * sizeof(T));
    }

    ~gfAux4DArray(void)
    {
        resize(0, 0, 0, 0);
    }

    gfAux4DArray& operator=(const gfAux4DArray<T> *a)
    {
        resize(a->GetISize(), a->GetJSize(), a->GetKSize(), a->GetLSize());

        for (gfInt i = 0;i < m_i_size; i++)
            for (gfInt j = 0;j < m_j_size; j++)
                for (gfInt k = 0;k < m_k_size; k++)
                    for (gfInt l = 0;l < m_l_size; l++)
                        Set(i, j, k, l, a->at(i, j, k, l));

        return *this;
    }

    GFRESULT resize(gfInt i_size, gfInt j_size, gfInt k_size, gfInt l_size)
    {
        if (i_size < 0 || j_size < 0 || k_size < 0 || l_size < 0)
            return GF_E_BADARG;

        if (i_size * j_size * k_size * l_size > 0)
        {
            if (m_array)
            {
                m_array = (T*) realloc(m_array, i_size * j_size * k_size * l_size * sizeof(T));
            }
            else
            {
                m_array = (T*) malloc(i_size * j_size * k_size * l_size * sizeof(T));
            }
        }
        else
        {
            free(m_array);
            m_array = NULL;
        }

        // null all newly alocated parts
        /*if (i_size > m_i_size)
        {
        for ()
        }*/

        m_i_size = i_size;
        m_j_size = j_size;
        m_k_size = k_size;
        m_l_size = l_size;

        return GF_S_OK;
    }

    T at(gfInt i, gfInt j, gfInt k, gfInt l)
    {
        if(i < 0 || j < 0 || k < 0 || l < 0)
            return m_array[0];
        if(i > m_i_size - 1 || j > m_j_size - 1 || k > m_k_size - 1 || l > m_l_size - 1)
            return m_array[m_i_size * m_j_size * m_k_size * m_l_size - 1];

        return m_array[i + j * m_i_size + k * m_i_size * m_j_size + l * m_i_size * m_j_size * m_k_size];
    }

    GFRESULT Set(gfInt i, gfInt j, gfInt k, gfInt l, T element)
    {
        if(i < 0 || i >= m_i_size || j < 0 || j >= m_j_size || k < 0 || k >= m_k_size || l < 0 || l >= m_l_size)
            return GF_E_BADARG;

        m_array[i + j * m_i_size + k * m_i_size * m_j_size + l * m_i_size * m_j_size * m_k_size] = element;

        return GF_S_OK;
    }

    gfInt GetISize(void)
    {
        return m_i_size;
    }

    gfInt GetJSize(void)
    {
        return m_j_size;
    }

    gfInt GetKSize(void)
    {
        return m_k_size;
    }

    gfInt GetLSize(void)
    {
        return m_l_size;
    }

    T* GetDataPtr(void)
    {
        return m_array;
    }

    GFRESULT clear(void)
    {
        return resize(0, 0, 0, 0);
    }
};


#endif