//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAuxBoundingVolume.h                                //
//   Description: Class for representing bounding volumes       //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_AUX_BOUNDING_VOLUME_H_
#define _GF_AUX_BOUNDING_VOLUME_H_

#include <windows.h>
#include <stdio.h>
#include "gfDefines.h"
#include "gfPoint.h"
#include "gfVector.h"
#include "gfMatrix.h"


class gfAuxBoundingVolume
{
public:

    virtual gfBoundVolID GetID(void) = 0;
	virtual GFRESULT RayIntersection(gfPoint3* ray_begin, gfVector3* ray_dir, gfDouble* tmin, gfDouble* tmax) = 0;
    virtual GFRESULT OGLDraw(gfLong flags) = 0;
    virtual GFRESULT Transform(gfMatrix4* matrix) = 0;
    virtual gfPoint3* GetCenter(void) = 0;
};


// bounding sphere
class gfAuxBoundingShere : public gfAuxBoundingVolume
{
public:

    gfDouble m_middle_x;
    gfDouble m_middle_y;
    gfDouble m_middle_z;

    gfDouble m_radius;

    gfAuxBoundingShere(void);

    // gfAuxBoundingVolume methods
    gfBoundVolID GetID(void);
	GFRESULT RayIntersection(gfPoint3* ray_begin, gfVector3* ray_dir, gfDouble* tmin, gfDouble* tmax);
    GFRESULT OGLDraw(gfLong flags);
    GFRESULT Transform(gfMatrix4* matrix);
    gfPoint3* GetCenter(void);

    // gfAuxBoundingShere methods
    GFRESULT InitFromPoints(gfInt num_points, gfPoint3* points);
};

// axis aligned bounding box
class gfAuxAABB : public gfAuxBoundingVolume
{
public:

    gfDouble m_min_x, m_max_x, m_min_y, m_max_y, m_min_z, m_max_z;

    gfAuxAABB(void);

    // gfAuxBoundingVolume methods
    gfBoundVolID GetID(void);
	GFRESULT RayIntersection(gfPoint3* ray_begin, gfVector3* ray_dir, gfDouble* tmin, gfDouble* tmax);
    GFRESULT OGLDraw(gfLong flags);
    GFRESULT Transform(gfMatrix4* matrix);
    gfPoint3* GetCenter(void);

    // gfAuxAABB methods
    GFRESULT InitFromPoints(gfInt num_points, gfPoint3* points);
};

// oriented bounding box
class gfAuxOBB : public gfAuxBoundingVolume
{
public:

    gfDouble  m_middle_x;
    gfDouble  m_middle_y;
    gfDouble  m_middle_z;

    gfDouble m_axis1_x;
    gfDouble m_axis1_y;
    gfDouble m_axis1_z;

    gfDouble m_axis2_x;
    gfDouble m_axis2_y;
    gfDouble m_axis2_z;

    gfDouble m_axis3_x;
    gfDouble m_axis3_y;
    gfDouble m_axis3_z;
   
    gfAuxOBB(void);

    // gfAuxBoundingVolume methods
    gfBoundVolID GetID(void);
	GFRESULT RayIntersection(gfPoint3* ray_begin, gfVector3* ray_dir, gfDouble* tmin, gfDouble* tmax);
    GFRESULT OGLDraw(gfLong flags);
    GFRESULT Transform(gfMatrix4* matrix);
    gfPoint3* GetCenter(void);

    // gfAuxOBB methods
    GFRESULT InitFromPoints(gfInt num_points, gfPoint3* points);
};

#endif