//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAuxGeomUtils.cpp                                   //
//   Description: Geometric functions                           //
//                                                              //
//////////////////////////////////////////////////////////////////

#include "gfAuxGeomUtils.h"
#include "gfObjectFactory.h"
#include "gfAuxNumUtils.h"


GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix2* matrix)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2);
    point->Set(ax, ay);

    return GF_S_OK;
}

GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix2* matrix, gfPoint2* result)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2);
    result->Set(ax, ay);

    return GF_S_OK;
}

GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix3* matrix)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + matrix->at(1, 3);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + matrix->at(2, 3);
    point->Set(ax, ay);

    return GF_S_OK;
}

GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix3* matrix, gfPoint2* result)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + matrix->at(1, 3);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + matrix->at(2, 3);
    result->Set(ax, ay);

    return GF_S_OK;
}

GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix3* matrix)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + point->Z() * matrix->at(1, 3);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + point->Z() * matrix->at(2, 3);
    gfDouble az = point->X() * matrix->at(3, 1) + point->Y() * matrix->at(3, 2) + point->Z() * matrix->at(3, 3);

    point->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix3* matrix, gfPoint3* result)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + point->Z() * matrix->at(1, 3);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + point->Z() * matrix->at(2, 3);
    gfDouble az = point->X() * matrix->at(3, 1) + point->Y() * matrix->at(3, 2) + point->Z() * matrix->at(3, 3);

    result->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix4* matrix)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + point->Z() * matrix->at(1, 3) + matrix->at(1, 4);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + point->Z() * matrix->at(2, 3) + matrix->at(2, 4);
    gfDouble az = point->X() * matrix->at(3, 1) + point->Y() * matrix->at(3, 2) + point->Z() * matrix->at(3, 3) + matrix->at(3, 4);

    point->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix4* matrix, gfPoint3* result)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + point->Z() * matrix->at(1, 3) + matrix->at(1, 4);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + point->Z() * matrix->at(2, 3) + matrix->at(2, 4);
    gfDouble az = point->X() * matrix->at(3, 1) + point->Y() * matrix->at(3, 2) + point->Z() * matrix->at(3, 3) + matrix->at(3, 4);

    result->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformPoint4(gfPoint4* point, gfMatrix4* matrix)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + point->Z() * matrix->at(1, 3) + point->W() * matrix->at(1, 4);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + point->Z() * matrix->at(2, 3) + point->W() * matrix->at(2, 4);
    gfDouble az = point->X() * matrix->at(3, 1) + point->Y() * matrix->at(3, 2) + point->Z() * matrix->at(3, 3) + point->W() * matrix->at(3, 4);
    gfDouble aw = point->X() * matrix->at(4, 1) + point->Y() * matrix->at(4, 2) + point->Z() * matrix->at(4, 3) + point->W() * matrix->at(4, 4);

    point->Set(ax, ay, az, aw);

    return GF_S_OK;
}

GFRESULT gfTransformPoint4(gfPoint4* point, gfMatrix4* matrix, gfPoint4* result)
{
    gfDouble ax = point->X() * matrix->at(1, 1) + point->Y() * matrix->at(1, 2) + point->Z() * matrix->at(1, 3) + point->W() * matrix->at(1, 4);
    gfDouble ay = point->X() * matrix->at(2, 1) + point->Y() * matrix->at(2, 2) + point->Z() * matrix->at(2, 3) + point->W() * matrix->at(2, 4);
    gfDouble az = point->X() * matrix->at(3, 1) + point->Y() * matrix->at(3, 2) + point->Z() * matrix->at(3, 3) + point->W() * matrix->at(3, 4);
    gfDouble aw = point->X() * matrix->at(4, 1) + point->Y() * matrix->at(4, 2) + point->Z() * matrix->at(4, 3) + point->W() * matrix->at(4, 4);

    result->Set(ax, ay, az, aw);

    return GF_S_OK;
}

GFRESULT gfTransformVector2(gfVector2* vector, gfMatrix2* matrix)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2);
    vector->Set(ax, ay);

    return GF_S_OK;
}

GFRESULT gfTransformVector2(gfVector2* vector, gfMatrix2* matrix, gfVector2* result)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2);
    result->Set(ax, ay);

    return GF_S_OK;
}

GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix3* matrix)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2) + vector->Z() * matrix->at(1, 3);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2) + vector->Z() * matrix->at(2, 3);
    gfDouble az = vector->X() * matrix->at(3, 1) + vector->Y() * matrix->at(3, 2) + vector->Z() * matrix->at(3, 3);

    vector->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix3* matrix, gfVector3* result)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2) + vector->Z() * matrix->at(1, 3);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2) + vector->Z() * matrix->at(2, 3);
    gfDouble az = vector->X() * matrix->at(3, 1) + vector->Y() * matrix->at(3, 2) + vector->Z() * matrix->at(3, 3);

    result->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix4* matrix)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2) + vector->Z() * matrix->at(1, 3);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2) + vector->Z() * matrix->at(2, 3);
    gfDouble az = vector->X() * matrix->at(3, 1) + vector->Y() * matrix->at(3, 2) + vector->Z() * matrix->at(3, 3);

    vector->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix4* matrix, gfVector3* result)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2) + vector->Z() * matrix->at(1, 3);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2) + vector->Z() * matrix->at(2, 3);
    gfDouble az = vector->X() * matrix->at(3, 1) + vector->Y() * matrix->at(3, 2) + vector->Z() * matrix->at(3, 3);

    result->Set(ax, ay, az);

    return GF_S_OK;
}

GFRESULT gfTransformVector4(gfVector4* vector, gfMatrix4* matrix)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2) + vector->Z() * matrix->at(1, 3) + vector->W() * matrix->at(1, 4);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2) + vector->Z() * matrix->at(2, 3) + vector->W() * matrix->at(2, 4);
    gfDouble az = vector->X() * matrix->at(3, 1) + vector->Y() * matrix->at(3, 2) + vector->Z() * matrix->at(3, 3) + vector->W() * matrix->at(3, 4);
    gfDouble aw = vector->X() * matrix->at(4, 1) + vector->Y() * matrix->at(4, 2) + vector->Z() * matrix->at(4, 3) + vector->W() * matrix->at(4, 4);

    vector->Set(ax, ay, az, aw);

    return GF_S_OK;
}

GFRESULT gfTransformVector4(gfVector4* vector, gfMatrix4* matrix, gfVector4* result)
{
    gfDouble ax = vector->X() * matrix->at(1, 1) + vector->Y() * matrix->at(1, 2) + vector->Z() * matrix->at(1, 3) + vector->W() * matrix->at(1, 4);
    gfDouble ay = vector->X() * matrix->at(2, 1) + vector->Y() * matrix->at(2, 2) + vector->Z() * matrix->at(2, 3) + vector->W() * matrix->at(2, 4);
    gfDouble az = vector->X() * matrix->at(3, 1) + vector->Y() * matrix->at(3, 2) + vector->Z() * matrix->at(3, 3) + vector->W() * matrix->at(3, 4);
    gfDouble aw = vector->X() * matrix->at(4, 1) + vector->Y() * matrix->at(4, 2) + vector->Z() * matrix->at(4, 3) + vector->W() * matrix->at(4, 4);

    result->Set(ax, ay, az, aw);

    return GF_S_OK;
}

gfVector2* gfSubstractPoints2(gfPoint2* pt1, gfPoint2* pt2)
{
    gfVector2* vector = gfCreateVector2();
    vector->Set(pt1->X() - pt2->X(), pt1->Y() - pt2->Y());

    return vector;
}

GFRESULT gfSubstractPoints2(gfPoint2* pt1, gfPoint2* pt2, gfVector2* result)
{
    result->Set(pt1->X() - pt2->X(), pt1->Y() - pt2->Y());

    return GF_S_OK;
}

gfVector3* gfSubstractPoints3(gfPoint3* pt1, gfPoint3* pt2)
{
    gfVector3* vector = gfCreateVector3();
    vector->Set(pt1->X() - pt2->X(), pt1->Y() - pt2->Y(), pt1->Z() - pt2->Z());

    return vector;
}

GFRESULT gfSubstractPoints3(gfPoint3* pt1, gfPoint3* pt2, gfVector3* result)
{
    result->Set(pt1->X() - pt2->X(), pt1->Y() - pt2->Y(), pt1->Z() - pt2->Z());

    return GF_S_OK;
}

gfVector4* gfSubstractPoints4(gfPoint4* pt1, gfPoint4* pt2)
{
    gfVector4* vector = gfCreateVector4();
    vector->Set(pt1->X() - pt2->X(), pt1->Y() - pt2->Y(), pt1->Z() - pt2->Z(), pt1->W() - pt2->W());

    return vector;
}

GFRESULT gfSubstractPoints4(gfPoint4* pt1, gfPoint4* pt2, gfVector4* result)
{
    result->Set(pt1->X() - pt2->X(), pt1->Y() - pt2->Y(), pt1->Z() - pt2->Z(), pt1->W() - pt2->W());

    return GF_S_OK;
}

gfDouble gfScalarProduct2(gfVector2* v1, gfVector2* v2)
{
    return (v1->X() * v2->X() + v1->Y() * v2->Y());
}

gfDouble gfScalarProduct3(gfVector3* v1, gfVector3* v2)
{
    return (v1->X() * v2->X() + v1->Y() * v2->Y() + v1->Z() * v2->Z());
}

gfDouble gfScalarProduct4(gfVector4* v1, gfVector4* v2)
{
    return (v1->X() * v2->X() + v1->Y() * v2->Y() + v1->Z() * v2->Z() + v1->W() * v2->W());
}

gfDouble gfVectorsAngle2(gfVector2* v1, gfVector2* v2)
{
    gfDouble alfa = gfScalarProduct2(v1, v2);

    gfDouble l1 = v1->Length();
    gfDouble l2 = v2->Length();
    if (!gfIsZero(l1))
        alfa /= l1;
    if (!gfIsZero(l2))
        alfa /= l2;

    if (alfa < -1.0)
        alfa = -1.0;
    if (alfa > 1.0)
        alfa = 1.0;

    return acos(alfa);
}

gfDouble gfVectorsAngle3(gfVector3* v1, gfVector3* v2)
{
    gfDouble alfa = gfScalarProduct3(v1, v2);

    gfDouble l1 = v1->Length();
    gfDouble l2 = v2->Length();
    if (!gfIsZero(l1))
        alfa /= l1;
    if (!gfIsZero(l2))
        alfa /= l2;

    return acos(alfa);
}

gfDouble gfVectorsAngle4(gfVector4* v1, gfVector4* v2)
{
    gfDouble alfa = gfScalarProduct4(v1, v2);

    gfDouble l1 = v1->Length();
    gfDouble l2 = v2->Length();
    if (!gfIsZero(l1))
        alfa /= l1;
    if (!gfIsZero(l2))
        alfa /= l2;

    return acos(alfa);
}

gfVector3* gfVectorProduct3(gfVector3* v1, gfVector3* v2)
{
    gfDouble tx = v1->Y() * v2->Z() - v1->Z() * v2->Y();
    gfDouble ty = - v1->X() * v2->Z() + v1->Z() * v2->X();
    gfDouble tz = v1->X() * v2->Y() - v1->Y() * v2->X();

    gfVector3* result = gfCreateVector3();
    result->Set(tx, ty, tz);

    return result;
}

GFRESULT gfVectorProduct3(gfVector3* v1, gfVector3* v2, gfVector3* result)
{
    gfDouble tx = v1->Y() * v2->Z() - v1->Z() * v2->Y();
    gfDouble ty = - v1->X() * v2->Z() + v1->Z() * v2->X();
    gfDouble tz = v1->X() * v2->Y() - v1->Y() * v2->X();

    result->Set(tx, ty, tz);

    return GF_S_OK;
}

gfVector2* gfCombineVectors2(gfDouble c1, gfVector2* v1, gfDouble c2, gfVector2* v2)
{
    gfDouble tx = c1 * v1->X() + c2 * v2->X();
    gfDouble ty = c1 * v1->Y() + c2 * v2->Y();

    gfVector2* result = gfCreateVector2();
    result->Set(tx, ty);

    return result;
}

GFRESULT gfCombineVectors2(gfDouble c1, gfVector2* v1, gfDouble c2, gfVector2* v2, gfVector2* result)
{
    gfDouble tx = c1 * v1->X() + c2 * v2->X();
    gfDouble ty = c1 * v1->Y() + c2 * v2->Y();

    result->Set(tx, ty);

    return GF_S_OK;
}

gfVector3* gfCombineVectors3(gfDouble c1, gfVector3* v1, gfDouble c2, gfVector3* v2)
{
    gfDouble tx = c1 * v1->X() + c2 * v2->X();
    gfDouble ty = c1 * v1->Y() + c2 * v2->Y();
    gfDouble tz = c1 * v1->Z() + c2 * v2->Z();

    gfVector3* result = gfCreateVector3();
    result->Set(tx, ty, tz);

    return result;
}

GFRESULT gfCombineVectors3(gfDouble c1, gfVector3* v1, gfDouble c2, gfVector3* v2, gfVector3* result)
{
    gfDouble tx = c1 * v1->X() + c2 * v2->X();
    gfDouble ty = c1 * v1->Y() + c2 * v2->Y();
    gfDouble tz = c1 * v1->Z() + c2 * v2->Z();

    result->Set(tx, ty, tz);

    return GF_S_OK;
}

gfVector4* gfCombineVectors4(gfDouble c1, gfVector4* v1, gfDouble c2, gfVector4* v2)
{
    gfDouble tx = c1 * v1->X() + c2 * v2->X();
    gfDouble ty = c1 * v1->Y() + c2 * v2->Y();
    gfDouble tz = c1 * v1->Z() + c2 * v2->Z();
    gfDouble tw = c1 * v1->W() + c2 * v2->W();

    gfVector4* result = gfCreateVector4();
    result->Set(tx, ty, tz, tw);

    return result;
}

GFRESULT gfCombineVectors4(gfDouble c1, gfVector4* v1, gfDouble c2, gfVector4* v2, gfVector4* result)
{
    gfDouble tx = c1 * v1->X() + c2 * v2->X();
    gfDouble ty = c1 * v1->Y() + c2 * v2->Y();
    gfDouble tz = c1 * v1->Z() + c2 * v2->Z();
    gfDouble tw = c1 * v1->W() + c2 * v2->W();

    result->Set(tx, ty, tz, tw);

    return GF_S_OK;
}

gfPoint2* gfCombinePointVector2(gfPoint2* point, gfDouble c, gfVector2* vector)
{
    gfDouble tx = point->X() + c * vector->X();
    gfDouble ty = point->Y() + c * vector->Y();

    gfPoint2* result = gfCreatePoint2();
    result->Set(tx, ty);

    return result;
}

GFRESULT gfCombinePointVector2(gfPoint2* point, gfDouble c, gfVector2* vector, gfPoint2* result)
{
    gfDouble tx = point->X() + c * vector->X();
    gfDouble ty = point->Y() + c * vector->Y();

    result->Set(tx, ty);

    return GF_S_OK;
}

gfPoint3* gfCombinePointVector3(gfPoint3* point, gfDouble c, gfVector3* vector)
{
    gfDouble tx = point->X() + c * vector->X();
    gfDouble ty = point->Y() + c * vector->Y();
    gfDouble tz = point->Z() + c * vector->Z();

    gfPoint3* result = gfCreatePoint3();
    result->Set(tx, ty, tz);

    return result;
}

GFRESULT gfCombinePointVector3(gfPoint3* point, gfDouble c, gfVector3* vector, gfPoint3* result)
{
    gfDouble tx = point->X() + c * vector->X();
    gfDouble ty = point->Y() + c * vector->Y();
    gfDouble tz = point->Z() + c * vector->Z();

    result->Set(tx, ty, tz);

    return GF_S_OK;
}

gfPoint4* gfCombinePointVector4(gfPoint4* point, gfDouble c, gfVector4* vector)
{
    gfDouble tx = point->X() + c * vector->X();
    gfDouble ty = point->Y() + c * vector->Y();
    gfDouble tz = point->Z() + c * vector->Z();
    gfDouble tw = point->W() + c * vector->W();

    gfPoint4* result = gfCreatePoint4();
    result->Set(tx, ty, tz, tw);

    return result;
}

GFRESULT gfCombinePointVector4(gfPoint4* point, gfDouble c, gfVector4* vector, gfPoint4* result)
{
    gfDouble tx = point->X() + c * vector->X();
    gfDouble ty = point->Y() + c * vector->Y();
    gfDouble tz = point->Z() + c * vector->Z();
    gfDouble tw = point->W() + c * vector->W();

    result->Set(tx, ty, tz, tw);

    return GF_S_OK;
}

gfBool gfArePointsEqual2(gfPoint2* pt1, gfPoint2* pt2)
{
    if (gfIsZero(pt1->Distance(pt2)))
        return true;
    else
        return false;
}

gfBool gfArePointsEqual3(gfPoint3* pt1, gfPoint3* pt2)
{
    if (gfIsZero(pt1->Distance(pt2)))
        return true;
    else
        return false;
}

gfBool gfArePointsEqual4(gfPoint4* pt1, gfPoint4* pt2)
{
    if (gfIsZero(pt1->Distance(pt2)))
        return true;
    else
        return false;
}

GFRESULT gfCompBaryCoords3(gfPoint3* point, gfPoint3* pt1, gfPoint3* pt2, gfPoint3* pt3, gfDouble* u, gfDouble* v, gfDouble* w)
{
    // check for planarity
    gfVector3* base1 = gfSubstractPoints3(pt2, pt1);
    gfVector3* base2 = gfSubstractPoints3(pt3, pt1);

    gfVectorProduct3(base1, base2, base1);
    gfDouble length = base1->Length();
    gfDeleteVector3(base1);
    gfDeleteVector3(base2);

    if (gfIsZero(length))
        return GF_E_FAIL;

    // compute coordinates
    gfDouble det = pt1->X() * pt2->Y() * pt3->Z() +
                   pt1->Y() * pt2->Z() * pt3->X() +
                   pt1->Z() * pt2->X() * pt3->Y() -
                   pt1->Z() * pt2->Y() * pt3->X() -
                   pt1->X() * pt2->Z() * pt3->Y() -
                   pt1->Y() * pt2->X() * pt3->Z();
    if (gfIsZero(det))
        return GF_E_FAIL;

    *u = (point->X() * pt2->Y() * pt3->Z() +
          point->Y() * pt2->Z() * pt3->X() +
          point->Z() * pt2->X() * pt3->Y() -
          point->Z() * pt2->Y() * pt3->X() -
          point->X() * pt2->Z() * pt3->Y() -
          point->Y() * pt2->X() * pt3->Z()) / det;

    *v = (pt1->X() * point->Y() * pt3->Z() +
          pt1->Y() * point->Z() * pt3->X() +
          pt1->Z() * point->X() * pt3->Y() -
          pt1->Z() * point->Y() * pt3->X() -
          pt1->X() * point->Z() * pt3->Y() -
          pt1->Y() * point->X() * pt3->Z()) / det;

    *w = (pt1->X() * pt2->Y() * point->Z() +
          pt1->Y() * pt2->Z() * point->X() +
          pt1->Z() * pt2->X() * point->Y() -
          pt1->Z() * pt2->Y() * point->X() -
          pt1->X() * pt2->Z() * point->Y() -
          pt1->Y() * pt2->X() * point->Z()) / det;

    if (gfIsZero(*u + *v + *w - 1.0))
        return GF_S_OK;
    else
        return GF_E_FAIL;
}

GFRESULT gfRotatePoint3(gfPoint3* origin, gfVector3* axis, gfDouble angle, gfPoint3* point)
{
    // find projection of point to line (origin, axis)
    gfDouble t = -((origin->X()-point->X())*axis->X()+
                  (origin->Y()-point->Y())*axis->Y()+
                  (origin->Z()-point->Z())*axis->Z())/
                  (axis->X()*axis->X()+axis->Y()*axis->Y()+axis->Z()*axis->Z());

    gfPoint3* proj = gfCombinePointVector3(origin, t, axis);
    gfVector3* base1 = gfSubstractPoints3(point, proj);
    gfVector3* base2 = gfVectorProduct3(base1, axis);
    gfDouble length = base1->Length();
    //base1->Normalize();
    base2->SetLength(length);

    gfVector3* result = gfCombineVectors3(gfCos(angle), base1, gfSin(angle), base2);
    //result->SetLength(length);
    gfCombinePointVector3(proj, 1.0, result, point);

    gfDeletePoint3(proj);
    gfDeleteVector3(base1);
    gfDeleteVector3(base2);
    gfDeleteVector3(result);

    return GF_S_OK;
}

GFRESULT gfCompGenBaryCoordsWachspress2(std::vector<gfPoint2> points, gfPoint2 point, std::vector<gfDouble>& coordinates)
{
    std::vector<gfDouble> areas;

    // compute signed area of triangles
    gfInt num_points = (gfInt)points.size();
    for (gfInt i = 0;i < num_points;i++)
    {
        gfPoint2 pt1 = points[i];
        gfPoint2 pt2 = points[(i+1) % num_points];

        std::vector<gfPoint2> triangle;
        triangle.push_back(point);
        triangle.push_back(pt1);
        triangle.push_back(pt2);

        gfDouble area = gfPolygonArea2(triangle);
        areas.push_back((area));
    }

    // get product of areas
    coordinates.clear();
    gfDouble sum = 0.0;
    for (gfInt i = 0;i < num_points;i++)
    {
        std::vector<gfPoint2> triangle;
        triangle.push_back(points[(i-1+num_points) % num_points]);
        triangle.push_back(points[i]);
        triangle.push_back(points[(i+1) % num_points]);

        gfDouble weight = gfPolygonArea2(triangle);
        for (gfInt j = 0;j < num_points;j++)
        {
            if (j == i)
                continue;
            if (j == (i-1+num_points) % num_points)
                continue;
            weight *= areas[j];
        }

        sum += weight;
        coordinates.push_back(weight);
    }

    // normalize
    gfDouble inv = 1.0 / sum;
    for (gfInt i = 0;i < num_points;i++)
    {
        gfDouble coord = inv * coordinates[i];
        coordinates[i] = coord;
    }

    return GF_S_OK;
}

GFRESULT gfCompGenBaryCoordsMean2(std::vector<gfPoint2> points, gfPoint2 point, std::vector<gfDouble>& coordinates)
{
    // weights by local properties
    gfInt num_points = (gfInt)points.size();
    coordinates.clear();
    gfDouble sum = 0.0;
    for (gfInt i = 0;i < num_points;i++)
    {
        gfPoint2 pt1 = points[(i-1+num_points) % num_points];
        gfPoint2 pt2 = points[i];
        gfPoint2 pt3 = points[(i+1) % num_points];

        gfVector2 v1, v2, v3;
        gfSubstractPoints2(&point, &pt1, &v1);
        gfSubstractPoints2(&point, &pt2, &v2);
        gfSubstractPoints2(&point, &pt3, &v3);

        gfDouble dist = v2.Length();
        gfDouble alfa1 = gfVectorsAngle2(&v1, &v2);
        gfDouble alfa2 = gfVectorsAngle2(&v2, &v3);

        if (dist < GF_EPS)
        {
            coordinates.clear();
            for (gfInt j = 0;j < num_points;j++)
                coordinates.push_back(0.0);
            coordinates[i] = 1.0;
            sum = 1.0;
            break;
        }
        else if (gfAbs(alfa1 - GF_PI) < GF_EPS)
        {
            coordinates.clear();
            for (gfInt j = 0;j < num_points;j++)
                coordinates.push_back(0.0);
            gfDouble d1 = points[(i-1+num_points) % num_points].Distance(&points[i]);
            gfDouble d2 = point.Distance(&points[i]);
            gfDouble coord = d2 / d1;
            coordinates[(i-1+num_points) % num_points] = coord;
            coordinates[i] = 1.0 - coord;
            sum = 1.0;
            break;
        }
        else if (gfAbs(alfa2 - GF_PI) < GF_EPS)
        {
            coordinates.clear();
            for (gfInt j = 0;j < num_points;j++)
                coordinates.push_back(0.0);
            gfDouble d1 = points[i].Distance(&points[(i+1) % num_points]);
            gfDouble d2 = point.Distance(&points[i]);
            gfDouble coord = d2 / d1;
            coordinates[(i+1) % num_points] = coord;
            coordinates[i] = 1.0 - coord;
            sum = 1.0;
            break;
        }

        gfDouble weight = tan(0.5 * alfa1) + tan(0.5 * alfa2);
        weight = weight / dist;
        sum += weight;
        coordinates.push_back(weight);
    }

    // normalize
    gfDouble inv = 1.0 / sum;
    for (gfInt i = 0;i < num_points;i++)
    {
        gfDouble coord = inv * coordinates[i];
        coordinates[i] = coord;
    }

    return GF_S_OK;
}

GFRESULT gfCompGenBaryCoordsMean3(std::vector<gfPoint3> points, std::vector<gfTripleIndex> triangles, gfPoint3 point, std::vector<gfDouble>& coordinates)
{
    // compute normals for triangles
    gfInt num_points = (gfInt)points.size();
    gfInt num_triangles = (gfInt) triangles.size();
    std::vector<gfVector3> normals;
    for (gfInt i = 0;i < num_triangles;i++)
    {
        gfTripleIndex tri = triangles[i];
        gfPoint3 pt1 = points[tri.i];
        gfPoint3 pt2 = points[tri.j]; 
        gfPoint3 pt3 = points[tri.k]; 
        gfVector3 v1, v2, normal;
        gfSubstractPoints3(&pt2, &pt1, &v1);
        gfSubstractPoints3(&pt3, &pt1, &v2);
        gfVectorProduct3(&v1, &v2, &normal);
        normal.Normalize();
        normals.push_back(normal);

        // check if given point is inside triangle
        gfVector3 v3;
        gfSubstractPoints3(&pt1, &point, &v3);
        if (gfAbs(gfScalarProduct3(&v3, &normal)) < GF_EPS)
        {
            // barycentric coordinates in triangle
            std::vector<gfPoint3> triangle;

            triangle.push_back(pt1);
            triangle.push_back(pt2);
            triangle.push_back(pt3);
            gfDouble area = gfPolygonArea3(triangle);

            if (gfAbs(area) < GF_EPS)
                continue;

            triangle.clear();
            triangle.push_back(point);
            triangle.push_back(pt2);
            triangle.push_back(pt3);
            gfDouble u = gfPolygonArea3(triangle);

            triangle.clear();
            triangle.push_back(point);
            triangle.push_back(pt1);
            triangle.push_back(pt3);
            gfDouble v = gfPolygonArea3(triangle);

            triangle.clear();
            triangle.push_back(point);
            triangle.push_back(pt2);
            triangle.push_back(pt1);
            gfDouble w = gfPolygonArea3(triangle);

            if (gfAbs(u + v+ w - area) > GF_EPS)
                continue;

            gfDouble inv = 1.0 / (u + v + w);
            u *= inv;
            v *= inv;
            w *= inv;
            if (u >= 0.0 && u <= 1.0 &&
                v >= 0.0 && v <= 1.0 &&
                w >= 0.0 && w <= 1.0)
            {
                coordinates.clear();
                for (gfInt j = 0;j < num_points;j++)
                    coordinates.push_back(0.0);
                coordinates[tri.i] = u;
                coordinates[tri.j] = v;
                coordinates[tri.k] = w;

                return GF_S_OK;
            }
        }
    }

    // compute incident triangles for each vertex
    std::vector< std::vector<gfInt> > neighs;
    neighs.resize(points.size());
    for (gfInt i = 0;i < num_triangles;i++)
    {
        gfTripleIndex tri = triangles[i];
        neighs[tri.i].push_back(i);
        neighs[tri.j].push_back(i);
        neighs[tri.k].push_back(i);
    }

    // now compute weights for each vertex
    gfDouble sum = 0.0;
    for (gfInt i = 0;i < num_points;i++)
    {
        gfVector3 dir;
        gfPoint3 pt = points[i];
        gfSubstractPoints3(&pt, &point, &dir);

        // check for boundary
        /*if (dir.Length() < GF_EPS)
        {
            coordinates.clear();
            for (gfInt j = 0;j < num_points;j++)
                coordinates.push_back(0.0);
            coordinates[i] = 1.0;
            sum = 1.0;
            break;
        }*/

        // compute points of cone
        std::vector<gfVector3> cone_points;
        gfDouble weight = 0;
        for (gfInt j = 1;j < (gfInt)neighs[i].size()-1;j++)
        {
            gfVector3 n1 = normals[neighs[i][0]];
            gfVector3 n2 = normals[neighs[i][j]];
            gfVector3 n3 = normals[neighs[i][j+1]];
            gfDouble prod1 = gfScalarProduct3(&n1, &dir);
            gfDouble prod2 = gfScalarProduct3(&n2, &dir);
            gfDouble prod3 = gfScalarProduct3(&n3, &dir);
            gfDouble prod = 6 * prod1 * prod2 * prod3;

            gfMatrix3 matrix;
            matrix.Set(n1.X(), n1.Y(), n1.Z(), n2.X(), n2.Y(), n2.Z(), n3.X(), n3.Y(), n3.Z());
            weight += gfAbs(matrix.Determinant() / prod);
        }

        weight *= 3;
        sum += weight;
        coordinates.push_back(weight);
    }

    // normalize
    gfDouble inv = 1.0 / sum;
    for (gfInt i = 0;i < num_points;i++)
    {
        gfDouble coord = inv * coordinates[i];
        coordinates[i] = coord;
    }

    return GF_S_OK;
}

gfDouble gfPolygonArea2(std::vector<gfPoint2> points)
{
    gfDouble result = 0.0;
    gfInt num_points = (gfInt)points.size();

    for (gfInt i = 0;i < num_points;i++)
    {
        gfInt index1 = i % num_points;
        gfInt index2 = (i + 1) % num_points;
        result += points[index1].X()*points[index2].Y() - points[index2].X()*points[index1].Y();
    }

    return 0.5 * result;
}

gfDouble gfPolygonArea3(std::vector<gfPoint3> points)
{
    gfDouble result = 0.0;
    gfInt num_points = (gfInt)points.size();
    for (gfInt i = 1;i < num_points-1;i++)
    {
        gfPoint3 pt1 = points[0];
        gfPoint3 pt2 = points[i];
        gfPoint3 pt3 = points[(i + 1)];
        
        gfVector3 v1, v2;
        gfSubstractPoints3(&pt2, &pt1, &v1);
        gfSubstractPoints3(&pt3, &pt1, &v2);
        gfVector3 normal;
        gfVectorProduct3(&v1, &v2, &normal);
        result += normal.Length();
    }

    return gfAbs(0.5*result);
}
