//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAuxGeomUtils.h                                     //
//   Description: Geometric functions                           //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_GEOM_UTILS_H_
#define _GF_GEOM_UTILS_H_


#include "gfDefines.h"
#include "gfVector.h"
#include "gfPoint.h"
#include "gfMatrix.h"
#include <vector>
#include <math.h>


GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix2* matrix);
GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix2* matrix, gfPoint2* result);
GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix3* matrix);
GFRESULT gfTransformPoint2(gfPoint2* point, gfMatrix3* matrix, gfPoint2* result);
GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix3* matrix);
GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix3* matrix, gfPoint2* result);
GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix4* matrix);
GFRESULT gfTransformPoint3(gfPoint3* point, gfMatrix4* matrix, gfPoint2* result);
GFRESULT gfTransformPoint4(gfPoint4* point, gfMatrix4* matrix);
GFRESULT gfTransformPoint4(gfPoint4* point, gfMatrix4* matrix, gfPoint2* result);

GFRESULT gfTransformVector2(gfVector2* vector, gfMatrix2* matrix);
GFRESULT gfTransformVector2(gfVector2* vector, gfMatrix2* matrix, gfPoint2* result);
GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix3* matrix);
GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix3* matrix, gfPoint2* result);
GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix4* matrix);
GFRESULT gfTransformVector3(gfVector3* vector, gfMatrix4* matrix, gfPoint2* result);
GFRESULT gfTransformVector4(gfVector4* vector, gfMatrix4* matrix);
GFRESULT gfTransformVector4(gfVector4* vector, gfMatrix4* matrix, gfPoint2* result);

gfVector2* gfSubstractPoints2(gfPoint2* pt1, gfPoint2* pt2);
GFRESULT gfSubstractPoints2(gfPoint2* pt1, gfPoint2* pt2, gfVector2* result);
gfVector3* gfSubstractPoints3(gfPoint3* pt1, gfPoint3* pt2);
GFRESULT gfSubstractPoints3(gfPoint3* pt1, gfPoint3* pt2, gfVector3* result);
gfVector4* gfSubstractPoints4(gfPoint4* pt1, gfPoint4* pt2);
GFRESULT gfSubstractPoints4(gfPoint4* pt1, gfPoint4* pt2, gfVector4* result);

gfDouble gfScalarProduct2(gfVector2* v1, gfVector2* v2);
gfDouble gfScalarProduct3(gfVector3* v1, gfVector3* v2);
gfDouble gfScalarProduct4(gfVector4* v1, gfVector4* v2);

gfDouble gfVectorsAngle2(gfVector2* v1, gfVector2* v2);
gfDouble gfVectorsAngle3(gfVector3* v1, gfVector3* v2);
gfDouble gfVectorsAngle4(gfVector4* v1, gfVector4* v2);

gfVector3* gfVectorProduct3(gfVector3* v1, gfVector3* v2);
GFRESULT gfVectorProduct3(gfVector3* v1, gfVector3* v2, gfVector3* result);

gfVector2* gfCombineVectors2(gfDouble c1, gfVector2* v1, gfDouble c2, gfVector2* v2);
GFRESULT gfCombineVectors2(gfDouble c1, gfVector2* v1, gfDouble c2, gfVector2* v2, gfVector2* result);
gfVector3* gfCombineVectors3(gfDouble c1, gfVector3* v1, gfDouble c2, gfVector3* v2);
GFRESULT gfCombineVectors3(gfDouble c1, gfVector3* v1, gfDouble c2, gfVector3* v2, gfVector3* result);
gfVector4* gfCombineVectors4(gfDouble c1, gfVector4* v1, gfDouble c2, gfVector4* v2);
GFRESULT gfCombineVectors4(gfDouble c1, gfVector4* v1, gfDouble c2, gfVector4* v2, gfVector4* result);

gfPoint2* gfCombinePointVector2(gfPoint2* point, gfDouble c, gfVector2* vector);
GFRESULT gfCombinePointVector2(gfPoint2* point, gfDouble c, gfVector2* vector, gfPoint2* result);
gfPoint3* gfCombinePointVector3(gfPoint3* point, gfDouble c, gfVector3* vector);
GFRESULT gfCombinePointVector3(gfPoint3* point, gfDouble c, gfVector3* vector, gfPoint3* result);
gfPoint4* gfCombinePointVector4(gfPoint4* point, gfDouble c, gfVector4* vector);
GFRESULT gfCombinePointVector4(gfPoint4* point, gfDouble c, gfVector4* vector, gfPoint4* result);

GFRESULT gfRotatePoint3(gfPoint3* origin, gfVector3* axis, gfDouble angle, gfPoint3* point);

gfBool gfArePointsEqual2(gfPoint2* pt1, gfPoint2* pt2);
gfBool gfArePointsEqual3(gfPoint3* pt1, gfPoint3* pt2);
gfBool gfArePointsEqual4(gfPoint4* pt1, gfPoint4* pt2);

// compute barycentric coordinates
GFRESULT gfCompGenBaryCoordsWachspress2(std::vector<gfPoint2> points, gfPoint2 point, std::vector<gfDouble>& coordinates);
GFRESULT gfCompGenBaryCoordsMean2(std::vector<gfPoint2> points, gfPoint2 poi110nt, std::vector<gfDouble>& coordinates);
GFRESULT gfCompGenBaryCoordsMean3(std::vector<gfPoint3> points, std::vector<gfTripleIndex> triangles, gfPoint3 point, std::vector<gfDouble>& coordinates);

gfDouble gfPolygonArea2(std::vector<gfPoint2> points);
gfDouble gfPolygonArea3(std::vector<gfPoint3> points);

#endif
