//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAuxList.h                                          //
//   Description: Template for connected list                   //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_AUX_LIST_H_
#define _GF_AUX_LIST_H_


#include "gfDefines.h"
#include <malloc.h>
#define NULL 0


/*template<class T>
class gfAuxList
{
    struct ListNode
    {
        ListNode* next;
        T data;
    };

private:
    gfInt m_current;
    gfInt m_size;
    gfInt m_alloc_size;
    T* m_data;

public:
    gfAuxList(void)
    {
        m_current = 0;
        m_size = 0;
        m_alloc_size = 8;
        m_data = (T*) malloc(sizeof(T)*m_alloc_size);
    }

    ~gfAuxList(void)
    {
        if (m_data)
            free(m_data);
    }

    void Clear(void)
    {
        m_size = 0;
        m_alloc_size = 8;
        m_current = 0;
        m_data = (T*) realloc(m_data, sizeof(T)*m_alloc_size);
        return;
    }

    T GetFirst(void)
    {
        if (!m_size)
            return NULL;
        m_current = 0;
        return m_data[0];
    }

    T GetNext(void)
    {
        m_current++;
        if (m_current < m_size)
            return m_data[m_current];
        else
            return NULL;                
    }

    gfInt size(void)
    {
        return m_size;
    }

    gfBool Empty(void)
    {
        return (m_size == 0);
    }

    void PushBack(T elem)
    {
        if (m_size == m_alloc_size)
        {
            m_alloc_size += 8;
            m_data = (T*) realloc(m_data, sizeof(T)*m_alloc_size);
        }

        m_data[m_size] = elem;

        m_size++;
    }

    void Remove(T elem)
    {
        if (!m_size)
            return;

        for (gfInt i = 0;i < m_size;i++)
        {
            if (m_data[i] == elem)
            {
                for (gfInt j = i+1;j < m_size;j++)
                {
                    m_data[j-1] = m_data[j];
                }
                break;
            }
        }
        m_size--;

        if (m_size < m_alloc_size-64 && m_alloc_size >= 16)
        {
            m_alloc_size = m_alloc_size - 8;
            m_data = (T*) realloc(m_data, sizeof(T)*m_alloc_size);
        }

        return;
    }

    gfInt GetAllocMemorySize(void)
    {
        gfInt alloc = sizeof(T)*m_alloc_size;
        return alloc;
    }
};*/

template<class T>
class gfAuxList
{
    struct ListNode
    {
        ListNode* next;
        T data;
    };

private:
    ListNode* m_begin;
    ListNode* m_current;
    ListNode* m_end;
    gfInt m_size;

public:
    gfAuxList(void)
    {
        m_begin = NULL;
        m_current = NULL;
        m_end = NULL;
        m_size = 0;
    }

    ~gfAuxList(void)
    {
        Clear();
    }

    void Clear(void)
    {
        m_size = 0;
        m_current = NULL;
        if (!m_begin)
            return;

        ListNode* aux = m_begin;
        while (aux)
        {
            ListNode* next = aux->next;
            delete aux;
            aux = next;
        }

        m_begin = NULL;
        m_end = NULL;
        return;
    }

    T GetFirst(void)
    {
        if (!m_begin)
            return NULL;
        m_current = m_begin;
        return m_begin->data;
    }

    T GetNext(void)
    {
        if (m_current && m_current->next)
        {
            m_current = m_current->next;
            return m_current->data;
        }
        else
            return NULL;                
    }

    gfInt size(void)
    {
        return m_size;
    }

    gfBool Empty(void)
    {
        return (m_size == 0);
    }

    void PushBack(T elem)
    {
        ListNode* new_node = new ListNode;
        new_node->data = elem;
        new_node->next = NULL;
        if (m_size)
        {
            m_end->next = new_node;
            m_current = new_node;
            m_end = new_node;
        }
        else
        {
            m_begin = m_end = m_current = new_node;
        }

        m_size++;
    }

    void Remove(T elem)
    {
        if (!m_begin)
            return;

        if (m_begin->data == elem)
        {
            m_size--;
            ListNode* aux = m_begin;
            if (m_current == m_begin)
                m_current = m_begin->next;
            m_begin = m_begin->next;
            delete aux;
            return;
        }

        ListNode* prev = m_begin;
        ListNode* curr = m_begin->next;

        while (curr)
        {
            if (curr->data == elem)
            {
                prev->next = curr->next;
                if (curr == m_end)
                    m_end = prev;
                if (curr == m_current)
                    m_current = prev;
                delete curr;
                m_size--;
                return;
            }

            prev = curr;
            curr = curr->next;
        }

        return;
    }

    gfInt GetAllocMemorySize(void)
    {
        gfInt alloc = (sizeof(this) + m_size * (sizeof(ListNode*) + sizeof(T)));
        return alloc;
    }
};


#endif