//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAuxLogging.cpp                                     //
//   Description: Logging engine for displaying status & errors //
//                                                              //
//////////////////////////////////////////////////////////////////

#include "gfAuxLogging.h"
#include <stdarg.h>
#include <stdio.h>

void gfSetLogType(gfInt log_type)
{
    m_log_type = log_type;
}

void gfSetLogFile(char* log_file)
{
    m_log_file = log_file;
    FILE* file = fopen(m_log_file, "w");
    fclose(file);
}

void gfEnableLogging(void)
{
    m_logging_enabled = true;
}

void gfDisableLogging(void)
{
    m_logging_enabled = false;
}

void gfLog(const char *format, ...)
{
    if (!m_logging_enabled)
        return;

    va_list marker;
    va_start(marker, format);   

    if (m_log_type == GF_LOG_CONSOLE)
    {
        vprintf(format, marker);
    }
    else if (m_log_type == GF_LOG_FILE && m_log_file)
    {
        FILE* file;
        file = fopen(m_log_file, "a+");
        vfprintf(file, format, marker);
        fclose(file);
    }

    va_end(marker);

    return;
}
