//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfNumUtils.cpp                                       //
//   Description: Numerical functions                           //
//                                                              //
//////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "gfAuxNumUtils.h"
#include "gfObjectFactory.h"
#include "gfAuxLogging.h"
#include <math.h>


gfBool gfIsZero(gfDouble numb)
{
    if (numb*numb < GF_EPS*GF_EPS )
        return 1;
    else
        return 0;
}

gfDouble gfSquareRoot(gfDouble numb)
{
    return sqrt(numb);
}

gfDouble gfSin(gfDouble numb)
{
    return sin(numb);
}

gfDouble gfArcsin(gfDouble numb)
{
    return asin(numb);
}

gfDouble gfCos(gfDouble numb)
{
    return cos(numb);
}

gfDouble gfArccos(gfDouble numb)
{
    return acos(numb);
}

gfLong gfFactorial(gfLong lNumb)
{
    gfLong res = 1;

    for (gfLong i = 1;i <= lNumb;i++)
    {
        res *= i;
    }

    return res;
}

gfDouble gfPower(gfDouble dNumb, gfInt iPow)
{
    gfDouble res = 1.0;

    for (gfLong i = 0;i < iPow;i++)
    {
        res *= dNumb;
    }

    return res;
}

gfDouble gfPower(gfDouble dNumb, gfDouble dPow)
{
    return pow(dNumb, dPow);
}

gfLong gfCombNumber(gfLong n, gfLong i)
{
    return gfFactorial(n) / gfFactorial(i) / gfFactorial(n - i);
}

gfLong gfCombNumber3(gfLong n, gfLong i, gfLong j)
{
    return gfFactorial(n) / gfFactorial(i) / gfFactorial(j) / gfFactorial(n - i - j);
}

gfLong gfCombNumber4(gfLong n, gfLong i, gfLong j, gfLong k)
{
    return gfFactorial(n) / gfFactorial(i) / gfFactorial(j) / gfFactorial(k) / gfFactorial(n - i - j - k);
}

gfLong gfCombNumberGeneral(std::vector<gfInt> multi_index)
{
    gfInt sum = 0.0;
    for (gfInt i = 0;i < (gfInt)multi_index.size();i++)
        sum += multi_index[i];

    gfLong result = 1;
    for (gfInt i = 0;i < (gfInt)multi_index.size();i++)
        result *= gfFactorial(multi_index[i]);

    return gfFactorial(sum) / result;
}

gfDouble gfBernsteinPoly(gfLong n, gfLong i, gfDouble u)
{
    return gfCombNumber(n, i) * gfPower(u, i) * gfPower(1.0 - u, n - i);
}

gfDouble gfBernsteinPoly3(gfLong n, gfLong i, gfLong j, gfDouble u, gfDouble v)
{
    return gfCombNumber3(n, i, j) * gfPower(u, i) * gfPower(v, j) * gfPower(1.0 - u - v, n - i - j);
}

gfDouble gfBernsteinPoly4(gfLong n, gfLong i, gfLong j, gfLong k, gfDouble u, gfDouble v, gfDouble w)
{
    return gfCombNumber4(n, i, j, k) * gfPower(u, i) * gfPower(v, j) * gfPower(w, k) * gfPower(1.0 - u - v - w, n - i - j - k);
}

gfDouble gfBernsteinPolyGeneral(std::vector<gfInt> multi_index, std::vector<gfDouble> coordinates)
{
    gfDouble result = 1.0;
    for (gfInt i = 0;i < (gfInt)multi_index.size();i++)
    {
        gfDouble d = coordinates[i];
        result *= gfPower(coordinates[i], multi_index[i]);
    }

    return gfCombNumberGeneral(multi_index)*result;
}

gfInt gfRand()
{
    return rand();
}

gfInt gfRand(gfInt max)
{
    return (gfInt) max * (rand() / (gfDouble) RAND_MAX);
}

gfDouble gfRand(gfDouble max)
{
    return max * (rand() / (gfDouble) RAND_MAX);
}

gfDouble gfRand(gfDouble min, gfDouble max)
{
    return (min + (max - min) * (rand() / (gfDouble) RAND_MAX));
}

gfDouble gfAbs(gfDouble val)
{
    if (val < 0)
        return -val;
    else
        return val;
}

gfDouble gfClamp(gfDouble min, gfDouble max, gfDouble value)
{
    if (min > max)
        return value;
    if (value < min)
        return min;
    if (value > max)
        return max;
    return value;
}

gfInt gfClamp(gfInt min, gfInt max, gfInt value)
{
    if (min > max)
        return value;
    if (value < min)
        return min;
    if (value > max)
        return max;
    return value;
}

GFRESULT gfSort1DArray(std::vector<gfDouble>* array)
{
    gfBool done  = false;
    gfInt curr = (gfInt)array->size() - 1;
    while (!done)
    {
        done = true;
        for (gfInt i = 0;i < curr;i++)
        {
            if (array->at(i) > array->at(i+1))
            {
                done = false;
                gfDouble aux = array->at(i);
                array->at(i) = array->at(i+1);
                array->at(i+1) = aux;
            }
        }
        curr--;
    }

    return GF_S_OK;
}

GFRESULT gfSortRayInterInfo(gfRayInterInfo* info)
{
    gfBool done  = false;
    gfInt curr = (gfInt)info->m_inters.size() - 1;
    while (!done)
    {
        done = true;
        for (gfInt i = 0;i < curr;i++)
        {
            if (info->m_inters.at(i) > info->m_inters.at(i+1))
            {
                done = false;
                gfDouble aux = info->m_inters.at(i);
                info->m_inters[i] = info->m_inters.at(i+1);
                info->m_inters[i+1] = aux;

                gfVector3 aux_normal = info->m_normals.at(i);
                info->m_normals[i] = info->m_normals.at(i+1);
                info->m_normals[i+1] = aux_normal;
            }
        }
        curr--;
    }

    return GF_S_OK;
}

GFRESULT gfGetEigensSym3x3(gfMatrix3* matrix, gfVector3* eigen_values, gfMatrix3* eigen_vectors, gfInt* nrot)
{
    // code from http://www.koders.com/c++/fidD97BA57A921DEDBF091B4CA38E54E3A394503C8D.aspx

#define ROTATE(a, i, j, k, l)   g = a->at(i, j);h = a->at(k, l); \
                                a->Set(i, j, g - s * (h + g * tau));a->Set(k, l, h + s * (g - h * tau));

	gfInt j, iq, ip, i;
	gfDouble tresh, theta, tau, t, sm, s, h, g, c;
    gfVector3* b,* z;

    b = gfCreateVector3();
    z = gfCreateVector3();
						
    //Initialize to the identity matrix.
    eigen_vectors->LoadIdentity();

    // initialize b and eigen_values to diagonal of matrix
    eigen_values->Set(matrix->at(1, 1),  matrix->at(2, 2), matrix->at(3, 3));
    b->Set(matrix->at(1, 1),  matrix->at(2, 2), matrix->at(3, 3));
    z->Set(0.0, 0.0, 0.0);

	*nrot = 0;
	for (i = 1;i <= 50;i++) 
    {
		sm = 0.0;
		for (ip = 1;ip <= 3-1;ip++) 
        {						
            //Sum of diagonal elements.
			for (iq = ip + 1;iq <= 3;iq++)
                sm += fabs(matrix->at(ip, iq));
		}
		if (sm == 0.0) 
        {	
            //The normal return, which relies on quadratic convergence to machine under ow.
			gfDeleteVector3(z);
			gfDeleteVector3(b);

			return GF_S_OK;
		}
		if (i < 4)
			tresh = 0.2 * sm / (3 * 3);								// ...on the first three sweeps.
		else
			tresh = 0.0;										//...thereafter.

		for (ip = 1;ip <= 3 - 1;ip++) 
        {
			for (iq = ip + 1;iq <= 3;iq++) 
            {
                g = 100.0 * fabs(matrix->at(ip, iq));

				//After four sweeps, skip the rotation if the o-diagonal element is small.

                if (i > 4 && (gfDouble)(fabs(eigen_values->at(ip))+ g) == (gfDouble)fabs(eigen_values->at(ip))
					&& (gfDouble)(fabs(eigen_values->at(iq)) + g) == (gfDouble)fabs(eigen_values->at(iq)))
                    matrix->Set(ip, iq, 0.0);
                else if (fabs(matrix->at(ip, iq)) > tresh) 
                {
					h = eigen_values->at(iq) - eigen_values->at(ip);
					if ((gfDouble)(fabs(h) + g) == (gfDouble)fabs(h))
                    {
						t = (matrix->at(ip, iq)) / h;				//t = 1=(2.)
                    }
					else 
                    {
						theta = 0.5 * h / (matrix->at(ip, iq));		//Equation (11.1.10).
						t = 1.0 / (fabs(theta) + gfSquareRoot(1.0 + theta * theta));
						if (theta < 0.0) 
                            t = -t;
					}

					c = 1.0 / gfSquareRoot(1 + t * t);
					s = t * c;
					tau = s / (1.0 + c);
					h = t * matrix->at(ip, iq);
                    z->Set(ip, z->at(ip) - h);
					z->Set(iq, z->at(iq) + h);
                    eigen_values->Set(ip, eigen_values->at(ip) - h);
					eigen_values->Set(iq, eigen_values->at(iq) + h);
                    matrix->Set(ip, iq, 0.0);

					for (j = 1;j <= ip - 1;j++) 
                    {					
                        //Case of rotations 1 . j < p.
						ROTATE(matrix, j, ip, j, iq);
					}
					for (j = ip + 1;j <= iq - 1;j++) 
                    {					
                        //Case of rotations p < j < q.
						ROTATE(matrix, ip, j, j, iq);
					}
					for (j = iq + 1;j <= 3;j++) 
                    {						
                        //Case of rotations q < j . n.
						ROTATE(matrix, ip, j, iq, j);
					}
					for (j = 1;j <= 3;j++) 
                    {
						ROTATE(eigen_vectors, j, ip, j, iq);
					}
					++(*nrot);
				}
			}
		}

		for (ip = 1;ip <= 3;ip++) 
        {
            b->Set(ip, b->at(ip) + z->at(ip));
            eigen_values->Set(ip, b->at(ip));           //Update d with the sum of tapq,
            z->Set(ip, 0.0);                            //and reinitialize z.
		}
	}

	gfLog("Too many iterations in routine jacobi.\n");
    gfDeleteVector3(z);
	gfDeleteVector3(b);

    return GF_E_FAIL;
}

void gfGetMultiindices(gfInt multi, gfInt sum, std::vector< std::vector<gfInt> >& multiindices)
{
    multiindices.clear();
    if (sum < 0 || multi < 1)
        return;

    if (multi == 1)
    {
        std::vector<gfInt> vec;
        vec.push_back(sum);
        multiindices.push_back(vec);
        return;
    }

    for (int i = 0;i <= sum;i++)
    {
        // get smaller multiindices
        std::vector< std::vector<gfInt> > aux;
        gfGetMultiindices(multi - 1, sum - i, aux);
        // put it together
        for (int j = 0;j < (int)aux.size();j++)
        {
            std::vector<gfInt> vec = aux[j];
            vec.push_back(0);
            for (int k = (int)vec.size()-2;k >= 0;k--)
                vec[k+1] = vec[k];
            vec[0] = i;
            multiindices.push_back(vec);
        }
    }
}
