//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfAuxRaytracer.h                                     //
//   Description: Class for computing raytraced scenes          //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_AUX_RAYTRACER_H_
#define _GF_C_AUX_RAYTRACER_H_


#include "gfDefines.h"
#include "gfPoint.h"
#include "gfVector.h"
#include <vector>
#include "gfScnGraph.h"
#include "gfScnCamera.h"
#include "gfScnLight.h"
#include "gfScnMaterial.h"


class gfAuxRaytracer
{
private:
    gfInt m_width;
    gfInt m_height;
    gfInt m_depth;
    gfDouble* m_pic_data;
    gfDouble m_ex, m_ey, m_ez;

    // auxiliary pointers
    gfScnGraph* m_scene;
    gfScnCamera* m_camera;
    gfInt m_hit_count;

    // state of raytracer
    gfBool m_shadows;
    gfBool m_interleaved;
    gfBool m_bezier_clipping;
    gfAntialiasingType m_aa;

    GFRESULT GetLocalLighting(std::vector<gfScnLight*>* vis_lights, gfScnMaterial* material, gfPoint3* pos, 
                              gfVector3* view_dir, gfVector2* uv, gfVector3* normal, gfVector3* res_color);
    GFRESULT ComputeColorByRay(gfPoint3* ray_begin, gfVector3* ray_dir, gfInt depth, gfVector3* res_color);

public:
    gfAuxRaytracer(void);
    ~gfAuxRaytracer(void);

    GFRESULT SetResolution(gfInt width, gfInt height);
    gfInt GetWidth(void);
    gfInt GetHeight(void);
    GFRESULT SetScene(gfScnGraph* scene);
    GFRESULT SetCamera(gfScnCamera* camera);
    GFRESULT Raytrace(gfDouble depth);
    GFRESULT GetPixel(gfInt i, gfInt j, gfVector3* color);
    gfDouble* GetPixels(void);
    gfDouble GetCameraRatio(void);
    GFRESULT SetShadowsRendering(gfBool state);
    GFRESULT SetInterleaved(gfBool state);
    GFRESULT SetBezierClipping(gfBool state);
    GFRESULT SetAAType(gfAntialiasingType aa);
};

#endif