//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfDefines.h                                          //
//   Description: Basic constants and data types                //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_DEFINES_H_
#define _GF_DEFINES_H_


// geometric and numeric constants
#define GF_EPS 0.0000000001
#define GF_PI 3.141592653589793238462643383279502
#define GF_MAX_DOUBLE 99999999
#define GF_MIN_DOUBLE -99999999
#define GF_MAX_NUM_LIGHTS 5
#define GF_BEZIER_CLIPPING_EPS 0.01
#define GF_BEZIER_CLIPPING_MAX_DEPTH 10
#define GF_BEZIER_CLIPPING_DIVIDE_TRESHOLD 0.95
#define GF_MAX_BEZIER_AABB_TREE_DEPTH 8
#define GF_MAX_MESH_AABB_TREE_DEPTH 10

// OpenGL vis. 
#define GF_VIS_PREC 30
#define GF_BORDER 10
#define GF_TOOLBAR_SIZE 20

// basic types
typedef int gfInt;
typedef float gfFloat;
typedef double gfDouble;
typedef long gfLong;
typedef bool gfBool;
typedef char gfChar;
typedef unsigned char gfByte;


//error handling
typedef int GFRESULT;
#define GF_S_OK 1
#define GF_S_FALSE 2
#define GF_E_FAIL 128
#define GF_E_BADARG 129
#define GF_E_NOTFOUND 130
#define GF_E_NOTIMPL 131

#define GF_FAIL(x) (x > 127)
#define GF_SUCC(x) (x < 128)

// logging
#define GF_LOG_CONSOLE 0
#define GF_LOG_FILE 1

// OGL draw flags
#define GF_FLG_NONE 0
#define GF_FLG_WIRE 1
#define GF_FLG_SOLID 2
#define GF_FLG_BORDER 4
#define GF_FLG_NORMALS 8
#define GF_FLG_BOUND_VOLUME 16
#define GF_FLG_CONTROL_NET 32
#define GF_FLG_VERTEX_ARRAY_CACHE 64
#define GF_FLG_COORDINATE_AXES 128
#define GF_FLG_GRID 256
#define GF_FLG_3D_OPACITY 512

// scene visualization flags
#define GF_SCENE_FORCE_WIRE 1
#define GF_SCENE_FORCE_SOLID 2
#define GF_SCENE_SHOW_CAMERAS 4
#define GF_SCENE_SHOW_LIGHTS 8
#define GF_SCENE_FORCE_CONTROL_NET 16
#define GF_SCENE_FORCE_COMPUTE_NORMALS 32


// enumerated data
enum gfGeomObjID
{
    GF_POLYLINE,
    GF_NURBS_CURVE,
    GF_NURBS_PATCH,
    GF_BEZIER_TRIANGLE,
    GF_BEZIER_SPATCH,
    GF_BEZIER_TETRAHEDRA,
    GF_BEZIER_SVOLUME,
    GF_NURBS_BOX,
    GF_HALF_EDGE_MESH,
    GF_3D_BOX_LATTICE,
};

enum gfBoundVolID
{
    GF_BOUND_VOL_GENERAL,
    GF_BOUND_VOL_AABB,
    GF_BOUND_VOL_OBB,
    GF_BOUND_VOL_SPHERE,
};

enum gfSceneNodeID
{
    GF_SCENE_EMPTY,
    GF_SCENE,
    GF_SCENE_GEOM_OBJECT,
    GF_SCENE_LIGHT,
    GF_SCENE_CAMERA,
    GF_SCENE_MATERIAL,
    GF_SCENE_GROUP,
    GF_SCENE_CONTROL_OBJECT_GROUP,
};

enum gfCameraType
{
    GF_CAMERA_PERSPECTIVE_DEF,
    GF_CAMERA_PERSPECTIVE_CUST,
    GF_CAMERA_ORTHO_FRONT,
    GF_CAMERA_ORTHO_RIGHT,
    GF_CAMERA_ORTHO_TOP,
    GF_CAMERA_ORTHO_CUST,
};

enum gfTextureType
{
    GF_TEXTURE_NONE,
    GF_TEXTURE_1D,
    GF_TEXTURE_2D,
    GF_TEXTURE_3D,
};

enum gfMouseTool
{
    GF_MOUSE_TOOL_NONE,
    GF_MOUSE_TOOL_VIEW_ROTATE,
    GF_MOUSE_TOOL_VIEW_PAN,
    GF_MOUSE_TOOL_VIEW_ZOOM,
    GF_MOUSE_TOOL_OBJECT_SELECT,
    GF_MOUSE_TOOL_OBJECT_MOVE,
    GF_MOUSE_TOOL_OBJECT_UNIFORM_SCALE,
    GF_MOUSE_TOOL_OBJECT_ROTATE_X,
    GF_MOUSE_TOOL_OBJECT_ROTATE_Y,
    GF_MOUSE_TOOL_OBJECT_ROTATE_Z,
};

enum gfAntialiasingType
{
    GF_AA_NONE,
    GF_AA_ADAPTIVE,
};;

// input handling
#define GF_MOUSE_LBUTTON 1
#define GF_MOUSE_MBUTTON 2
#define GF_MOUSE_RBUTTON 3
#define GF_KEY_SHIFT 0x0004
#define GF_KEY_CONTROL 0x0008
#define GF_KEY_DOWN 0
#define GF_KEY_UP 0

// structures
struct gfMouseState
{
    gfBool left_down;
    gfBool right_down;
    gfInt last_down_x;
    gfInt last_down_y;
    gfMouseTool tool;
};

struct gfTripleIndex
{
    gfInt i, j, k;
};


#endif

