//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfMatrix.h                                           //
//   Description: Class for work with 4x4 matrix                //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_MATRIX_H_
#define _GF_C_MATRIX_H_


#include "gfDefines.h"
#include "gfVector.h"


// 2x2 matrix
class gfMatrix2  
{
private:
    gfDouble m_data[2][2];

public:
    gfMatrix2();
    virtual ~gfMatrix2();

    gfMatrix2* Clone(void);
    GFRESULT Set(gfDouble a11, gfDouble a12, gfDouble a21, gfDouble a22);
    GFRESULT Set(gfInt i,gfInt j, gfDouble value);
    gfDouble at(gfInt i,gfInt j);
    gfDouble* GetData(void);
    GFRESULT LoadIdentity(void);
    GFRESULT LoadScale(gfDouble sx, gfDouble sy);
    GFRESULT LoadRotation(gfDouble angle);
    GFRESULT AddScale(gfDouble sx, gfDouble sy);
    GFRESULT AddRotation(gfDouble angle);
    GFRESULT PreMultiply(gfMatrix2* matrix);
    GFRESULT Multiply(gfMatrix2* matrix);
    gfDouble Determinant(void);
    GFRESULT Inverse(void);
};


// 3x3 matrix
class gfMatrix3  
{
private:
    gfDouble m_data[3][3];

public:
    gfMatrix3();
    virtual ~gfMatrix3();

    gfMatrix3* Clone(void);
    GFRESULT Set(gfDouble a11, gfDouble a12, gfDouble a13,
                 gfDouble a21, gfDouble a22, gfDouble a23,
                 gfDouble a31, gfDouble a32, gfDouble a33);
    GFRESULT Set(gfInt i,gfInt j, gfDouble value);
    gfDouble at(gfInt i,gfInt j);
    gfDouble* GetData(void);
    GFRESULT LoadIdentity(void);
    GFRESULT LoadScale(gfDouble sx, gfDouble sy, gfDouble sz);
    GFRESULT LoadRotationX(gfDouble angle);
    GFRESULT LoadRotationY(gfDouble angle);
    GFRESULT LoadRotationZ(gfDouble angle);
    GFRESULT LoadTranslation(gfDouble tx, gfDouble ty);
    GFRESULT AddScale(gfDouble sx, gfDouble sy, gfDouble sz);
    GFRESULT AddRotationX(gfDouble angle);
    GFRESULT AddRotationY(gfDouble angle);
    GFRESULT AddRotationZ(gfDouble angle);
    GFRESULT AddTranslation(gfDouble tx, gfDouble ty);
    GFRESULT PreMultiply(gfMatrix3* matrix);
    GFRESULT Multiply(gfMatrix3* matrix);
    gfDouble Determinant(void);
    GFRESULT Inverse(void);
};


// 4x4 matrix
class gfMatrix4
{
private:
    gfDouble m_data[4][4];

public:
    gfMatrix4();
    virtual ~gfMatrix4();

    gfMatrix4* Clone(void);
    GFRESULT Set(gfDouble a11, gfDouble a12, gfDouble a13, gfDouble a14,
                 gfDouble a21, gfDouble a22, gfDouble a23, gfDouble a24,
                 gfDouble a31, gfDouble a32, gfDouble a33, gfDouble a34,
                 gfDouble a41, gfDouble a42, gfDouble a43, gfDouble a44);
    GFRESULT Set(gfInt i,gfInt j, gfDouble value);
    gfDouble at(gfInt i,gfInt j);
    gfDouble* GetData(void);
    GFRESULT LoadIdentity(void);
    GFRESULT LoadScale(gfDouble sx, gfDouble sy, gfDouble sz, gfDouble sw);
    GFRESULT LoadRotationX(gfDouble angle);
    GFRESULT LoadRotationY(gfDouble angle);
    GFRESULT LoadRotationZ(gfDouble angle);
    GFRESULT LoadTranslation(gfDouble tx, gfDouble ty, gfDouble tz);
    GFRESULT AddScale(gfDouble sx, gfDouble sy, gfDouble sz, gfDouble sw);
    GFRESULT AddRotationX(gfDouble angle);
    GFRESULT AddRotationY(gfDouble angle);
    GFRESULT AddRotationZ(gfDouble angle);
    GFRESULT AddTranslation(gfDouble tx, gfDouble ty, gfDouble tz);
    GFRESULT PreMultiply(gfMatrix4* matrix);
    GFRESULT Multiply(gfMatrix4* matrix);
    gfDouble Determinant(void);
    GFRESULT Inverse(void);
};


#endif
