//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfObj3DBoxLattice.h                                  //
//   Description: Parallerpiped lattice class                   //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_BOX_LATTICE_H_
#define _GF_C_BOX_LATTICE_H_


#include "gfDefines.h"
#include <vector>
#include "gfAux3DArray.h"
#include "gfPoint.h"
#include "gfVector.h"
#include "gfMatrix.h"
#include "gfObjGeomObject.h"
#include "gfObjNURBSCurve.h"
#include "gfObjNURBSPatch.h"
#include "gfObjHalfEdgeMesh.h"


class gfObj3DBoxLattice;


class gfObj3DBoxLattice : public gfObjGeomObject
{
private:

    std::vector<gfPoint3 *> m_control_points;
    gfInt m_num_u_control_points;
    gfInt m_num_v_control_points;
    gfInt m_num_w_control_points;

public:

    gfObj3DBoxLattice(void);
    ~gfObj3DBoxLattice(void);

    // gfObjGeomObject functions
    GFRESULT OGLDraw(gfLong flags);
    gfRayInterInfo* RayIntersection(gfMatrix4* obj_matrix, gfPoint3* ray_begin, gfVector3* ray_dir);
    gfGeomObjID GetID(void);
    GFRESULT Transform(gfMatrix4* matrix);
    gfObjGeomObject* Clone(void);
    gfAuxBoundingVolume* GetBoundVolume(gfBoundVolID id, gfMatrix4* obj_matrix);

    // initialize NURBS box
    GFRESULT Clear(void);
    GFRESULT Init(gfInt num_u_points, gfInt num_v_points, gfInt num_w_points);
    GFRESULT InitUniformNet(gfInt num_u_control_points, gfInt num_v_control_points, gfInt num_w_control_points);

    // setting a retrieving parameters of NURBS box
    gfInt SetParams(gfInt u_degree, gfInt v_degree, gfInt w_degree, gfInt num_u_control_points, gfInt num_v_control_points, gfInt num_w_control_points);
    gfInt GetNumUControlPoints(void);
    gfInt GetNumVControlPoints(void);
    gfInt GetNumWControlPoints(void);
    gfPoint3* GetControlPoint(gfInt u_index, gfInt v_index, gfInt w_index);

    // modelling functions
    GFRESULT DooSabinSubdivisionStep();
    GFRESULT CatmullClarkSubdivisionStep();

} ;


#endif