//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfObjBezierSPatch.h                                  //
//   Description: Rational Bezier N sideed patch class          //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_BEZIER_SPATCH_H_
#define _GF_C_BEZIER_SPATCH_H_


#include "gfDefines.h"
#include <vector>
#include "gfPoint.h"
#include "gfVector.h"
#include "gfMatrix.h"
#include "gfObjGeomObject.h"
#include "gfObjNURBSCurve.h"
#include "gfObjHalfEdgeMesh.h"
#include "gfObjBezierTriangle.h"


class gfObjBezierSPatch : public gfObjGeomObject
{
private:

    std::vector<gfDouble> m_weights;
    std::vector<gfPoint3*> m_control_points;
    std::vector<gfPoint2> m_domain_points;
    std::vector< std::vector<gfInt> > m_multiindices;
    gfInt m_degree;
    gfInt m_multi;
    gfInt m_old_stamp;

    // visualization caches
    std::vector<gfPoint3> m_cached_points;
    std::vector<gfVector3> m_cached_normals;
    std::vector<gfObjBezierTriangle*> m_cached_triangles;
    
    gfBool m_wachspress;

    gfVector3 GetNormal(std::vector<gfDouble> coordinates);

public:

    gfObjBezierSPatch(void);
    ~gfObjBezierSPatch(void);


    // gfObjGeomObject functions
    GFRESULT OGLDraw(gfLong flags);
    gfRayInterInfo* RayIntersection(gfMatrix4* obj_matrix, gfPoint3* ray_begin, gfVector3* ray_dir);
    gfGeomObjID GetID(void);
    GFRESULT Transform(gfMatrix4* matrix);
    gfObjGeomObject* Clone(void);
    gfAuxBoundingVolume* GetBoundVolume(gfBoundVolID id, gfMatrix4* obj_matrix);

    GFRESULT InitRandom(gfInt degree, gfInt multi);

    // working with parameters of patch
    gfInt SetParams(gfInt degree, gfInt multi);
    gfInt GetDegree(void);
    gfInt GetMulti(void);
    gfInt GetNumControlPoints(void);
    gfPoint3* GetControlPoint(std::vector<gfInt> multi_index);
    gfDouble GetWeight(std::vector<gfInt> multi_index);
    GFRESULT SetWeight(std::vector<gfInt> multi_index, gfDouble weight);
    std::vector<gfPoint2> GetDomain();
    void SetDomainPoint(gfInt index, gfPoint2 pt);

    void ResetBaryCoordinatesComp();

    // modeling features
    gfObjBezierSPatch* GetSubpatch(std::vector<gfPoint2> domain_points);
    gfObjBezierTriangle* GetSubtriangle(gfPoint2* pt1, gfPoint2* pt2, gfPoint2* pt3);
    std::vector<gfObjBezierTriangle*> DecomposeToBezierTriangles();

} ;


#endif