//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfObjBezierSVolume.h                                  //
//   Description: Rational Bezier N sideed patch class          //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_BEZIER_SVOLUME_H_
#define _GF_C_BEZIER_SVOLUME_H_


#include "gfDefines.h"
#include <vector>
#include "gfPoint.h"
#include "gfVector.h"
#include "gfMatrix.h"
#include "gfObjGeomObject.h"
#include "gfObjNURBSCurve.h"
#include "gfObjHalfEdgeMesh.h"
#include "gfObjBezierTetrahedra.h"
#include "gfObjBezierSPatch.h"


class gfObjBezierSVolume : public gfObjGeomObject
{
private:

    std::vector<gfDouble> m_weights;
    std::vector<gfPoint3 *> m_control_points;
    gfInt m_num_control_points;
    gfInt m_degree;
    gfInt m_multi;
    gfInt m_old_stamp;
    std::vector< std::vector<gfInt> > m_cached_samples;
    std::vector<gfPoint3> m_cached_points;

public:

    gfObjBezierSVolume(void);
    ~gfObjBezierSVolume(void);


    // gfObjGeomObject functions
    GFRESULT OGLDraw(gfLong flags);
    gfRayInterInfo* RayIntersection(gfMatrix4* obj_matrix, gfPoint3* ray_begin, gfVector3* ray_dir);
    gfGeomObjID GetID(void);
    GFRESULT Transform(gfMatrix4* matrix);
    gfObjGeomObject* Clone(void);
    gfAuxBoundingVolume* GetBoundVolume(gfBoundVolID id, gfMatrix4* obj_matrix);

    GFRESULT InitRandom(gfInt degree, gfInt multi);

    // working with parameters of patch
    gfInt SetParams(gfInt degree, gfInt multi);
    gfInt GetDegree(void);
    gfInt GetMulti(void);
    gfInt GetNumControlPoints(void);
    gfPoint3* GetControlPoint(std::vector<gfInt> multi_index);
    gfDouble GetWeight(std::vector<gfInt> multi_index);
    GFRESULT SetWeight(std::vector<gfInt> multi_index, gfDouble weight);

    gfObjBezierTetrahedra* gfObjBezierSVolume::GetSubtetrahedron(gfPoint3* pt1, gfPoint3* pt2, gfPoint3* pt3, gfPoint3* pt4);
    std::vector<gfObjBezierTetrahedra*> DecomposeToBezierTetrahedras();
    GFRESULT RaiseDegree();
    std::vector<gfObjBezierSPatch*> DecomposeToBoundarySPatches();
    std::vector<gfObjBezierSVolume*> DecomposeToSVolumes(std::vector<gfDouble> value);

} ;


#endif