//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfObjBezierTetrahedra.h                              //
//   Description: Rational Bezier tetrahedra class              //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_BEZIER_TETRAHEDRA_H_
#define _GF_C_BEZIER_TETRAHEDRA_H_


#include "gfDefines.h"
#include <vector>
#include "gfPoint.h"
#include "gfMatrix.h"
#include "gfObjGeomObject.h"
#include "gfObjNURBSCurve.h"
#include "gfObjHalfEdgeMesh.h"


class gfObjBezierTetrahedra : public gfObjGeomObject
{
private:

    std::vector<gfDouble> m_weights;
    std::vector<gfPoint3 *> m_control_points;
    gfInt m_degree;

    gfInt GetPtsOffset(gfInt u_index, gfInt v_index, gfInt w_index, gfInt t_index);

public:

    gfObjBezierTetrahedra(void);
    ~gfObjBezierTetrahedra(void);

    // gfObjGeomObject functions
    GFRESULT OGLDraw(gfLong flags);
    gfRayInterInfo* RayIntersection(gfMatrix4* obj_matrix, gfPoint3* ray_begin, gfVector3* ray_dir);
    gfGeomObjID GetID(void);
    GFRESULT Transform(gfMatrix4* matrix);
    gfObjGeomObject* Clone(void);
    gfAuxBoundingVolume* GetBoundVolume(gfBoundVolID id, gfMatrix4* obj_matrix);

    // initialization
    GFRESULT InitTetrahedron(gfPoint3* center, gfDouble size);
    GFRESULT InitCone(gfPoint3* center, gfDouble radius, gfDouble height);
    GFRESULT InitSphereSector(gfPoint3* center, gfDouble radius);
    GFRESULT InitRandom(gfPoint3* center, gfDouble tx, gfDouble ty, gfDouble tz);

    // working with parameters of tetrahedra
    gfInt SetParams(gfInt degree);
    gfInt GetDegree(void);
    gfInt GetNumControlPoints(void);
    gfPoint3 * GetControlPoint(gfInt u_index, gfInt v_index, gfInt w_index, gfInt t_index);
    gfDouble GetWeight(gfInt u_index, gfInt v_index, gfInt w_index, gfInt t_index);
    GFRESULT SetWeight(gfInt u_index, gfInt v_index, gfInt w_index, gfInt t_index, gfDouble weight);

    // modeling
    gfObjBezierTetrahedra* GetContinuousTetrahedronU(gfInt order);

    //gfObjHalfEdgeMesh* GetControlMesh(void);
    GFRESULT DrawControlNet(void);
    
} ;


#endif