//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfObjGeomObject.h                                    //
//   Description: Interface for every geometric object          //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_GEOM_OBJECT_H_
#define _GF_GEOM_OBJECT_H_


#include "gfDefines.h"
#include "gfMatrix.h"
#include "gfPoint.h"
#include "gfVector.h"
#include <vector>
#include "gfAuxBoundingVolume.h"


struct gfRayInterInfo
{
    // all intersections of ray and object
    std::vector<gfDouble> m_inters;
    std::vector<gfVector3> m_normals;
    std::vector<gfPoint3> m_uvw;

    // normal in the point of first intersection
    /*gfDouble normal_x;
    gfDouble normal_y;
    gfDouble normal_z;*/

};

class gfObjGeomObject
{
public:
    gfInt m_time_stamp;
    gfLong m_flags;

    virtual GFRESULT OGLDraw(gfLong flags) = 0;
    virtual gfRayInterInfo* RayIntersection(gfMatrix4* obj_matrix, gfPoint3* ray_begin, gfVector3* ray_dir) = 0;
    virtual GFRESULT Transform(gfMatrix4* matrix) = 0;
    virtual gfGeomObjID GetID(void) = 0;
    virtual gfObjGeomObject* Clone(void) = 0;
    virtual gfAuxBoundingVolume* GetBoundVolume(gfBoundVolID id, gfMatrix4* obj_matrix) = 0;
    //virtual GFRESULT Randomize(gfDouble size);

    gfLong GetFlags(void)
    {
        return m_flags;
    };
    void SetFlags(gfLong flags)
    {
        m_flags = flags;
    };
};


#endif
