//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfObjPolyLine.h                                      //
//   Description: Polyline class                                //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_POLY_LINE_H_
#define _GF_C_POLY_LINE_H_


#include "gfDefines.h"
#include <vector>
#include "gfPoint.h"
#include "gfVector.h"
#include "gfMatrix.h"
#include "gfObjGeomObject.h"


class gfObjPolyLine : public gfObjGeomObject
{
private:

    std::vector<gfPoint3*> m_points;
    gfBool m_closed;

public:

    gfObjPolyLine(void);
    ~gfObjPolyLine(void);

    // gfObjGeomObject functions
    GFRESULT OGLDraw(gfLong flags);
    gfRayInterInfo* RayIntersection(gfMatrix4* obj_matrix, gfPoint3* ray_begin, gfVector3* ray_dir);
    gfGeomObjID GetID(void);
    GFRESULT Transform(gfMatrix4* matrix);
    gfObjGeomObject* Clone(void);
    gfAuxBoundingVolume* GetBoundVolume(gfBoundVolID id, gfMatrix4* obj_matrix);

    // initialize poly line
    GFRESULT Init(gfInt num_points, gfBool closed);
    GFRESULT InitRandom(gfInt num_points, gfBool closed);
    GFRESULT InitNgon(gfPoint3* center, gfVector3* up, gfInt num_points, gfDouble radius);

    // setting
    gfBool IsClosed(void);
    GFRESULT SetClosed(gfBool closed);
    gfInt GetNumPoints(void);
    GFRESULT SetNumPoints(gfInt num_points);
    gfPoint3* GetPoint(int index);

    // subdivision
    gfObjPolyLine* ChainkinSubdivision(void);
    gfObjPolyLine* CatmulClarkSubdivision(void);
} ;


#endif