//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfObjbjectFactory.h                                  //
//   Description: Factory class for working with objects        //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_OBJECT_FACTORY_H_
#define _GF_OBJECT_FACTORY_H_

// definitions
#include "gfDefines.h"
#include "gfObjGeomObject.h"

// auxiliary objects
#include "gfAuxBoundingVolume.h"
#include <string>

// basic objects
#include "gfVector.h"
#include "gfPoint.h"
#include "gfMatrix.h"

// scene objects
#include "gfScnCamera.h"
#include "gfScnLight.h"
#include "gfScnGraph.h"
#include "gfScnMaterial.h"


// counters
static gfInt m_alloc_geom_objects = 0;
static gfInt m_alloc_points = 0;
static gfInt m_alloc_vectors = 0;
static gfInt m_alloc_matrices = 0;
static gfInt m_alloc_strings = 0;
static gfInt m_alloc_bound_volumes = 0;
static gfInt m_alloc_scene_nodes = 0;
static gfInt m_alloc_cameras = 0;
static gfInt m_alloc_lights = 0;
static gfInt m_alloc_materials = 0;
static gfInt m_alloc_ray_infos = 0;
void gfPrintAllocationStatus(void);

gfObjGeomObject* gfCreateGeomObject(gfGeomObjID id);
GFRESULT gfDeleteGeomObject(gfObjGeomObject* object);

// create string
std::string* gfCreateString(void);
GFRESULT gfDeleteString(std::string* string);

// create matrices
gfMatrix2* gfCreateMatrix2(void);
GFRESULT gfDeleteMatrix2(gfMatrix2* matrix);
gfMatrix3* gfCreateMatrix3(void);
GFRESULT gfDeleteMatrix3(gfMatrix3* matrix);
gfMatrix4* gfCreateMatrix4(void);
GFRESULT gfDeleteMatrix4(gfMatrix4* matrix);

// create points
gfPoint2* gfCreatePoint2(void);
GFRESULT gfDeletePoint2(gfPoint2* point);
gfPoint3* gfCreatePoint3(void);
GFRESULT gfDeletePoint3(gfPoint3* point);
gfPoint4* gfCreatePoint4(void);
GFRESULT gfDeletePoint4(gfPoint4* point);

// create vectors
gfVector2* gfCreateVector2(void);
GFRESULT gfDeleteVector2(gfVector2* vector);
gfVector3* gfCreateVector3(void);
GFRESULT gfDeleteVector3(gfVector3* vector);
gfVector4* gfCreateVector4(void);
GFRESULT gfDeleteVector4(gfVector4* vector);

// create bounding volumes
gfAuxBoundingVolume* gfCreateBoundVolume(gfBoundVolID id);
GFRESULT gfDeleteBoundVolume(gfAuxBoundingVolume* vol);

// create scene objects
gfScnNode* gfCreateSceneNode(void);
GFRESULT gfDeleteSceneNode(gfScnNode* node);
gfScnCamera* gfCreateCamera(void);
GFRESULT gfDeleteCamera(gfScnCamera* camera);
gfScnLight* gfCreateLight(void);
GFRESULT gfDeleteLight(gfScnLight* camera);
gfScnMaterial* gfCreateMaterial(void);
GFRESULT gfDeleteMaterial(gfScnMaterial* material);

gfRayInterInfo* gfCreateRayInterInfo(void);
GFRESULT gfDeleteRayInterInfo(gfRayInterInfo* info);


#endif 
