//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfPoint.cpp                                         //
//   Description: Class for work with 3D homogenous points      //
//                                                              //
//////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gfPoint.h"
#include "gfAuxNumUtils.h"
#include "gfObjectFactory.h"


// ---------------- 2D point -----------------------
gfPoint2::gfPoint2()
{
    m_data[0] = 0.0;
    m_data[1] = 0.0;
}

gfPoint2::gfPoint2(gfDouble x, gfDouble y)
{
    m_data[0] = x;
    m_data[1] = y;
}

gfPoint2::~gfPoint2()
{
}

gfPoint2* gfPoint2::Clone()
{
    gfPoint2* new_point = gfCreatePoint2();
    new_point->Set(m_data[0], m_data[1]);
    return new_point;
}

GFRESULT gfPoint2::Set(gfDouble x, gfDouble y)
{
    m_data[0] = x;
    m_data[1] = y;

    return GF_S_OK;
}

gfDouble* gfPoint2::GetData(void)
{
    return (gfDouble*) m_data;
}

gfDouble gfPoint2::Distance(gfPoint2* vertex)
{
    gfDouble a = m_data[0] - vertex->X();
    gfDouble b = m_data[1] - vertex->Y();
    return gfSquareRoot(a * a + b * b);
}

gfDouble gfPoint2::X(void)
{
    return m_data[0];
}

gfDouble gfPoint2::Y(void)
{
    return m_data[1];
}

GFRESULT gfPoint2::SetX(gfDouble x)
{
    m_data[0] = x;
    return GF_S_OK;
}

GFRESULT gfPoint2::SetY(gfDouble y)
{
    m_data[1] = y;
    return GF_S_OK;
}

GFRESULT gfPoint2::Translate(gfDouble tx, gfDouble ty)
{
    m_data[0] += tx;
    m_data[1] += ty;

    return GF_S_OK;
}


// ---------------- 3D point -----------------------
gfPoint3::gfPoint3()
{
    m_data[0] = 0.0;
    m_data[1] = 0.0;
    m_data[2] = 0.0;
}

gfPoint3::gfPoint3(gfDouble x, gfDouble y, gfDouble z)
{
    m_data[0] = x;
    m_data[1] = y;
    m_data[2] = z;
}

gfPoint3::~gfPoint3()
{
}

gfPoint3* gfPoint3::Clone()
{
    gfPoint3* new_point = gfCreatePoint3();
    new_point->Set(m_data[0], m_data[1], m_data[2]);
    return new_point;
}

GFRESULT gfPoint3::Set(gfDouble x, gfDouble y, gfDouble z)
{
    m_data[0] = x;
    m_data[1] = y;
    m_data[2] = z;

    return GF_S_OK;
}

gfDouble* gfPoint3::GetData(void)
{
    return (gfDouble*) m_data;
}

gfDouble gfPoint3::Distance(gfPoint3* vertex)
{
    gfDouble aux = (m_data[0] - vertex->X()) * (m_data[0] - vertex->X());
    aux += (m_data[1] - vertex->Y()) * (m_data[1] - vertex->Y());
    aux += (m_data[2] - vertex->Z()) * (m_data[2] - vertex->Z());
    return gfSquareRoot(aux);
}

gfDouble gfPoint3::X(void)
{
    return m_data[0];
}

gfDouble gfPoint3::Y(void)
{
    return m_data[1];
}

gfDouble gfPoint3::Z(void)
{
    return m_data[2];
}

GFRESULT gfPoint3::SetX(gfDouble x)
{
    m_data[0] = x;
    return GF_S_OK;
}

GFRESULT gfPoint3::SetY(gfDouble y)
{
    m_data[1] = y;
    return GF_S_OK;
}

GFRESULT gfPoint3::SetZ(gfDouble z)
{
    m_data[2] = z;
    return GF_S_OK;
}

GFRESULT gfPoint3::Translate(gfDouble tx, gfDouble ty, gfDouble tz)
{
    m_data[0] += tx;
    m_data[1] += ty;
    m_data[2] += tz;

    return GF_S_OK;
}


// ---------------- 4D point -----------------------
gfPoint4::gfPoint4()
{
    m_data[0] = 0.0;
    m_data[1] = 0.0;
    m_data[2] = 0.0;
    m_data[3] = 0.0;
}

gfPoint4::gfPoint4(gfDouble x, gfDouble y, gfDouble z, gfDouble w)
{
    m_data[0] = x;
    m_data[1] = y;
    m_data[2] = z;
    m_data[3] = w;
}

gfPoint4::~gfPoint4()
{
}

gfPoint4* gfPoint4::Clone()
{
    gfPoint4* new_point = gfCreatePoint4();
    new_point->Set(m_data[0], m_data[1], m_data[2], m_data[3]);
    return new_point;
}

GFRESULT gfPoint4::Set(gfDouble x, gfDouble y, gfDouble z, gfDouble w)
{
    m_data[0] = x;
    m_data[1] = y;
    m_data[2] = z;
    m_data[3] = w;

    return GF_S_OK;
}

gfDouble* gfPoint4::GetData(void)
{
    return (gfDouble*) m_data;
}

gfDouble gfPoint4::Distance(gfPoint4* vertex)
{
    return gfSquareRoot(m_data[0] * vertex->X() + m_data[1] * vertex->Y() + m_data[2] * vertex->Z() + m_data[3] * vertex->W());
}

gfDouble gfPoint4::X(void)
{
    return m_data[0];
}

gfDouble gfPoint4::Y(void)
{
    return m_data[1];
}

gfDouble gfPoint4::Z(void)
{
    return m_data[2];
}

gfDouble gfPoint4::W(void)
{
    return m_data[3];
}

GFRESULT gfPoint4::SetX(gfDouble x)
{
    m_data[0] = x;
    return GF_S_OK;
}

GFRESULT gfPoint4::SetY(gfDouble y)
{
    m_data[1] = y;
    return GF_S_OK;
}

GFRESULT gfPoint4::SetZ(gfDouble z)
{
    m_data[2] = z;
    return GF_S_OK;
}

GFRESULT gfPoint4::SetW(gfDouble w)
{
    m_data[3] = w;
    return GF_S_OK;
}

GFRESULT gfPoint4::Translate(gfDouble tx, gfDouble ty, gfDouble tz, gfDouble tw)
{
    m_data[0] += tx;
    m_data[1] += ty;
    m_data[2] += tz;
    m_data[3] += tw;

    return GF_S_OK;
}

