//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfPoint.h                                           //
//   Description: Class for work with 3D homogenous points      //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_POINT_H_
#define _GF_C_POINT_H_


#include "gfDefines.h"


// two dimensional point
class gfPoint2  
{
private:
    gfDouble m_data[2];

public:
    gfPoint2();
    gfPoint2(gfDouble x, gfDouble y);
    virtual ~gfPoint2();

    gfPoint2* Clone(void);
    GFRESULT Set(gfDouble x, gfDouble y);
    gfDouble X(void);
    gfDouble Y(void);
    gfDouble* GetData(void);
    gfDouble Distance(gfPoint2* vertex);
    GFRESULT SetX(gfDouble x);
    GFRESULT SetY(gfDouble y);
    GFRESULT Translate(gfDouble tx, gfDouble ty);
};

// three dimensional point
class gfPoint3  
{
private:
    gfDouble m_data[3];

public:
    gfPoint3();
    gfPoint3(gfDouble x, gfDouble y, gfDouble z);
    virtual ~gfPoint3();

    gfPoint3* Clone(void);
    GFRESULT Set(gfDouble x, gfDouble y, gfDouble z);
    gfDouble X(void);
    gfDouble Y(void);
    gfDouble Z(void);
    gfDouble* GetData(void);
    gfDouble Distance(gfPoint3* vertex);
    GFRESULT SetX(gfDouble x);
    GFRESULT SetY(gfDouble y);
    GFRESULT SetZ(gfDouble z);
    GFRESULT Translate(gfDouble tx, gfDouble ty, gfDouble tz);
};

// four dimensional point
class gfPoint4  
{
private:
    gfDouble m_data[4];

public:
    gfPoint4();
    gfPoint4(gfDouble x, gfDouble y, gfDouble z, gfDouble w);
    virtual ~gfPoint4();

    gfPoint4* Clone(void);
    GFRESULT Set(gfDouble x, gfDouble y, gfDouble z, gfDouble w);
    gfDouble X(void);
    gfDouble Y(void);
    gfDouble Z(void);
    gfDouble W(void);
    gfDouble* GetData(void);
    gfDouble Distance(gfPoint4* vertex);
    GFRESULT SetX(gfDouble x);
    GFRESULT SetY(gfDouble y);
    GFRESULT SetZ(gfDouble z);
    GFRESULT SetW(gfDouble w);
    GFRESULT Translate(gfDouble tx, gfDouble ty, gfDouble tz, gfDouble tw);
};

#endif 
