//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfScnCamera.h                                        //
//   Description: Scene camera manipulations                    //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_SCN_CAMERA_H_
#define _GF_SCN_CAMERA_H_


#include "gfDefines.h"
#include <vector>
#include "gfVector.h"
#include "gfPoint.h"


class gfScnCamera
{
public:
    gfCameraType m_type;

    gfDouble m_fi;
    gfDouble m_xi;
    gfDouble m_dist;

    gfDouble m_target_x;
    gfDouble m_target_y;
    gfDouble m_target_z;

    gfDouble m_up_x;
    gfDouble m_up_y;
    gfDouble m_up_z;

    gfDouble m_near;
    gfDouble m_far;
    
    gfDouble m_left;
    gfDouble m_right;
    gfDouble m_top;
    gfDouble m_bottom;

public:
    gfScnCamera(void);

    // initialization functions
    GFRESULT InitDefaultPerspective(void);
    GFRESULT InitFrontView(void);
    GFRESULT InitRightView(void);
    GFRESULT InitTopView(void);

    // OGL support functions
    GFRESULT SetOGLProjectionMatrix(void);
    GFRESULT SetOGLViewMatrix(void);

    // manipulation
    GFRESULT Zoom(gfDouble delta);
    GFRESULT Pan(gfDouble px, gfDouble py);
    GFRESULT RotateHorizontal(gfDouble angle);
    GFRESULT RotateVertical(gfDouble angle);

    GFRESULT SetAspectRatio(gfDouble ratio);
    GFRESULT GetCameraFrame(gfVector3* dir, gfVector3* up, gfVector3* right);
    GFRESULT GetCameraPosition(gfPoint3* pos);
};



#endif