//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfScnLight.h                                         //
//   Description: Scene camera manipulations                    //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_SCN_LIGHT_H_
#define _GF_SCN_LIGHT_H_


#include "gfDefines.h"
#include <vector>


class gfScnLight
{
public:
    gfDouble m_pos_x;
    gfDouble m_pos_y;
    gfDouble m_pos_z;
    gfDouble m_pos_w;

    gfDouble m_ambient_r;
    gfDouble m_ambient_g;
    gfDouble m_ambient_b;
    gfDouble m_ambient_a;

    gfDouble m_diffuse_r;
    gfDouble m_diffuse_g;
    gfDouble m_diffuse_b;
    gfDouble m_diffuse_a;

    gfDouble m_specular_r;
    gfDouble m_specular_g;
    gfDouble m_specular_b;
    gfDouble m_specular_a;
    
    gfDouble m_spot_dir_x;
    gfDouble m_spot_dir_y;
    gfDouble m_spot_dir_z;
    gfDouble m_spot_exp;
    gfDouble m_spot_cutoff;

    gfDouble m_att_const;
    gfDouble m_att_lin;
    gfDouble m_att_quad;

    gfInt m_ogl_id;
    gfBool m_state;

public:
    gfScnLight(void);

    // initialization functions
    GFRESULT InitDefaultLight(void);
    GFRESULT SetAmbient(gfDouble r, gfDouble g, gfDouble b, gfDouble a);
    GFRESULT SetDiffuse(gfDouble r, gfDouble g, gfDouble b, gfDouble a);
    GFRESULT SetSpecular(gfDouble r, gfDouble g, gfDouble b, gfDouble a);
    GFRESULT SetPosition(gfDouble x, gfDouble y, gfDouble z, gfDouble w);
    GFRESULT SetSpotLight(gfDouble dir_x, gfDouble dir_y, gfDouble dir_z, gfDouble exp, gfDouble cutoff);
    GFRESULT SetAttenuation(gfDouble constant, gfDouble linear, gfDouble quadratic);

    // Enable / Disable
    GFRESULT Enable(gfBool state);

    // OGL support functions
    GFRESULT SetAsOGLLight(gfInt light_id);
    GFRESULT SetPosToOGL(void);
    gfInt GetOGLLightId(void);

};



#endif