//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfScnMaterial.h                                      //
//   Description: Scene camera manipulations                    //
//                                                              //
//////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gfScnMaterial.h"
#include "gfAuxNumUtils.h"
#include "gfObjectFactory.h"

#include "OGL/glut.h"


gfScnMaterial::gfScnMaterial(void)
{
    InitDefaultMaterial();
}

// initialization functions
GFRESULT gfScnMaterial::InitDefaultMaterial(void)
{
    m_ambient_r = 0.2;
    m_ambient_g = 0.2;
    m_ambient_b = 0.2;
    m_ambient_a = 1.0;

    m_diffuse_r = 0.8;
    m_diffuse_g = 0.8;
    m_diffuse_b = 0.8;
    m_diffuse_a = 1.0;

    m_specular_r = 1.0;
    m_specular_g = 1.0;
    m_specular_b = 1.0;
    m_specular_a = 1.0;
    
    m_emission_r = 0.0;
    m_emission_g = 0.0;
    m_emission_b = 0.0;
    m_emission_a = 0.0;

    m_shininess = 16.0;
    m_reflection = 0.0;
    m_refraction = 0.0;

    m_texture_type = GF_TEXTURE_NONE;
    m_ogl_texture_id = 0;

    return GF_S_OK;
}

GFRESULT gfScnMaterial::InitAllWhiteMaterial(void)
{
    m_ambient_r = 1.0;
    m_ambient_g = 1.0;
    m_ambient_b = 1.0;
    m_ambient_a = 1.0;

    m_diffuse_r = 1.0;
    m_diffuse_g = 1.0;
    m_diffuse_b = 1.0;
    m_diffuse_a = 1.0;

    m_specular_r = 1.0;
    m_specular_g = 1.0;
    m_specular_b = 1.0;
    m_specular_a = 1.0;
    
    m_emission_r = 0.0;
    m_emission_g = 0.0;
    m_emission_b = 0.0;
    m_emission_a = 0.0;

    m_shininess = 16.0;
    m_reflection = 0.0;
    m_refraction = 0.0;

    m_texture_type = GF_TEXTURE_NONE;
    m_ogl_texture_id = 0;

    return GF_S_OK;
}

gfScnMaterial* gfScnMaterial::Clone(void)
{
    gfScnMaterial* clone = gfCreateMaterial();

    clone->m_ambient_r = m_ambient_r;
    clone->m_ambient_g = m_ambient_g;
    clone->m_ambient_b = m_ambient_b;
    clone->m_ambient_a = m_ambient_a;

    clone->m_diffuse_r = m_diffuse_r;
    clone->m_diffuse_g = m_diffuse_g;
    clone->m_diffuse_b = m_diffuse_b;
    clone->m_diffuse_a = m_diffuse_a;

    clone->m_specular_r = m_specular_r;
    clone->m_specular_g = m_specular_g;
    clone->m_specular_b = m_specular_b;
    clone->m_specular_a = m_specular_a;

    clone->m_emission_r = m_emission_r;
    clone->m_emission_g = m_emission_g;
    clone->m_emission_b = m_emission_b;
    clone->m_emission_a = m_emission_a;

    clone->m_shininess = m_shininess;
    clone->m_reflection = m_reflection;
    clone->m_refraction = m_refraction;

    // TODO:clone textures
    if (m_texture_type == GF_TEXTURE_1D)
    {
        //GLint width, height;
        //gfFloat* pixels = ;
        glBindTexture(GL_TEXTURE_1D, m_ogl_texture_id);
        //glGetTexImage(GL_TEXTURE_1D, 0, GL_RGBA, GL_FLOAT, &pixels);
    }

    return clone;
}

// OGL support functions
GFRESULT gfScnMaterial::SetAsOGLMaterial(void)
{
    // ambient color
    GLfloat ambient[4];
    ambient[0] = (GLfloat) m_ambient_r;
    ambient[1] = (GLfloat) m_ambient_g;
    ambient[2] = (GLfloat) m_ambient_b;
    ambient[3] = (GLfloat) 3*m_ambient_a / GF_VIS_PREC;
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, ambient);

    // specular color
    GLfloat specular[4];
    specular[0] = (GLfloat) m_specular_r;
    specular[1] = (GLfloat) m_specular_g;
    specular[2] = (GLfloat) m_specular_b;
    specular[3] = (GLfloat) 3*m_specular_a / GF_VIS_PREC;
    glMaterialfv(GL_FRONT_AND_BACK, GL_SPECULAR, specular);

    // diffuse color
    GLfloat diffuse[4];
    diffuse[0] = (GLfloat) m_diffuse_r;
    diffuse[1] = (GLfloat) m_diffuse_g;
    diffuse[2] = (GLfloat) m_diffuse_b;
    diffuse[3] = (GLfloat) 3*m_diffuse_a / GF_VIS_PREC;
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, diffuse);

    // emission color
    GLfloat emission[4];
    emission[0] = (GLfloat) m_emission_r;
    emission[1] = (GLfloat) m_emission_g;
    emission[2] = (GLfloat) m_emission_b;
    emission[3] = (GLfloat) m_emission_a;
    glMaterialfv(GL_FRONT_AND_BACK, GL_EMISSION, emission);

    glMaterialf(GL_FRONT_AND_BACK, GL_SHININESS, m_shininess);

    // check if there is any texture
    if (m_texture_type == GF_TEXTURE_NONE)
        return GF_S_OK;

    // bind this texture
    if (m_texture_type = GF_TEXTURE_1D)
    {
        glBindTexture(GL_TEXTURE_1D, m_ogl_texture_id);
        glEnable(GL_TEXTURE_1D);
    }
    else if (m_texture_type = GF_TEXTURE_2D)
    {
        glBindTexture(GL_TEXTURE_2D, m_ogl_texture_id);
        glEnable(GL_TEXTURE_2D);
    }

    return GF_S_OK;
}

GFRESULT gfScnMaterial::LoadTexture(gfTextureType type, std::string file)
{
    return GF_S_OK;
}

GFRESULT gfScnMaterial::ClearTexture(void)
{
    if (m_texture_type == GF_TEXTURE_NONE)
        return GF_S_OK;

    GLuint aux[1];
    aux[0] = m_ogl_texture_id;
    glDeleteTextures(1, aux);
    m_ogl_texture_id = 0;
    m_texture_type = GF_TEXTURE_NONE;

    return GF_S_OK;
}