//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfScnMaterial.h                                      //
//   Description: Scene camera manipulations                    //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_SCN_MATERIAL_H_
#define _GF_SCN_MATERIAL_H_


#include "gfDefines.h"
#include <vector>
#include <string>


class gfScnMaterial
{
public:
    // color material properties
    gfDouble m_ambient_r;
    gfDouble m_ambient_g;
    gfDouble m_ambient_b;
    gfDouble m_ambient_a;

    gfDouble m_diffuse_r;
    gfDouble m_diffuse_g;
    gfDouble m_diffuse_b;
    gfDouble m_diffuse_a;

    gfDouble m_specular_r;
    gfDouble m_specular_g;
    gfDouble m_specular_b;
    gfDouble m_specular_a;
    
    gfDouble m_emission_r;
    gfDouble m_emission_g;
    gfDouble m_emission_b;
    gfDouble m_emission_a;

    gfDouble m_shininess;
    gfDouble m_reflection;
    gfDouble m_refraction;

    // textures
    gfTextureType m_texture_type;
    gfInt m_ogl_texture_id;

public:
    gfScnMaterial(void);

    // initialization functions
    GFRESULT InitDefaultMaterial(void);
    GFRESULT InitAllWhiteMaterial(void);
    gfScnMaterial* Clone(void);

    // OGL support functions
    GFRESULT SetAsOGLMaterial(void);

    // texture functions
    GFRESULT LoadTexture(gfTextureType type, std::string file);
    GFRESULT ClearTexture(void);
};


#endif