//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfVector.h                                          //
//   Description: Class for work with 3D homogenous vectors     //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_C_VECTOR_H_
#define _GF_C_VECTOR_H_


#include "gfDefines.h"


// two dimensional vector
class gfVector2
{
private:
    gfDouble m_data[2];

public:
    gfVector2(void);
    gfVector2(gfDouble x, gfDouble y);
    virtual ~gfVector2(void);
    
    gfVector2* Clone(void);
    GFRESULT Set(gfDouble x, gfDouble y);
    GFRESULT Set(gfInt index, gfDouble value);
    gfDouble at(gfInt index);
    gfDouble X(void);
    gfDouble Y(void);
    GFRESULT SetX(gfDouble x);
    GFRESULT SetY(gfDouble y);
    gfDouble* GetData(void);
    gfDouble Length(void);
    GFRESULT SetLength(gfDouble new_length);
    GFRESULT Normalize(void);
    GFRESULT Rotate(gfDouble alpha);
    GFRESULT Translate(gfDouble tx, gfDouble ty);
};

// three dimensional vector
class gfVector3
{
private:
    gfDouble m_data[3];

public:
    gfVector3(void);
    gfVector3(gfDouble x, gfDouble y, gfDouble z);
    ~gfVector3(void);
    
    gfVector3* Clone(void);
    GFRESULT Set(gfDouble x, gfDouble y, gfDouble z);
    GFRESULT Set(gfInt index, gfDouble value);
    gfDouble at(gfInt index);
    gfDouble X(void);
    gfDouble Y(void);
    gfDouble Z(void);
    GFRESULT SetX(gfDouble x);
    GFRESULT SetY(gfDouble y);
    GFRESULT SetZ(gfDouble z);
    gfDouble* GetData(void);
    gfDouble Length(void);
    GFRESULT SetLength(gfDouble new_length);
    GFRESULT Normalize(void);
    GFRESULT Rotate(gfVector3* axis, gfDouble alpha);
    GFRESULT Translate(gfDouble tx, gfDouble ty, gfDouble tz);
    GFRESULT Flip(void);
};

// four dimensional vector
class gfVector4
{
private:
    gfDouble m_data[4];

public:
    gfVector4();
    gfVector4(gfDouble x, gfDouble y, gfDouble z, gfDouble w);
    virtual ~gfVector4();
    
    gfVector4* Clone(void);
    GFRESULT Set(gfDouble x, gfDouble y, gfDouble z, gfDouble w);
    GFRESULT Set(gfInt index, gfDouble value);
    gfDouble at(gfInt index);
    gfDouble X(void);
    gfDouble Y(void);
    gfDouble Z(void);
    gfDouble W(void);
    GFRESULT SetX(gfDouble x);
    GFRESULT SetY(gfDouble y);
    GFRESULT SetZ(gfDouble z);
    GFRESULT SetW(gfDouble w);
    gfDouble* GetData(void);
    gfDouble Length(void);
    GFRESULT SetLength(gfDouble new_length);
    GFRESULT Normalize(void);
    GFRESULT Translate(gfDouble tx, gfDouble ty, gfDouble tz, gfDouble tw);
};



#endif
