//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinApplication.cpp                                 //
//   Description: Main GeomForge application window             //
//                                                              //
//////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "resource.h"
#include "gfWinApplication.h"
#include "gfKernel.h"


gfWinApplication* gfWinApplication::curr_application = NULL;

LRESULT CALLBACK gfWinApplication::WndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
    gfWinApplication* acc_win = (gfWinApplication*)::GetWindowLong(hwnd, GWL_USERDATA);

	switch (msg) 
	{
	case WM_CLOSE:
        PostQuitMessage(0);
        return 0;
		break;
    case WM_ACTIVATE:
        if ((LOWORD(wparam) == WA_ACTIVE || LOWORD(wparam) == WA_CLICKACTIVE) && acc_win)
        {
            curr_application = acc_win;
            acc_win->stOGLdisplay();
        }
        break;
    case WM_SETFOCUS:
        if (acc_win)
        {
            acc_win->stOGLdisplay();
        }

        break;
	case WM_SIZE:
		{
			if (acc_win)
            {
                return acc_win->OnSize(hwnd, msg, wparam, lparam);
            }
		}
		break;
    case WM_PAINT:
		{
			if (acc_win)
                return acc_win->OnPaint(hwnd, msg, wparam, lparam);
		}
		break;
    case WM_ERASEBKGND:
		{
			if (acc_win)
                return acc_win->OnEraseBackground(hwnd, msg, wparam, lparam);
		}
		break;
    case WM_COMMAND:
		{
			if (acc_win)
                return acc_win->OnCommand(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYDOWN:
		{
			if (acc_win)
                return acc_win->OnKeyDown(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYUP:
		{
			if (acc_win)
                return acc_win->OnKeyUp(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_DESTROY:
		PostQuitMessage(0);
		break;
	default:
        {
		    return DefWindowProc(hwnd, msg, wparam, lparam);
        }
	}

	return DefWindowProc(hwnd, msg, wparam, lparam);
}

gfWinApplication::gfWinApplication(void)
{
    m_mouse_state.left_down = false;
    m_mouse_state.right_down = false;

    SetMouseTool(GF_MOUSE_TOOL_OBJECT_MOVE);
    SetGlobalOGLFlags(GF_FLG_SOLID | GF_FLG_COORDINATE_AXES | GF_FLG_GRID /*| GF_FLG_NORMALS*/);

    srand((unsigned int)GetTickCount());

    gfSetLogFile("c:\\GeomForge.log");
    gfSetLogType(GF_LOG_FILE);
}

gfWinApplication::~gfWinApplication(void)
{
    m_scene.Clear();
    m_ogl_window.KillGLWindow();
    m_raytracer.SetResolution(0, 0);
    m_mouse_toolbar.DestroyToolbar();

    gfLog("\n*************************************\n");
    gfLog("*        Exiting GeomForge...       *\n");
    gfLog("*************************************\n\n");
}

LRESULT gfWinApplication::OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    if (wparam == 'R')
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;

            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SPATCH)
                continue;

            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierSPatch* patch = (gfObjBezierSPatch*) node->m_data;
            patch->ResetBaryCoordinatesComp();
        }

        OGLdisplay();
    }

    return 0;
}

LRESULT gfWinApplication::OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinApplication::OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    OGLdisplay();
    RedrawWindow(m_mouse_toolbar.GetHWND(), NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    return DefWindowProc(hwnd, msg, wparam, lparam);
}

LRESULT gfWinApplication::OnSize(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    Resize(LOWORD(lparam), HIWORD(lparam));
    return 0;
}

LRESULT gfWinApplication::OnEraseBackground(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return DefWindowProc(hwnd, msg, wparam, lparam);
    //return 0;
}

LRESULT gfWinApplication::OnCommand(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt wmId    = LOWORD(wparam); 
    gfInt wmEvent = HIWORD(wparam);

    // Parse the menu selections:
    // -------------------------------------- File menu ---------------------------------------
    if (wmId == ID_FILE_CLEARSCENE)
    {
        m_scene.Clear();
        m_scene.InitDefaultScene();
        m_ogl_window.ClearViews();
        m_ogl_window.CreateDefault1View(m_scene.GetDefPerspectiveCam());

        //OGLdisplay();
    }
    else if (wmId == IDM_EXIT)
        PostQuitMessage(0);
    else if (wmId == ID_FILE_SAVESNAPSHOT)
    {
        // add half-edge mesh from ply file using open dialog
        OPENFILENAME ofn;       // common dialog box structure
        char szFile[260];       // buffer for file name

        // Initialize OPENFILENAME
        ZeroMemory(&ofn, sizeof(ofn));
        ofn.lStructSize = sizeof(ofn);
        ofn.hwndOwner = m_hwnd;
        ofn.lpstrFile = szFile;
        ofn.lpstrFile[0] = '\0';
        ofn.nMaxFile = sizeof(szFile);
        ofn.lpstrFilter = "All\0*.*\0Bmp\0*.bmp\0";
        ofn.nFilterIndex = 1;
        ofn.lpstrFileTitle = NULL;
        ofn.nMaxFileTitle = 0;
        ofn.lpstrInitialDir = NULL;
        ofn.Flags = OFN_SHOWHELP | OFN_OVERWRITEPROMPT;

        // Display the Open dialog box. 
        if (GetSaveFileName(&ofn)) 
        {
            OGLdisplay();
            std::string str;
            str.append(ofn.lpstrFile);
            m_ogl_window.SaveToBmp(str);
        }
    }

    // -------------------------------------- Objects menu ------------------------------------

    else if (wmId == ID_POLYLINE_TRIANGLE)
    {
        // add polyline - triangle
        gfObjPolyLine* curve = (gfObjPolyLine*) gfCreateGeomObject(GF_POLYLINE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitNgon(&center, &normal, 3, 0.7);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_POLYLINE_SQUARE)
    {
        // add polyline - square
        gfObjPolyLine* curve = (gfObjPolyLine*) gfCreateGeomObject(GF_POLYLINE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitNgon(&center, &normal, 4, 0.6);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_POLYLINE_HEXAGON)
    {
        // add polyline - hexagon
        gfObjPolyLine* curve = (gfObjPolyLine*) gfCreateGeomObject(GF_POLYLINE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitNgon(&center, &normal, 6, 0.6);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_POLYLINE_RANDOM4)
    {
        // add polyline - random
        gfObjPolyLine* curve = (gfObjPolyLine*) gfCreateGeomObject(GF_POLYLINE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitRandom(6, false);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_9CIRCLE)
    {
        // add NURBS curve - circle from 9 control points
        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfPoint3 start(0.7, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitCircle(&center, &start, &normal);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_7CIRCLE)
    {
        // add NURBS curve - circle from 7 control points
        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfPoint3 start(0.7, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitCircle(&center, &start, &normal);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_HALFCIRCLE)
    {
        // add NURBS curve - circle from 9 control points
        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfPoint3 start(0.7, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitHalfCircle(&center, &start, &normal);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_RANDOMBEZIER)
    {
        // add NURBS curve - circle from 9 control points
        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        curve->InitBezier(4);
        curve->GetControlPoint(0)->Set(-2.0, gfRand(-2.0, 2.0), 0.0);
        curve->GetControlPoint(1)->Set(-1.0, gfRand(-2.0, 2.0), 0.0);
        curve->GetControlPoint(2)->Set(0.0, gfRand(-2.0, 2.0), 0.0);
        curve->GetControlPoint(3)->Set(1.0, gfRand(-2.0, 2.0), 0.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_SPIRAL)
    {
        // add NURBS curve - circle from 9 control points
        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfPoint3 start(0.7, 0.0, 0.0);
        gfVector3 normal(0.0, 0.0, 1.0);
        curve->InitSpiral(&center, &start, &normal, 5, 0.4);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_RANDOMB)
    {
        // add NURBS curve - circle from 9 control points
        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        curve->InitRandom(2, 9, -2.0, 2.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, curve));

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERTRIANGLE_RANDOM3)
    {
        // add NURBS curve - circle from 9 control points
        gfObjBezierTriangle* triangle = (gfObjBezierTriangle*) gfCreateGeomObject(GF_BEZIER_TRIANGLE);
        triangle->InitRandom(3);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, triangle));

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERSPATCH_RANDOM)
    {
        m_double_dlg.SetInitValue(2);
        m_double_dlg.SetParameterText("Input Bezier SPatch Degree");
        m_double_dlg.CreateDoubleDialog(m_hwnd);
        gfInt degree = (gfInt)m_double_dlg.GetValue();

        m_double_dlg.SetInitValue(5);
        m_double_dlg.SetParameterText("Input Bezier SPatch Points Count");
        m_double_dlg.CreateDoubleDialog(m_hwnd);
        gfInt multi = (gfInt)m_double_dlg.GetValue();

        gfObjBezierSPatch* patch = (gfObjBezierSPatch*) gfCreateGeomObject(GF_BEZIER_SPATCH);
        patch->InitRandom(degree, multi);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_SPHERE)
    {
        // add NURBS patch - sphere
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfPoint3 start(0.7, 0.0, 0.0);
        patch->InitSphere(&center, &start);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TORUS)
    {
        // add NURBS patch - torus
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfPoint3 start1(0.6, 0.0, 0.0);
        gfPoint3 start2(0.4, 0.0, 0.0);
        patch->InitTorus(&center, &start1, &start2);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_CONE)
    {
        // add NURBS patch - cone
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center(0.0, 0.0, 0.0);
        patch->InitCone(&center, 0.7, 0.0, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_CYLINDER)
    {
        // add NURBS patch - cone
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center(0.0, 0.0, 0.0);
        patch->InitCone(&center, 0.4, 0.4, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_RANDOM)
    {
        // add NURBS patch - random
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center(0.0, 0.0, 0.0);
        patch->InitRandom(&center, 1.0, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TRIMMEDSPHERE)
    {
        // add NURBS patch - sphere
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center(0.0, 0.0, 0.0);
        gfPoint3 start(0.7, 0.0, 0.0);
        patch->InitSphere(&center, &start);

        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        center.Set(0.5, 0.5, 0.0);
        start.Set(0.7, 0.5, 0.0);
        gfVector3 up(0.0, 0.0, 1.0);
        curve->InitCircle(&center, &start, &up);

        patch->AddTrimCurve(curve);

        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TRIMMEDSPHEREDOMAIN)
    {
        // add NURBS patch - sphere
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center;
        gfPoint3 start;
        patch->InitPlane(0.0, 1.0, 0.0, 1.0);

        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        center.Set(0.5, 0.5, 0.0);
        start.Set(0.7, 0.5, 0.0);
        gfVector3 up(0.0, 0.0, 1.0);
        curve->InitCircle(&center, &start, &up);

        patch->AddTrimCurve(curve);

        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TRIMMEDCYLINDER)
    {
        // add NURBS patch - trimmed cylinder
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        gfPoint3 center(0.0, 0.0, 0.0);
        patch->InitCone(&center, 0.4, 0.4, 1.0);
        gfPoint3 start;

        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        center.Set(1.6, 0.4, 0.0);
        start.Set(1.6, 0.7, 0.0);
        gfVector3 up(0.0, 0.0, 1.0);
        curve->InitCircle(&center, &start, &up);
        patch->AddTrimCurve(curve);

        curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        center.Set(2.5, 0.5, 0.0);
        start.Set(2.5, 0.7, 0.0);
        curve->InitCircle(&center, &start, &up);
        patch->AddTrimCurve(curve);

        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TRIMMEDCYLINDERDOMAIN)
    {
        // add NURBS patch - trimmed cylinder
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        patch->InitPlane(1.0, 4.0, 0.0, 1.0);
        gfPoint3 center, start;

        gfObjNURBSCurve* curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        center.Set(1.6, 0.4, 0.0);
        start.Set(1.6, 0.7, 0.0);
        gfVector3 up(0.0, 0.0, 1.0);
        curve->InitCircle(&center, &start, &up);
        patch->AddTrimCurve(curve);

        curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
        center.Set(2.5, 0.5, 0.0);
        start.Set(2.5, 0.7, 0.0);
        curve->InitCircle(&center, &start, &up);
        patch->AddTrimCurve(curve);

        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_PROFILEPATCH)
    {
        // add NURBS patch - profile
        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        patch->InitProfile(-2,2,10,-2,2,10);

        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_HALFEDGE_PLANE)
    {
        gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) gfCreateGeomObject(GF_HALF_EDGE_MESH);
        gfPoint3 pt(-1.0, -1.0, 0.0);
        gfVector3 b1(2.0, 0.0, 0.0);
        gfVector3 b2(0.0, 2.0, 0.0);
        mesh->InitPlane(&pt, &b1, &b2, 2.0, 2.0);
        mesh->ComputeFacesNormals();
        gfScnNode* node = m_scene.AddGeomObject(NULL, NULL, mesh);
        m_scene.AddObjectToSelection(node);
        std::vector<gfScnMaterial*> materials;
        m_scene.GetNodeMaterials(node, &materials);
        if (materials.size() > 0)
        {
            gfScnMaterial* material = materials.at(0);
            material->m_reflection = 1.0;
        }

        OGLdisplay();
    }
    else if (wmId == ID_HALFEDGE_CUBE)
    {
        gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) gfCreateGeomObject(GF_HALF_EDGE_MESH);
        gfPoint3 center(0.0, 0.0, 0.0);
        mesh->InitCube(&center, 1.0, 1.0, 1.0);
        mesh->ComputeFacesNormals();
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, mesh));

        //OGLdisplay();
    }
    else if (wmId == ID_HALFEDGE_SPHERE)
    {
        gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) gfCreateGeomObject(GF_HALF_EDGE_MESH);
        gfPoint3 center(0.0, 0.0, 0.0);
        mesh->InitSphere(&center, 1.0, 3*GF_VIS_PREC, 3*GF_VIS_PREC);
        mesh->ComputeSmoothNormals();
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, mesh));

        //OGLdisplay();
    }
    else if (wmId == ID_HALFEDGE_FROM_PLY)
    {
        // add half-edge mesh from ply file using open dialog
        OPENFILENAME ofn;       // common dialog box structure
        char szFile[260];       // buffer for file name

        // Initialize OPENFILENAME
        ZeroMemory(&ofn, sizeof(ofn));
        ofn.lStructSize = sizeof(ofn);
        ofn.hwndOwner = m_hwnd;
        ofn.lpstrFile = szFile;
        ofn.lpstrFile[0] = '\0';
        ofn.nMaxFile = sizeof(szFile);
        ofn.lpstrFilter = "All\0*.*\0Ply\0*.ply\0";
        ofn.nFilterIndex = 1;
        ofn.lpstrFileTitle = NULL;
        ofn.nMaxFileTitle = 0;
        ofn.lpstrInitialDir = NULL;
        ofn.Flags = OFN_PATHMUSTEXIST | OFN_FILEMUSTEXIST;

        // Display the Open dialog box. 
        if (GetOpenFileName(&ofn) == TRUE) 
        {
            SetWindowText(m_hwnd, "GeomForge - Processing .............");
            OGLdisplay();
            gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) gfCreateGeomObject(GF_HALF_EDGE_MESH);
            std::string str;
            str.append(ofn.lpstrFile);
            mesh->InitFromPly(str);
            mesh->ComputeSmoothNormals();
            m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, mesh));
            SetWindowText(m_hwnd, "GeomForge");
        }

        //OGLdisplay();
    }
    else if (wmId == ID_BEZIERTETRAHEDRA_CONE)
    {
        // add NURBS volume - box
        gfObjBezierTetrahedra* tetra = (gfObjBezierTetrahedra*) gfCreateGeomObject(GF_BEZIER_TETRAHEDRA);
        gfPoint3 center(0.0, 0.0, 0.0);
        tetra->InitCone(&center, 0.5, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, tetra));

        //OGLdisplay();
    }
    else if (wmId == ID_BEZIERTETRAHEDRA_TETRAHEDRON)
    {
        // add NURBS volume - box
        gfObjBezierTetrahedra* tetra = (gfObjBezierTetrahedra*) gfCreateGeomObject(GF_BEZIER_TETRAHEDRA);
        gfPoint3 center(0.0, 0.0, 0.0);
        tetra->InitTetrahedron(&center, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, tetra));

        //OGLdisplay();
    }
    else if (wmId == ID_BEZIERTETRAHEDRA_SPHERE)
    {
        // add NURBS volume - box
        gfObjBezierTetrahedra* tetra = (gfObjBezierTetrahedra*) gfCreateGeomObject(GF_BEZIER_TETRAHEDRA);
        gfPoint3 center(0.0, 0.0, 0.0);
        tetra->InitSphereSector(&center, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, tetra));

        //OGLdisplay();
    }
    else if (wmId == ID_BEZIERTETRAHEDRA_RANDOM)
    {
        // add NURBS volume - box
        gfObjBezierTetrahedra* tetra = (gfObjBezierTetrahedra*) gfCreateGeomObject(GF_BEZIER_TETRAHEDRA);
        gfPoint3 center(0.0, 0.0, 0.0);
        tetra->InitRandom(&center, 1.0, 1.0, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, tetra));

        //OGLdisplay();
    }
    else if (wmId == ID_BEZIERSVOLUME_RANDOM)
    {
        m_double_dlg.SetInitValue(2);
        m_double_dlg.SetParameterText("Input Bezier SVolume Degree");
        m_double_dlg.CreateDoubleDialog(m_hwnd);
        gfInt degree = (gfInt)m_double_dlg.GetValue();

        m_double_dlg.SetInitValue(5);
        m_double_dlg.SetParameterText("Input Bezier SVolume Points Count");
        m_double_dlg.CreateDoubleDialog(m_hwnd);
        gfInt multi = (gfInt)m_double_dlg.GetValue();


        // add NURBS curve - circle from 9 control points
        gfObjBezierSVolume* patch = (gfObjBezierSVolume*) gfCreateGeomObject(GF_BEZIER_SVOLUME);
        patch->InitRandom(degree, multi);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, patch));

        OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_BOX)
    {
        // add NURBS volume - box
        gfObjNURBSBox* box = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
        gfPoint3 center(0.0, 0.0, 0.0);
        box->InitBox(&center, 0.5, 0.5, 0.5);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, box));

        //OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_SPHERE)
    {
        // add NURBS volume - sphere
        gfObjNURBSBox* sphere = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
        gfPoint3 center(0.0, 0.0, 0.0);
        sphere->InitSphere(&center, 0.6);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, sphere));

        //OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_TORUS)
    {
        // add NURBS volume - torus
        gfObjNURBSBox* torus = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
        gfPoint3 center(0.0, 0.0, 0.0);
        torus->InitTorus(&center, 0.8, 0.2);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, torus));

        //OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_CYLINDER)
    {
        // add NURBS volume - cone
        gfObjNURBSBox* cone = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
        gfPoint3 center(0.0, 0.0, 0.0);
        cone->InitCone(&center, 0.4, 0.4, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, cone));

        //OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_CONE)
    {
        // add NURBS volume - cone
        gfObjNURBSBox* cone = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
        gfPoint3 center(0.0, 0.0, 0.0);
        cone->InitCone(&center, 0.6, 0.0, 1.0);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, cone));

        //OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_RANDOMBEZIER)
    {
        // add NURBS volume - cone
        gfObjNURBSBox* obj = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
        gfPoint3 center(0.0, 0.0, 0.0);
        obj->InitRandom(&center, 0.8, 0.8, 0.8);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, obj));

        //OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_RANDOM)
    {
        // add NURBS volume - cone
        gfObjNURBSBox* obj = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
        obj->InitProfile(-2.0, 2.0, 5, -2.0, 2.0, 4, -2.0, 2.0, 5);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, obj));

        //OGLdisplay();
    }
    else if (wmId == ID_3DBOXLATTICE_UNIFORM)
    {
        // add box lattice
        gfObj3DBoxLattice* obj = (gfObj3DBoxLattice*) gfCreateGeomObject(GF_3D_BOX_LATTICE);
        obj->InitUniformNet(5,6,7);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, obj));

        //OGLdisplay();
    }
    else if (wmId == ID_3DBOXLATTICE_RANDOM)
    {
        // add box lattice
        gfObj3DBoxLattice* obj = (gfObj3DBoxLattice*) gfCreateGeomObject(GF_3D_BOX_LATTICE);
        obj->InitUniformNet(5, 3, 6);
        m_scene.AddObjectToSelection(m_scene.AddGeomObject(NULL, NULL, obj));

        // add random to points
        for (gfInt i = 0;i < obj->GetNumUControlPoints();i++)
        {
            for (gfInt j = 0;j < obj->GetNumUControlPoints();j++)
            {
                for (gfInt k = 0;k < obj->GetNumUControlPoints();k++)
                {
                    gfPoint3* pt = obj->GetControlPoint(i, j, k);
                    pt->SetX(pt->X() + gfRand(-0.2, 0.2));
                    pt->SetY(pt->Y() + gfRand(-0.2, 0.2));
                    pt->SetZ(pt->Z() + gfRand(-0.2, 0.2));
                }
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_OBJECT_COPY)
    {
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;

            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            gfObjGeomObject* new_obj = obj->Clone();
            m_scene.AddGeomObject(NULL, NULL, new_obj);
        }

        //OGLdisplay();
    }
    else if (wmId == ID_OBJECT_DELETE)
    {
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        gfInt i = 0;
        while (i < num_selected)
        {    
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            i++;
            if (!sel_node)
                continue;

            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT &&
                sel_node->m_id != GF_SCENE_CONTROL_OBJECT_GROUP &&
                sel_node->m_id != GF_SCENE_GROUP)
                continue;

            m_scene.RemoveObjectFromSelection(sel_node);
            m_scene.DeleteSubtree(sel_node, true, true);

            num_selected--;
            i--;
        }

        //OGLdisplay();
    }

    // ----------------------- Modifiers menu ------------------------

    else if (wmId == ID_POLYLINE_CHAINKINSUBDIVISION)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_POLYLINE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to Beziers
            gfScnNode* node = good_nodes.at(i);
            gfObjPolyLine* curve = (gfObjPolyLine*) node->m_data;
            gfObjPolyLine* new_curve = curve->ChainkinSubdivision();
            m_scene.DeleteSubtree(node, true, true);
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_curve);
        }

        OGLdisplay();
    }
    else if (wmId == ID_POLYLINE_CATMULL)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_POLYLINE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to Beziers
            gfScnNode* node = good_nodes.at(i);
            gfObjPolyLine* curve = (gfObjPolyLine*) node->m_data;
            gfObjPolyLine* new_curve = curve->CatmulClarkSubdivision();
            m_scene.DeleteSubtree(node, true, true);
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_curve);
        }

        OGLdisplay();
    }
    else if (wmId == ID_POLYLINE_TONURBSPOLYLINE)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_POLYLINE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to Beziers
            gfScnNode* node = good_nodes.at(i);
            gfObjPolyLine* curve = (gfObjPolyLine*) node->m_data;
            gfObjNURBSCurve* new_curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);

            // fill control points
            gfInt num_new_points = curve->GetNumPoints();
            if (curve->IsClosed())
                num_new_points++;
            new_curve->InitPolyline(num_new_points);
            for (gfInt j = 0;j < num_new_points;j++)
            {
                gfPoint3* pt = curve->GetPoint(j % curve->GetNumPoints());
                new_curve->GetControlPoint(j)->Set(pt->X(), pt->Y(), pt->Z());
            }

            m_scene.DeleteSubtree(node, true, true);
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_curve);
        }

        OGLdisplay();
    }
    else if (wmId == ID_POLYLINE_TOBEZIERCURVE)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_POLYLINE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to Beziers
            gfScnNode* node = good_nodes.at(i);
            gfObjPolyLine* curve = (gfObjPolyLine*) node->m_data;
            gfObjNURBSCurve* new_curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);

            // fill control points
            gfInt num_new_points = curve->GetNumPoints();
            if (curve->IsClosed())
                num_new_points++;
            new_curve->InitBezier(num_new_points);
            for (gfInt j = 0;j < num_new_points;j++)
            {
                gfPoint3* pt = curve->GetPoint(j % curve->GetNumPoints());
                new_curve->GetControlPoint(j)->Set(pt->X(), pt->Y(), pt->Z());
            }

            m_scene.DeleteSubtree(node, true, true);
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_curve);
        }

        OGLdisplay();
    }
    else if (wmId == ID_POLYLINE_TONURBSCURVE)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_POLYLINE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to Beziers
            gfScnNode* node = good_nodes.at(i);
            gfObjPolyLine* curve = (gfObjPolyLine*) node->m_data;
            gfObjNURBSCurve* new_curve = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);

            // fill control points and knots
            gfInt num_new_points = curve->GetNumPoints();
            if (curve->IsClosed())
                num_new_points++;
            new_curve->SetParams(3, num_new_points);
            for (gfInt j = 0;j < num_new_points;j++)
            {
                gfPoint3* pt = curve->GetPoint(j % curve->GetNumPoints());
                new_curve->GetControlPoint(j)->Set(pt->X(), pt->Y(), pt->Z());
                new_curve->SetWeight(j, 1.0);
            }
            for (gfInt j = 0;j < new_curve->GetNumKnots();j++)
            {
                new_curve->SetKnot(j, (gfDouble) j);
            }

            m_scene.DeleteSubtree(node, true, true);
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_curve);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_DECOMPOSETOPCB)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_CURVE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to Beziers
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSCurve* curve = (gfObjNURBSCurve*) node->m_data;
            std::vector<gfObjNURBSCurve*> segs;
            curve->DecomposeToBezierSegments(&segs);
            m_scene.DeleteSubtree(node, true, true);

            gfInt num_segs = (gfInt)segs.size();
            for (gfInt r = 0;r < num_segs;r++)
            {
                gfObjNURBSCurve* new_curve = segs.at(r);
                gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_curve);
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_DECOMPOSEBEZIERTOTWO)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_CURVE)
                continue;
            gfObjNURBSCurve* curve = (gfObjNURBSCurve*) sel_node->m_data;
            if (!curve->IsBezier())
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to Beziers
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSCurve* curve = (gfObjNURBSCurve*) node->m_data;
            gfDouble min, max;
            curve->GetDomain(&min, &max);

            gfObjNURBSCurve* curve1 = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
            gfObjNURBSCurve* curve2 = (gfObjNURBSCurve*) gfCreateGeomObject(GF_NURBS_CURVE);
            curve->DecomposeDeCasteljau((min+max)/2.0, curve1, curve2);
            m_scene.DeleteSubtree(node, true, true);

            gfScnNode* new_node1 = m_scene.AddGeomObject(NULL, NULL, curve1);
            gfScnNode* new_node2 = m_scene.AddGeomObject(NULL, NULL, curve2);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_TOSURFACEOFREVOLUTION)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_CURVE)
                continue;
            good_nodes.push_back(sel_node);
        }

        // input angle
        m_double_dlg.SetInitValue(360);
        m_double_dlg.SetParameterText("Angle");
        m_double_dlg.CreateDoubleDialog(m_hwnd);
        gfDouble angle = GF_PI * m_double_dlg.GetValue() / 180.0; 

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to surface of revolution
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSCurve* curve = (gfObjNURBSCurve*) node->m_data;

            gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
            gfPoint3 center(0.0, 0.0, 0.0);
            gfVector3 axis(0.0, 0.0, 1.0);
            patch->InitRotational(&center, &axis, angle, curve);

            // remove curve
            m_scene.DeleteSubtree(node, true, true);

            // add surface
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, patch);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_TOVOLUMEOFREVOLUTION)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_CURVE)
                continue;
            good_nodes.push_back(sel_node);
        }

        // input angle
        m_double_dlg.SetInitValue(360);
        m_double_dlg.SetParameterText("Angle");
        m_double_dlg.CreateDoubleDialog(m_hwnd);
        gfDouble angle = GF_PI * m_double_dlg.GetValue() / 180.0; 

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each curve to surface of revolution
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSCurve* curve = (gfObjNURBSCurve*) node->m_data;

            gfObjNURBSBox* box = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
            gfPoint3 center(0.0, 0.0, 0.0);
            gfVector3 axis(0.0, 0.0, 1.0);
            if (m_double_dlg.GetValue() == 360.0)
                box->InitRotational(&center, &axis, curve);
            else
                box->InitRotational(&center, &axis, angle, curve);

            // remove curve
            m_scene.DeleteSubtree(node, true, true);

            // add surface
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, box);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TOSWEEPVOLUME)
    {
        std::vector<gfScnNode*> good_nodes;
        gfScnNode* curve_node = NULL;

        // find all selected NURBS patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() == GF_NURBS_CURVE)
                curve_node = sel_node;
            if (obj->GetID() != GF_NURBS_PATCH)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        if (num_selected > 0 && curve_node)
        {
            for (gfInt i = 0;i < num_selected;i++)
            {
                gfScnNode* node = good_nodes.at(i);
                gfObjNURBSPatch* patch = (gfObjNURBSPatch*) node->m_data;

                gfObjNURBSBox* box = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
                box->InitTransitional((gfObjNURBSCurve*)curve_node->m_data, patch);

                // remove patches
                m_scene.DeleteSubtree(node, true, true);

                // add surface
                gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, box);
            }
            
            m_scene.DeleteSubtree(curve_node, true, true);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TORULEDVOLUME)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_PATCH)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        if (num_selected > 1)
        {
            gfScnNode* node1 = good_nodes.at(0);
            gfObjNURBSPatch* patch1 = (gfObjNURBSPatch*) node1->m_data;
            gfScnNode* node2 = good_nodes.at(1);
            gfObjNURBSPatch* patch2 = (gfObjNURBSPatch*) node2->m_data;

            gfObjNURBSBox* box = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
            box->InitRuledVolume(patch1, patch2);

            // remove patches
            m_scene.DeleteSubtree(node1, true, false);
            m_scene.DeleteSubtree(node2, true, true);

            // add surface
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, box);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_TOSWEEPSURFACE)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_CURVE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        if (num_selected != 2)
            return 0;

        // convert two curves to sweep surface
        gfScnNode* node1 = good_nodes.at(0);
        gfScnNode* node2 = good_nodes.at(1);
        gfObjNURBSCurve* curve1 = (gfObjNURBSCurve*) node1->m_data;
        gfObjNURBSCurve* curve2 = (gfObjNURBSCurve*) node2->m_data;

        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        patch->InitTransitional(curve1, curve2);

        // remove curve
        m_scene.DeleteSubtree(node1, true, true);
        m_scene.DeleteSubtree(node2, true, true);

        // add surface
        gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, patch);

        OGLdisplay();
    }
    else if (wmId == ID_NURBSCURVE_TONORMALSWEEPSURFACE)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS curves
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_CURVE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        if (num_selected != 2)
            return 0;

        // convert two curves to sweep surface
        gfScnNode* node1 = good_nodes.at(0);
        gfScnNode* node2 = good_nodes.at(1);
        gfObjNURBSCurve* curve1 = (gfObjNURBSCurve*) node1->m_data;
        gfObjNURBSCurve* curve2 = (gfObjNURBSCurve*) node2->m_data;

        gfObjNURBSPatch* patch = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
        patch->InitNormalTransitional(curve1, curve2);

        // remove curve
        m_scene.DeleteSubtree(node1, true, true);
        m_scene.DeleteSubtree(node2, true, true);

        // add surface
        gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, patch);

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERTRIANGLE_SUBDIVIDE)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_TRIANGLE)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierTriangle* patch = (gfObjBezierTriangle*) node->m_data;
            std::vector<gfObjBezierTriangle*> segs = patch->Subdivide(0.0, 0.5, 0.5);
            m_scene.DeleteSubtree(node, true, true);

            gfInt num_segs = (gfInt)segs.size();
            for (gfInt r = 0;r < num_segs;r++)
            {
                gfObjBezierTriangle* new_patch = segs[r];
                gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_patch);
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERSPATCH_DECOMPOSE_TO_TRIANGLES)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SPATCH)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierSPatch* patch = (gfObjBezierSPatch*) node->m_data;
            std::vector<gfObjBezierTriangle*> segs = patch->DecomposeToBezierTriangles();
            m_scene.DeleteSubtree(node, true, true);

            gfInt num_segs = (gfInt)segs.size();
            for (gfInt r = 0;r < num_segs;r++)
            {
                gfObjBezierTriangle* new_patch = segs[r];
                gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_patch);
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERSPATCH_ADDCONTINUOUSPATCH)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected Bezier S-patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SPATCH)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierSPatch* patch = (gfObjBezierSPatch*) node->m_data;
            
            // get patch domain
            std::vector<gfPoint2> domain = patch->GetDomain();
            // compute barycenter of domain points
            gfPoint2 bary(0.0, 0.0);
            gfInt multi = (gfInt)domain.size();
            for (gfInt j = 0;j < multi;j++)
                bary.Translate(domain[j].X() / multi, domain[j].Y() / multi);

            gfPoint2 pt1 = domain[0];
            gfPoint2 pt2 = domain[1];
            gfPoint2 pt3(2*pt2.X()-bary.X(), 2*pt2.Y()-bary.Y());
            gfPoint2 pt4(2*pt1.X()-bary.X(), 2*pt1.Y()-bary.Y());

            std::vector<gfPoint2> domain_new;
            domain_new.push_back(pt1);
            domain_new.push_back(pt2);
            //domain_new.push_back(bary);
            domain_new.push_back(pt3);
            domain_new.push_back(pt4);
            
            gfObjBezierSPatch* new_patch = patch->GetSubpatch(domain_new);
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_patch);

        }

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERSPATCH_SUBPATCH)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected Bezier S-patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SPATCH)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierSPatch* patch = (gfObjBezierSPatch*) node->m_data;

            // get patch domain
            std::vector<gfPoint2> domain = patch->GetDomain();
            // compute barycenter of domain points
            gfPoint2 bary(0.0, 0.0);
            gfInt multi = (gfInt)domain.size();
            for (gfInt j = 0;j < multi;j++)
                bary.Translate(domain[j].X() / multi, domain[j].Y() / multi);

            gfInt new_multi = 5;

            std::vector<gfPoint2> domain_new;
            // regular polygon around bary
            for (gfInt i = 0;i < new_multi;i++)
            {
                gfPoint2 pt;
                pt.SetX(bary.X() + 0.7*cos(2*GF_PI*i/new_multi));
                pt.SetY(bary.Y() + 0.7*sin(2*GF_PI*i/new_multi));
                domain_new.push_back(pt);
            }

            gfObjBezierSPatch* new_patch = patch->GetSubpatch(domain_new);
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_patch);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_DECOMPOSETOPCB)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_PATCH)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSPatch* patch = (gfObjNURBSPatch*) node->m_data;
            gfAux2DArray<gfObjNURBSPatch*> segs;
            patch->DecomposeToBezierSegments(&segs);
            m_scene.DeleteSubtree(node, true, true);

            gfInt num_u_segs = segs.GetISize();
            gfInt num_v_segs = segs.GetJSize();
            for (gfInt r = 0;r < num_u_segs;r++)
            {
                for (gfInt s = 0;s < num_v_segs;s++)
                {
                    gfObjNURBSPatch* new_patch = segs.at(r, s);
                    gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_patch);
                }
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_DECOMPOSEBEZIERTOTWO)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_PATCH)
                continue;
            gfObjNURBSPatch* patch = (gfObjNURBSPatch*) obj;
            if (!patch->IsBezier())
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSPatch* patch = (gfObjNURBSPatch*) node->m_data;

            gfDouble u_min, u_max, v_min, v_max;
            patch->GetDomain(&u_min, &u_max, &v_min, &v_max);

            gfObjNURBSPatch* patch1 = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
            gfObjNURBSPatch* patch2 = (gfObjNURBSPatch*) gfCreateGeomObject(GF_NURBS_PATCH);
            if ((u_max-u_min) > (v_max-v_min))
                patch->DecomposeUDeCasteljau((u_max+u_min)/2.0, patch1, patch2);
            else
                patch->DecomposeVDeCasteljau((v_max+v_min)/2.0, patch1, patch2);

            m_scene.DeleteSubtree(node, true, true);
            m_scene.AddGeomObject(NULL, NULL, patch1);
            m_scene.AddGeomObject(NULL, NULL, patch2);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSPATCH_TOVOLUMEOFREVOLUTION)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS patches
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_PATCH)
                continue;
            gfObjNURBSPatch* patch = (gfObjNURBSPatch*) obj;
            if (!patch->IsBezier())
                continue;
            good_nodes.push_back(sel_node);
        }

        // input angle
        m_double_dlg.SetInitValue(360);
        m_double_dlg.SetParameterText("Angle");
        m_double_dlg.CreateDoubleDialog(m_hwnd);
        gfDouble angle = GF_PI * m_double_dlg.GetValue() / 180.0; 

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each patch to volume of revolution
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSPatch* patch = (gfObjNURBSPatch*) node->m_data;

            gfObjNURBSBox* box = (gfObjNURBSBox*) gfCreateGeomObject(GF_NURBS_BOX);
            gfPoint3 center(0.0, 0.0, 0.0);
            gfVector3 axis(0.0, 0.0, 1.0);
            box->InitRotational(&center, &axis, angle, patch);

            // remove curve
            m_scene.DeleteSubtree(node, true, true);

            // add surface
            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, box);
        }

        OGLdisplay();
    }
    else if (wmId == ID_MODIFIER_CATMULCLARK)
    {
        std::vector<gfScnNode*> good_nodes;

        // subdivide each selected  half-edge mesh
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;

            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_HALF_EDGE_MESH)
                continue;

            good_nodes.push_back(sel_node);
        }

        m_scene.ClearObjectsSelection();
        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* node = good_nodes.at(i);
            gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) node->m_data;
            gfObjHalfEdgeMesh* new_mesh = mesh->CatmullClarkSubdivision();
            new_mesh->ComputeSmoothNormals();

            gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_mesh);

            // copy materials from old to new object
            std::vector<gfScnNode*> mat_nodes;
            for (gfScnNode* child = node->GetFirstChild(); child; child = node->GetNextChild())
            {
                if ((child->m_id == GF_SCENE_MATERIAL) && child->m_data)
                {
                    new_node->AddChild(child);
                    mat_nodes.push_back(child);
                }
            }
            node->RemoveChildsCons(&mat_nodes);

            m_scene.AddObjectToSelection(new_node);
            m_scene.DeleteSubtree(node, true, true);
        }

        //OGLdisplay();
    }
    else if (wmId == ID_MODIFIER_REVERSENORMALS)
    {
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_HALF_EDGE_MESH)
                continue;

            gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) obj;
            mesh->ReverseNormals();
        }
    }
    else if (wmId == ID_HALF_DEFORM)
    {
        // find last NURBS Box objects and all meshes between selected objects
        gfObjNURBSBox* lattice = NULL;
        std::vector<gfScnNode*> meshes;

        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() == GF_NURBS_BOX)
                lattice = (gfObjNURBSBox*) obj;
            if (obj->GetID() != GF_HALF_EDGE_MESH)
                continue;

            //gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) obj;
            meshes.push_back(sel_node);
        }

        if (lattice)
        {
            for (gfInt i = 0;i < (gfInt)meshes.size();i++)
            {
                gfObjHalfEdgeMesh* mesh = (gfObjHalfEdgeMesh*) meshes[i]->m_data;
                mesh->DeformByNURBSLattice(lattice);
                meshes[i]->UpdateBoundVolume();
            }
        }
    }
    else if (wmId == ID_BEZIERSVOLUME_BOUNDARYPATCHES)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SVOLUME)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierSVolume* box = (gfObjBezierSVolume*) node->m_data;

            std::vector<gfObjBezierSPatch*> patches = box->DecomposeToBoundarySPatches();

            for (gfInt j = 0;j < (gfInt)patches.size();j++)
            {
                gfScnNode* new_node1 = m_scene.AddGeomObject(NULL, NULL, patches[j]);
            }

            m_scene.DeleteSubtree(node, true, true);
        }
    }
    else if (wmId == ID_BEZIERSVOLUME_DECOMPOSE_TO_TETRAHEDRAS)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected SVolumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SVOLUME)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierSVolume* patch = (gfObjBezierSVolume*) node->m_data;
            std::vector<gfObjBezierTetrahedra*> segs = patch->DecomposeToBezierTetrahedras();
            m_scene.DeleteSubtree(node, true, true);

            gfInt num_segs = (gfInt)segs.size();
            for (gfInt r = 0;r < num_segs;r++)
            {
                gfObjBezierTetrahedra* new_patch = segs[r];

                gfAuxBoundingVolume* bound = new_patch->GetBoundVolume(GF_BOUND_VOL_AABB, NULL);
                gfPoint3* pt = bound->GetCenter();
                gfMatrix4 mat;
                mat.LoadTranslation(0.2*pt->X(), 0.2*pt->Y(), 0.2*pt->Z());
                gfDeletePoint3(pt);
                new_patch->Transform(&mat);
                gfDeleteBoundVolume(bound);

                gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_patch);
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERSVOLUME_DECOMPOSEDECASTELJAU)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected SVolumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SVOLUME)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();

        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjBezierSVolume* patch = (gfObjBezierSVolume*) node->m_data;

            // prepare value
            std::vector<gfDouble> value;
            value.resize(patch->GetMulti(), 1.0 / patch->GetMulti());

            // decompose
            std::vector<gfObjBezierSVolume*> segs = patch->DecomposeToSVolumes(value);
            m_scene.DeleteSubtree(node, true, true);

            gfInt num_segs = (gfInt)segs.size();
            for (gfInt r = 0;r < num_segs;r++)
            {
                gfObjBezierSVolume* new_patch = segs[r];

                gfAuxBoundingVolume* bound = new_patch->GetBoundVolume(GF_BOUND_VOL_AABB, NULL);
                gfPoint3* pt = bound->GetCenter();
                gfMatrix4 mat;
                mat.LoadTranslation(0.2*pt->X(), 0.2*pt->Y(), 0.2*pt->Z());
                gfDeletePoint3(pt);
                new_patch->Transform(&mat);
                gfDeleteBoundVolume(bound);

                gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_patch);
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_BEZIERSVOLUME_RAISEDEGREE)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected SVolumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_BEZIER_SVOLUME)
                continue;
            
            gfObjBezierSVolume* patch = (gfObjBezierSVolume*) obj;
            patch->RaiseDegree();
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_ADDCONTINUOUSVOLUME)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSBox* box = (gfObjNURBSBox*) node->m_data;

            m_double_dlg.SetInitValue(2);
            m_double_dlg.SetParameterText("Input Continuity");
            m_double_dlg.CreateDoubleDialog(m_hwnd);
            gfInt order = (gfInt)m_double_dlg.GetValue();
            gfObjNURBSBox* box2 = box->AddContinuousBoxU(order);

            gfScnNode* new_node1 = m_scene.AddGeomObject(NULL, NULL, box2);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_DECOMPOSEBEZIERU)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSBox* box = (gfObjNURBSBox*) node->m_data;

            gfDouble u_min, u_max, v_min, v_max, w_min, w_max;
            box->GetDomain(&u_min, &u_max, &v_min, &v_max, &w_min, &w_max);
            gfObjNURBSBox* U1 = (gfObjNURBSBox*)gfCreateGeomObject(GF_NURBS_BOX);
            gfObjNURBSBox* U2 = (gfObjNURBSBox*)gfCreateGeomObject(GF_NURBS_BOX);

            box->DecomposeUDeCasteljau(0.5*(u_min+u_max), U1, U2);

            gfScnNode* new_node1 = m_scene.AddGeomObject(NULL, NULL, U1);
            gfScnNode* new_node2 = m_scene.AddGeomObject(NULL, NULL, U2);

            m_scene.DeleteSubtree(node, true, true);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_RAISEDEGREE)
    {

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;
            
            ((gfObjNURBSBox*)obj)->RaiseDegreeU();
            ((gfObjNURBSBox*)obj)->RaiseDegreeV();
            ((gfObjNURBSBox*)obj)->RaiseDegreeW();
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_BOUNDARYPATCHES)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSBox* box = (gfObjNURBSBox*) node->m_data;
            
            gfDouble u_min, u_max, v_min, v_max, w_min, w_max;
            box->GetDomain(&u_min, &u_max, &v_min, &v_max, &w_min, &w_max);
            gfObjNURBSPatch* U1 = box->GetVWIsopatch(u_min);
            gfObjNURBSPatch* U2 = box->GetVWIsopatch(u_max-GF_EPS);
            gfObjNURBSPatch* V1 = box->GetUWIsopatch(v_min);
            gfObjNURBSPatch* V2 = box->GetUWIsopatch(v_max-GF_EPS);
            gfObjNURBSPatch* W1 = box->GetUVIsopatch(w_min);
            gfObjNURBSPatch* W2 = box->GetUVIsopatch(w_max-GF_EPS);

            gfScnNode* new_node1 = m_scene.AddGeomObject(NULL, NULL, U1);
            gfScnNode* new_node2 = m_scene.AddGeomObject(NULL, NULL, U2);
            gfScnNode* new_node3 = m_scene.AddGeomObject(NULL, NULL, V1);
            gfScnNode* new_node4 = m_scene.AddGeomObject(NULL, NULL, V2);
            gfScnNode* new_node5 = m_scene.AddGeomObject(NULL, NULL, W1);
            gfScnNode* new_node6 = m_scene.AddGeomObject(NULL, NULL, W2);

            m_scene.DeleteSubtree(node, true, true);
        }

        OGLdisplay();
    }
    else if (wmId == ID_NURBSVOLUME_DECOMPOSETOPCB)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;
            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSBox* box = (gfObjNURBSBox*) node->m_data;
            gfAux3DArray<gfObjNURBSBox*> segs;
            box->DecomposeToBezierSegments(&segs);
            m_scene.DeleteSubtree(node, true, true);

            // prepare matrix
            gfMatrix4 mat;
            gfInt num_u_segs = segs.GetISize();
            gfInt num_v_segs = segs.GetJSize();
            gfInt num_w_segs = segs.GetKSize();
            for (gfInt r = 0;r < num_u_segs;r++)
            {
                for (gfInt s = 0;s < num_v_segs;s++)
                {
                    for (gfInt t = 0;t < num_w_segs;t++)
                    {
                        gfObjNURBSBox* new_box = segs.at(r, s, t);

                        // move it from center
                        //mat.LoadTranslation(r-0.5*(num_u_segs-1), s-0.5*(num_v_segs-1), t-0.5*(num_w_segs-1));
                        gfDouble u_min, u_max, v_min, v_max, w_min, w_max;
                        new_box->GetDomain(&u_min, &u_max, &v_min, &v_max, &w_min, &w_max);
                        gfPoint3* pt = new_box->GetVolumePoint(0.5*(u_min + u_max), 0.5*(v_min + v_max), 0.5*(w_min + w_max));
                        mat.LoadTranslation(0.2*pt->X(), 0.2*pt->Y(), 0.2*pt->Z());
                        gfDeletePoint3(pt);
                        new_box->Transform(&mat);

                        gfScnNode* new_node = m_scene.AddGeomObject(NULL, NULL, new_box);
                    }
                }
            }
        }

        OGLdisplay();
    }
    else if (wmId == ID_3DBOXLATTICE_DOO)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_3D_BOX_LATTICE)
                continue;
            
            gfObj3DBoxLattice* box = (gfObj3DBoxLattice*)obj;
            box->DooSabinSubdivisionStep();
        }
    }
    else if (wmId == ID_3DBOXLATTICE_CATMULL)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;
            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;
            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_3D_BOX_LATTICE)
                continue;

            gfObj3DBoxLattice* box = (gfObj3DBoxLattice*)obj;
            box->CatmullClarkSubdivisionStep();
        }
    }
    else if (wmId == ID_MODIFIER_UVISOPATCHES)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;

            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;

            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSBox* box = (gfObjNURBSBox*) node->m_data;

            gfDouble u_min, u_max, v_min, v_max, w_min, w_max;
            box->GetDomain(&u_min, &u_max, &v_min, &v_max, &w_min, &w_max);

            for (gfInt j = 0;j < GF_VIS_PREC;j++)
            {
                
                gfObjNURBSPatch* patch = box->GetUVIsopatch(w_min + j * (w_max - w_min) / (gfDouble)(GF_VIS_PREC));

                gfScnNode* new_node = m_scene.AddGeomObject(node, NULL, patch);
            }

            //gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            node->m_id = GF_SCENE_CONTROL_OBJECT_GROUP;
        }

        OGLdisplay();
    }
    else if (wmId == ID_MODIFIER_UWISOPATCHES)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;

            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;

            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSBox* box = (gfObjNURBSBox*) node->m_data;

            gfDouble u_min, u_max, v_min, v_max, w_min, w_max;
            box->GetDomain(&u_min, &u_max, &v_min, &v_max, &w_min, &w_max);

            for (gfInt j = 0;j < GF_VIS_PREC;j++)
            {
                gfObjNURBSPatch* patch = box->GetUWIsopatch(v_min + j * (v_max - v_min) / (gfDouble)(GF_VIS_PREC));

                gfScnNode* new_node = m_scene.AddGeomObject(node, NULL, patch);
            }

            //gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            node->m_id = GF_SCENE_CONTROL_OBJECT_GROUP;
        }

        OGLdisplay();
    }
    else if (wmId == ID_MODIFIER_VWISOPATCHES)
    {
        std::vector<gfScnNode*> good_nodes;

        // find all selected NURBS volumes
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;

            if (sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            if (obj->GetID() != GF_NURBS_BOX)
                continue;

            good_nodes.push_back(sel_node);
        }

        num_selected = (gfInt)good_nodes.size();
        for (gfInt i = 0;i < num_selected;i++)
        {
            // convert each volume to patches
            gfScnNode* node = good_nodes.at(i);
            gfObjNURBSBox* box = (gfObjNURBSBox*) node->m_data;

            gfDouble u_min, u_max, v_min, v_max, w_min, w_max;
            box->GetDomain(&u_min, &u_max, &v_min, &v_max, &w_min, &w_max);

            for (gfInt j = 0;j < GF_VIS_PREC;j++)
            {
                gfObjNURBSPatch* patch = box->GetVWIsopatch(u_min + j * (u_max - u_min) / (gfDouble)(GF_VIS_PREC));

                gfScnNode* new_node = m_scene.AddGeomObject(node, NULL, patch);
            }

            //gfObjGeomObject* obj = (gfObjGeomObject*) sel_node->m_data;
            node->m_id = GF_SCENE_CONTROL_OBJECT_GROUP;
        }

        OGLdisplay();
    }
    else if (wmId == ID_MODIFIERS_CENTEROBJECT)
    {
        gfInt num_selected = m_scene.GetNumSelectedObjects();
        for (gfInt i = 0;i < num_selected;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node)
                continue;

            if (!sel_node->m_bound_vol)
                continue;

            gfPoint3* center = sel_node->m_bound_vol->GetCenter();

            gfMatrix4* mat = gfCreateMatrix4();
            mat->LoadTranslation(-center->X(), -center->Y(), -center->Z());

            m_scene.TransformNode(sel_node, mat);

            gfDeletePoint3(center);
            gfDeleteMatrix4(mat);
        }

        OGLdisplay();
    }
    else if (wmId == ID_MODIFIERS_MATERIALEDITOR)
    {
        gfInt num_selected = m_scene.GetNumSelectedObjects();

        if (num_selected > 0)
        {
            gfScnNode* node = m_scene.GetSelectedObject(0);
            std::vector<gfScnMaterial*> materials;
            m_scene.GetNodeMaterials(node, &materials);
            gfScnMaterial* material = NULL;
            if (materials.size() > 0)
                material = materials.at(0);

            gfScnMaterial* new_mat = m_material_editor.CreateMaterialEditorDialog(material, m_hwnd);

            if (new_mat)
            {
                for (gfInt i = 0;i < num_selected;i++)
                {
                    gfScnNode* sel_node = m_scene.GetSelectedObject(i);
                    if (!sel_node)
                        continue;

                    m_scene.RemoveAllMaterials(sel_node);
                    m_scene.AddMaterial(sel_node, new_mat->Clone());
                }
            }

            gfDeleteMaterial(new_mat);
        }
    }

    // ----------------------- Window menu ---------------------------

    else if (wmId == ID_VIEWS_1VIEW)
    {
        gfWinView* view = m_ogl_window.GetCurrentView();
        if (view)
        {
            if (view->m_camera->m_type == GF_CAMERA_PERSPECTIVE_DEF)
                m_ogl_window.CreateDefault1View(m_scene.GetDefPerspectiveCam());
            else if (view->m_camera->m_type == GF_CAMERA_ORTHO_TOP)
                m_ogl_window.CreateDefault1View(m_scene.GetDefOrthoTopCam());
            else if (view->m_camera->m_type == GF_CAMERA_ORTHO_RIGHT)
                m_ogl_window.CreateDefault1View(m_scene.GetDefOrthoRightCam());
            else if (view->m_camera->m_type == GF_CAMERA_ORTHO_FRONT)
                m_ogl_window.CreateDefault1View(m_scene.GetDefOrthoFrontCam());
        }
        else
        {
            m_ogl_window.CreateDefault1View(m_scene.GetDefPerspectiveCam());
        }

        OGLdisplay();
    }
    else if (wmId == ID_VIEWS_4VIEW)
    {
        m_ogl_window.CreateDefault4View(m_scene.GetDefOrthoFrontCam(),
            m_scene.GetDefOrthoRightCam(),
            m_scene.GetDefOrthoTopCam(),
            m_scene.GetDefPerspectiveCam());
        OGLdisplay();
    }
    else if (wmId == ID_MOUSETOOL_ROTATEVIEW)
    {
        SetMouseTool(GF_MOUSE_TOOL_VIEW_ROTATE);
    }
    else if (wmId == ID_MOUSETOOL_PANVIEW)
    {
        SetMouseTool(GF_MOUSE_TOOL_VIEW_PAN);
    }
    else if (wmId == ID_MOUSETOOL_ZOOMVIEW)
    {
        SetMouseTool(GF_MOUSE_TOOL_VIEW_ZOOM);
    }
    else if (wmId == ID_MOUSETOOL_SELECTOBJECT)
    {
        SetMouseTool(GF_MOUSE_TOOL_OBJECT_SELECT);
    }
    else if (wmId == ID_MOUSETOOL_MOVEOBJECT)
    {
        SetMouseTool(GF_MOUSE_TOOL_OBJECT_MOVE);
    }
    else if (wmId == ID_MOUSETOOL_SCALEOBJECT)
    {
        SetMouseTool(GF_MOUSE_TOOL_OBJECT_UNIFORM_SCALE);
    }
    else if (wmId == ID_MOUSETOOL_ROATATEOBJECTX)
    {
        SetMouseTool(GF_MOUSE_TOOL_OBJECT_ROTATE_X);
    }
    else if (wmId == ID_MOUSETOOL_ROATATEOBJECTY)
    {
        SetMouseTool(GF_MOUSE_TOOL_OBJECT_ROTATE_Y);
    }
    else if (wmId == ID_MOUSETOOL_ROATATEOBJECTZ)
    {
        SetMouseTool(GF_MOUSE_TOOL_OBJECT_ROTATE_Z);
    }
    else if (wmId == ID_DISPLAY_WIREFRAME)
    {
        if (m_ogl_global_flags & GF_FLG_WIRE)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_WIRE);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_WIRE);
    }
    else if (wmId == ID_DISPLAY_SOLID)
    {
        if (m_ogl_global_flags & GF_FLG_SOLID)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_SOLID);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_SOLID);
    }
    else if (wmId == ID_DISPLAY_COMPUTENORMALS)
    {
        if (m_ogl_global_flags & GF_FLG_NORMALS)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_NORMALS);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_NORMALS);
    }
    else if (wmId == ID_DISPLAY_CONTROLNET)
    {
        if (m_ogl_global_flags & GF_FLG_CONTROL_NET)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_CONTROL_NET);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_CONTROL_NET);
    }
    else if (wmId == ID_DISPLAY_CACHED)
    {
        if (m_ogl_global_flags & GF_FLG_VERTEX_ARRAY_CACHE)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_VERTEX_ARRAY_CACHE);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_VERTEX_ARRAY_CACHE);
    }
    else if (wmId == ID_DISPLAY_COORDINATEAXES)
    {
        if (m_ogl_global_flags & GF_FLG_COORDINATE_AXES)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_COORDINATE_AXES);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_COORDINATE_AXES);
    }
    else if (wmId == ID_DISPLAY_GRID)
    {
        if (m_ogl_global_flags & GF_FLG_GRID)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_GRID);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_GRID);
    }
    else if (wmId == ID_DISPLAY_TRANSPARENCY)
    {
        if (m_ogl_global_flags & GF_FLG_3D_OPACITY)
            SetGlobalOGLFlags(m_ogl_global_flags & ~GF_FLG_3D_OPACITY);
        else
            SetGlobalOGLFlags(m_ogl_global_flags | GF_FLG_3D_OPACITY);
    }

    // ----------------------- Raytrace menu -----------------------

    else if (wmId == ID_RAYTRACE)
    {
        gfWinView* curr_view = m_ogl_window.GetCurrentView();
        if (curr_view && curr_view->m_camera)
        {
            m_raytracer.SetCamera(curr_view->m_camera);
            m_raytracer.SetScene(&m_scene);

            m_raytrace_dlg.CreateRaytraceDialog("", &m_raytracer, m_ogl_window.GetHWND());
            //DialogBoxParam(m_hinstance, MAKEINTRESOURCE(IDD_RAYTRACE_DLG), m_ogl_window.GetHWND(), (DLGPROC)NULL, (LPARAM)this);
        }
    }

    // ----------------------- Help menu ----------------------------

    else if (wmId == IDM_ABOUT)
    {
        m_about_dlg.CreateAboutDialog(m_hwnd);
    }

    return 0;
}

void gfWinApplication::stOGLdisplay(void)
{
    if (curr_application)
        curr_application->OGLdisplay();
}

void gfWinApplication::stOGLkeyboard(gfInt key, gfInt dir)
{
    if (curr_application)
        curr_application->OGLkeyboard(key, dir);
}

void gfWinApplication::stOGLmousemotion(gfInt x, gfInt y, gfInt special)
{
    if (curr_application)
        curr_application->OGLmousemotion(x, y, special);
}

void gfWinApplication::stOGLmouseup(gfInt x, gfInt y, gfInt button, gfInt special)
{
    if (curr_application)
        curr_application->OGLmouseup(x, y, button, special);
}

void gfWinApplication::stOGLmousedown(gfInt x, gfInt y, gfInt button, gfInt special)
{
    if (curr_application)
        curr_application->OGLmousedown(x, y, button, special);
}

void gfWinApplication::stMouseToolbarFunc(gfInt tool)
{
    if (curr_application)
        curr_application->OnMouseToolbar(tool);
}

GFRESULT gfWinApplication::CreateInstance(HINSTANCE hinstance)
{
    WNDCLASSEX wcex;

    wcex.cbSize = sizeof(WNDCLASSEX); 
    wcex.style			= CS_HREDRAW | CS_VREDRAW;
    wcex.lpfnWndProc	= (WNDPROC)WndProc;
    wcex.cbClsExtra		= 0;
    wcex.cbWndExtra		= 0;
    wcex.hInstance		= hinstance;
    wcex.hIcon			= LoadIcon(hinstance, (LPCTSTR)IDI_MAIN_ICON);
    wcex.hCursor		= LoadCursor(NULL, IDC_ARROW);
    wcex.hbrBackground	= (HBRUSH)(COLOR_WINDOW+1);
    wcex.lpszMenuName	= (LPCTSTR)IDC_GEOMFORGE;
    wcex.lpszClassName	= "GFAppWndClass";
    wcex.hIconSm		= LoadIcon(wcex.hInstance, (LPCTSTR)IDI_MAIN_ICON);

    RegisterClassEx(&wcex);

    m_hinstance = hinstance; // Store instance handle in our global variable

    m_hwnd = CreateWindow("GFAppWndClass", "GeomForge", WS_OVERLAPPEDWINDOW,
        CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, NULL, NULL, hinstance, NULL);

    ::SetWindowLong(m_hwnd, GWL_USERDATA, (long) this);

    if (!m_hwnd)
    {
        return GF_E_FAIL;
    }

    gfLog("*************************************\n");
    gfLog("* Creating instance of GeomForge... *\n");
    gfLog("*************************************\n\n");

    // set callback functions
    m_ogl_window.RenderFunc(stOGLdisplay);
    m_ogl_window.MouseDownFunc(stOGLmousedown);
    m_ogl_window.MouseUpFunc(stOGLmouseup);
    m_ogl_window.MouseMoveFunc(stOGLmousemotion);
    m_ogl_window.KeyboardFunc(stOGLkeyboard);

    RECT rect;
    GetClientRect(m_hwnd, &rect);
    m_ogl_window.CreateGLWindow("", rect.right - rect.left, rect.bottom - rect.top, 32, m_hwnd, NULL, wcex.hCursor);

    //m_ogl_window.Resize(rect.right - rect.left, rect.bottom - rect.top);

    return GF_S_OK;
}

GFRESULT gfWinApplication::Initialize(int cmd_show)
{
    // setup extensions
    gfSetup_GL_ARB_shading_language_100();
    gfSetup_GL_ARB_shader_objects();
    gfSetup_GL_ARB_vertex_shader();
    gfSetup_GL_ARB_fragment_shader();
    gfSetup_WGL_ARB_extensions_string();
    gfSetup_WGL_ARB_pixel_format();
    gfSetup_WGL_ARB_pbuffer();
    gfSetup_WGL_ARB_render_texture();
    //gfSetup_GL_ARB_vertex_buffer_object();
    gfSetup_GL_EXT_compiled_vertex_array();

    // initialize window
    m_ogl_window.MakeCurrent();
    glClearColor(0.2f, 0.2f, 0.2f, 0.5f);
    glShadeModel(GL_SMOOTH);
    glClearDepth(1.0f);
    glEnable(GL_DEPTH_TEST);
    glDepthFunc(GL_LEQUAL);
    //glCullFace(GL_BACK);
    //glEnable(GL_CULL_FACE);
    glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
    glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
    //glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);

    // prepare shaders
    phong_shader = gfCreateShader("./Shaders/phong.vert", "./Shaders/phong.frag");
    if (!phong_shader)
        exit(0);

    // initialize scene
    m_scene.InitDefaultScene();
    
    m_ogl_window.CreateDefault1View(m_scene.GetDefPerspectiveCam());
    /*m_ogl_window.CreateDefault4View(m_scene.GetDefOrthoFrontCam(),
                                    m_scene.GetDefOrthoRightCam(),
                                    m_scene.GetDefOrthoTopCam(),
                                    m_scene.GetDefPerspectiveCam());*/

    // prepare toolbars
    m_mouse_toolbar.RegisterWindowClass(m_hinstance);
    // mouse tools toolbar
    m_mouse_toolbar.SetNotifyFunc(stMouseToolbarFunc);
    m_mouse_toolbar.CreateToolbar(m_hinstance, false, true, "", m_hwnd);
    m_mouse_toolbar.AddTool("Rotate View", IDB_VIEW_ROTATE);
    m_mouse_toolbar.AddTool("Pan View", IDB_VIEW_PAN);
    m_mouse_toolbar.AddTool("Zoom View", IDB_VIEW_ZOOM);
    m_mouse_toolbar.AddTool("Select Object", IDB_OBJECT_SELECT);
    m_mouse_toolbar.AddTool("Move Object", IDB_OBJECT_MOVE);
    m_mouse_toolbar.AddTool("Scale Object", IDB_OBJECT_SCALE);
    m_mouse_toolbar.AddTool("Rotate Object X", IDB_OBJECT_ROTATE_X);
    m_mouse_toolbar.AddTool("Rotate Object Y", IDB_OBJECT_ROTATE_Y);
    m_mouse_toolbar.AddTool("Rotate Object Z", IDB_OBJECT_ROTATE_Z);

    // set flags
    SetMouseTool(GF_MOUSE_TOOL_OBJECT_MOVE);
    SetGlobalOGLFlags(GF_FLG_SOLID | GF_FLG_NORMALS | GF_FLG_VERTEX_ARRAY_CACHE | GF_FLG_COORDINATE_AXES | GF_FLG_GRID);

    // show main window maximized
    ShowWindow(m_hwnd, cmd_show);
    SetFocus(m_hwnd);
    //SendMessage(m_hwnd, WM_SYSCOMMAND, SC_MAXIMIZE, NULL);
    SetWindowPos(m_hwnd, NULL, 0, 0, 1024, 768, SWP_NOMOVE);

    return GF_S_OK;
}

GFRESULT gfWinApplication::StartMainLoop(void)
{
    MSG msg;
    while (true) 
	{
        gfInt res = PeekMessage(&msg, NULL, NULL, NULL, PM_REMOVE);
        if (msg.message == WM_QUIT || msg.message == WM_CLOSE)
        {
            break;
        }

        if (res /*&& !IsDialogMessage(m_raytrace_dlg.m_hWnd, &msg)*/)
        {
            TranslateMessage(&msg);
            DispatchMessage(&msg);
        }
        else
        {
            OGLdisplay();
        }
	}

	return GF_S_OK;
}

GFRESULT gfWinApplication::Resize(gfInt width, gfInt height)
{
    // get size of application client area
    RECT rc_app;
    GetClientRect(m_hwnd, &rc_app);
    //InflateRect(&rc_app, -4, -23);

    // set position of toolbars
    gfInt real_toolbar = (GF_TOOLBAR_SIZE + 5);
    SetWindowPos(m_mouse_toolbar.GetHWND(), NULL, 0, 0, real_toolbar, m_mouse_toolbar.GetNumTools()*(real_toolbar-1), SWP_SHOWWINDOW);

    // set position of OGL window
    SetWindowPos(m_ogl_window.GetHWND(), NULL, real_toolbar, 0, rc_app.right - real_toolbar, rc_app.bottom - 0, SWP_SHOWWINDOW);

    OGLdisplay();

    return GF_S_OK;
}

void gfWinApplication::OGLdisplay(void)
{   
    glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);

    // draw scene for all views
    gfInt num_views = m_ogl_window.GetViewsCount();
    for (gfInt i = 0;i < num_views;i++)
    {
        gfWinView* view = m_ogl_window.GetView(i);

        view->SetOGLViewport();
        view->m_camera->SetOGLProjectionMatrix();
        view->m_camera->SetOGLViewMatrix();

        // draw world coordinate frame
        if (m_ogl_global_flags & GF_FLG_COORDINATE_AXES)
        {
            glBegin( GL_LINES );
            glColor3f(1.0f, 0.0f, 0.0f);glVertex3f(0.0f, 0.0f, 0.0f);glVertex3f(100.0f, 0.0f, 0.0f);
            glColor3f(0.0f, 1.0f, 0.0f);glVertex3f(0.0f, 0.0f, 0.0f);glVertex3f(0.0f, 100.0f, 0.0f);
            glColor3f(0.0f, 0.0f, 1.0f);glVertex3f(0.0f, 0.0f, 0.0f);glVertex3f(0.0f, 0.0f, 100.0f);
            glEnd();
        }

        // draw grid
        if (m_ogl_global_flags & GF_FLG_GRID)
        {
            glBegin(GL_LINES);
            glColor3f(1.0f, 1.0f, 1.0f);
            for (gfDouble i = -10.0;i <= 10.0;i += 20.0 / 15.0)
            {
                glVertex3d(i, -10.0, 0.0);
                glVertex3d(i, 10.0, 0.0);
                glVertex3d(-10, i, 0.0);
                glVertex3d(10, i, 0.0);
            }
            glEnd();
        }

        gfSetActiveShader(phong_shader);
        gfSetShaderUniform4f(phong_shader, "light", 3.0, 3.0, 3.0, 1.0);

        m_scene.DrawScene(m_ogl_global_flags);

        gfSetActiveShader(NULL);
    }

    m_ogl_window.DrawViewFrames();

    if (m_fps_counter.SceneUpdated())
    {
        char str[255];
        sprintf(str, "GeomForge - FPS=%d", gfInt(m_fps_counter.GetFPS()));
        SetWindowText(m_hwnd, str);
    }

    SwapBuffers(m_ogl_window.GetHDC());
}

void gfWinApplication::OGLkeyboard(gfInt key, gfInt dir)
{
    switch(key)
    {
    case 27:
        PostQuitMessage(0);
        break;
    default:
        break;
    }
}

void gfWinApplication::OGLmousedown(gfInt x, gfInt y, gfInt button, gfInt special)
{
    gfWinView* view = m_ogl_window.GetCurrentView();
    if (!view)
        return;

    if (m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_SELECT)
    {
        gfPoint3 begin;
        gfVector3 ray;
        m_ogl_window.Get3DRayFromPoint(x, y, &begin, &ray);
        gfDouble t;
        gfScnNode* node = m_scene.PickNodeByRay(&begin, &ray, &t);
        if (!(special & GF_KEY_CONTROL))
            m_scene.ClearObjectsSelection();
        if (node)
        {
            m_scene.AddObjectToSelection(node);
        }
    }

    if (button == GF_MOUSE_LBUTTON)
    {
        m_mouse_state.left_down = true;
        m_mouse_state.last_down_x = x;
        m_mouse_state.last_down_y = y;
    }
    else if (button == GF_MOUSE_RBUTTON)
    {
        m_mouse_state.right_down = true;
        m_mouse_state.last_down_x = x;
        m_mouse_state.last_down_y = y;
    }

    OGLdisplay();
}

void gfWinApplication::OGLmouseup(gfInt x, gfInt y, gfInt button, gfInt special)
{
    if (button == GF_MOUSE_LBUTTON)
    {
        m_mouse_state.left_down = false;
    }
    if (button==GF_MOUSE_RBUTTON)
    {
        m_mouse_state.right_down = false;
    }

    OGLdisplay();
}

void gfWinApplication::OGLmousemotion(gfInt x, gfInt y, gfInt special)
{
    OGLdisplay();

    gfWinView* view = m_ogl_window.GetCurrentView();
    if (!view)
        return;

    if (m_mouse_state.tool == GF_MOUSE_TOOL_VIEW_ROTATE && (m_mouse_state.left_down || m_mouse_state.right_down))
    {
        if (view->m_camera->m_type == GF_CAMERA_PERSPECTIVE_DEF || view->m_camera->m_type == GF_CAMERA_PERSPECTIVE_DEF)
        {
            view->m_camera->RotateHorizontal((m_mouse_state.last_down_x - x) / 100.0f);
            view->m_camera->RotateVertical((m_mouse_state.last_down_y - y) / 100.0f);
        }
    }

    if (m_mouse_state.tool == GF_MOUSE_TOOL_VIEW_PAN && (m_mouse_state.left_down || m_mouse_state.right_down))
    {
        view->m_camera->Pan((m_mouse_state.last_down_x - x) / 100.0f, (m_mouse_state.last_down_y - y) / 100.0f);
    }

    if (m_mouse_state.tool == GF_MOUSE_TOOL_VIEW_ZOOM && (m_mouse_state.left_down || m_mouse_state.right_down))
    {
        view->m_camera->Zoom((m_mouse_state.last_down_y - y) / 100.0f);
    }

    if (m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_MOVE)
    {
        gfInt num_sel = m_scene.GetNumSelectedObjects();

        if (num_sel < 1 || (!m_mouse_state.left_down && !m_mouse_state.right_down))
            return;

        gfMatrix4* mat = gfCreateMatrix4();
        gfWinView* curr_view = m_ogl_window.GetCurrentView();
        if (m_mouse_state.left_down && curr_view)
        {
            // convert to camera coordinates
            gfVector3 dir, up, right;
            curr_view->m_camera->GetCameraFrame(&dir, &up, &right);
            mat->LoadTranslation(right.X()*(-m_mouse_state.last_down_x+x)/100.0f+up.X()*(-m_mouse_state.last_down_y+y)/100.0f, 
                                 right.Y()*(-m_mouse_state.last_down_x+x)/100.0f+up.Y()*(-m_mouse_state.last_down_y+y)/100.0f, 
                                 0.0);
        }
        if (m_mouse_state.right_down)
            mat->LoadTranslation(0.0, 0.0, -(m_mouse_state.last_down_y - y) / 100.0f);

        for (gfInt i = 0;i < num_sel;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node && sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            m_scene.TransformNode(sel_node, mat);
        }

        gfDeleteMatrix4(mat);
    }

    if (m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_UNIFORM_SCALE)
    {
        gfInt num_sel = m_scene.GetNumSelectedObjects();

        if (num_sel < 1 || (!m_mouse_state.left_down && !m_mouse_state.right_down))
            return;

        gfMatrix4* mat = gfCreateMatrix4();
        gfDouble sx = (m_mouse_state.last_down_x - x) / 100.0f;
        mat->LoadScale(1.0 + sx, 1.0 + sx, 1.0 + sx, 1.0);

        for (gfInt i = 0;i < num_sel;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node && sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            m_scene.TransformNode(sel_node, mat);
        }

        gfDeleteMatrix4(mat);
    }

    if (m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_ROTATE_X ||
        m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_ROTATE_Y ||
        m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_ROTATE_Z)
    {
        gfInt num_sel = m_scene.GetNumSelectedObjects();

        if (num_sel < 1 || (!m_mouse_state.left_down && !m_mouse_state.right_down))
            return;

        gfMatrix4* mat = gfCreateMatrix4();
        gfDouble angle = (m_mouse_state.last_down_x - x) / 100.0f;
        if (m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_ROTATE_X)
            mat->LoadRotationX(angle);
        else if (m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_ROTATE_Y)
            mat->LoadRotationY(angle);
        else if (m_mouse_state.tool == GF_MOUSE_TOOL_OBJECT_ROTATE_Z)
            mat->LoadRotationZ(angle);

        for (gfInt i = 0;i < num_sel;i++)
        {
            gfScnNode* sel_node = m_scene.GetSelectedObject(i);
            if (!sel_node && sel_node->m_id != GF_SCENE_GEOM_OBJECT)
                continue;

            m_scene.TransformNode(sel_node, mat);
        }

        gfDeleteMatrix4(mat);
    }

    if (m_mouse_state.left_down || m_mouse_state.right_down)
    {
        m_mouse_state.last_down_x = x;
        m_mouse_state.last_down_y = y;
    }
}

GFRESULT gfWinApplication::SetMouseTool(gfMouseTool tool)
{
    m_mouse_state.tool = tool;

    if (tool == GF_MOUSE_TOOL_VIEW_ROTATE)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROTATEVIEW, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROTATEVIEW, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_VIEW_PAN)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_PANVIEW, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_PANVIEW, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_VIEW_ZOOM)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ZOOMVIEW, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ZOOMVIEW, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_OBJECT_SELECT)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_SELECTOBJECT, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_SELECTOBJECT, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_OBJECT_MOVE)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_MOVEOBJECT, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_MOVEOBJECT, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_OBJECT_UNIFORM_SCALE)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_SCALEOBJECT, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_SCALEOBJECT, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_OBJECT_ROTATE_X)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROATATEOBJECTX, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROATATEOBJECTX, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_OBJECT_ROTATE_Y)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROATATEOBJECTY, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROATATEOBJECTY, MF_UNCHECKED);

    if (tool == GF_MOUSE_TOOL_OBJECT_ROTATE_Z)
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROATATEOBJECTZ, MF_CHECKED);
    else
        CheckMenuItem(GetMenu(m_hwnd), ID_MOUSETOOL_ROATATEOBJECTZ, MF_UNCHECKED);

    m_mouse_toolbar.SetCurrentTool((gfInt)tool - 1); 

    return GF_S_OK;
}

GFRESULT gfWinApplication::SetGlobalOGLFlags(gfInt flags)
{
    m_ogl_global_flags = flags;

    if (flags & GF_FLG_WIRE)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_WIREFRAME, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_WIREFRAME, MF_UNCHECKED);
    }

    if (flags & GF_FLG_SOLID)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_SOLID, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_SOLID, MF_UNCHECKED);
    }

    if (flags & GF_FLG_NORMALS)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_COMPUTENORMALS, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_COMPUTENORMALS, MF_UNCHECKED);
    }

    if (flags & GF_FLG_CONTROL_NET)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_CONTROLNET, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_CONTROLNET, MF_UNCHECKED);
    }

    if (flags & GF_FLG_VERTEX_ARRAY_CACHE)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_CACHED, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_CACHED, MF_UNCHECKED);
    }

    if (flags & GF_FLG_COORDINATE_AXES)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_COORDINATEAXES, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_COORDINATEAXES, MF_UNCHECKED);
    }

    if (flags & GF_FLG_GRID)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_GRID, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_GRID, MF_UNCHECKED);
    }

    if (flags & GF_FLG_3D_OPACITY)
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_TRANSPARENCY, MF_CHECKED);
    }
    else
    {
        CheckMenuItem(GetMenu(m_hwnd), ID_DISPLAY_TRANSPARENCY, MF_UNCHECKED);
    }

    //OGLdisplay();
    return GF_S_OK;
}

void gfWinApplication::OnMouseToolbar(gfInt tool)
{
    SetMouseTool((gfMouseTool) (tool+1));
}