//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinApplication.h                                   //
//   Description: Main GeomForge application window             //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_WIN_APPLICATION_H_
#define _GF_WIN_APPLICATION_H_


#include <windows.h>
#include "gfDefines.h"
#include <string>
#include "gfWinOGLWindow.h"
#include "gfWinRaytrace.h"
#include "gfWinMaterialEditor.h"
#include "gfWinAboutDlg.h"
#include "gfScnGraph.h"
#include "gfWinOGLExtensions.h"
#include "gfWinFPSCounter.h"
#include "gfAuxRaytracer.h"
#include "gfWinToolbar.h"
#include "gfWinInputDoubleDlg.h"


class gfWinApplication
{
private:
    // scene
    gfScnGraph m_scene;
    gfMouseState m_mouse_state;

    // OGL
    gfWinOGLWindow m_ogl_window;
    gfOGLShaderInfo* phong_shader;
    gfInt m_ogl_global_flags;
    gfWinFPSCounter m_fps_counter;

    // raytracer
    gfAuxRaytracer m_raytracer;

    // dialogs
    gfWinRaytrace m_raytrace_dlg;
    gfWinAboutDlg m_about_dlg;
    gfWinMaterialEditor m_material_editor;
    gfWinInputDoubleDlg m_double_dlg;

    // toolbars
    gfWinToolbar m_mouse_toolbar;

    // window management
    HINSTANCE m_hinstance;
    HWND m_hwnd;
    static LRESULT CALLBACK WndProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam);
    static gfWinApplication* curr_application;
    

    // messages to window
    LRESULT OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnSize(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnCommand(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnEraseBackground(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);

    // static callbacks from OGL window
    static void stOGLdisplay(void);
    static void stOGLkeyboard(gfInt key, gfInt dir);
    static void stOGLmousemotion(gfInt x, gfInt y, gfInt special);
    static void stOGLmouseup(gfInt x, gfInt y, gfInt button, gfInt special);
    static void stOGLmousedown(gfInt x, gfInt y, gfInt button, gfInt special);

    // static callbacks from toolbars
    static void stMouseToolbarFunc(gfInt tool);

public:
    gfWinApplication(void);
    ~gfWinApplication(void);

    GFRESULT CreateInstance(HINSTANCE hinstance);
    GFRESULT Initialize(int cmd_show);
    GFRESULT StartMainLoop(void);
    GFRESULT Resize(gfInt width, gfInt height);

    GFRESULT SetMouseTool(gfMouseTool tool);
    GFRESULT SetGlobalOGLFlags(gfInt flags);

    void OGLdisplay(void);
    void OGLkeyboard(gfInt key, gfInt dir);
    void OGLmousemotion(gfInt x, gfInt y, gfInt special);
    void OGLmouseup(gfInt x, gfInt y, gfInt button, gfInt special);
    void OGLmousedown(gfInt x, gfInt y, gfInt button, gfInt special);

    void OnMouseToolbar(gfInt tool);

};

#endif