//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinInputDoubleDlg.h                                //
//   Description: Window for input double parameter             //
//                                                              //
//////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "resource.h"
#include "stdio.h"
#include "gfWinInputDoubleDlg.h"
#include "gfAuxLogging.h"


LRESULT _stdcall gfWinInputDoubleDlg::event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{	
    gfWinInputDoubleDlg* acc_win = (gfWinInputDoubleDlg*)::GetWindowLong(hwnd, GWL_USERDATA);

    switch(msg)
    {
    case WM_INITDIALOG:
        {
            gfWinInputDoubleDlg* acc_win = (gfWinInputDoubleDlg*)lparam;
            ::SetWindowLong(hwnd, GWL_USERDATA, (long) lparam);
            if (acc_win)
            {
                acc_win->m_hWnd = hwnd;
                return acc_win->OnInitDialog(hwnd, msg, wparam, lparam);
            }
        }
        break;
    case WM_COMMAND:
        {
            if (acc_win)
                return acc_win->OnCommand(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_CLOSE:
        {
            if (acc_win)
                return acc_win->OnClose(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_MOVE:
        {
            if (acc_win)
                return acc_win->OnMove(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_PAINT:
        {
            if (acc_win)
                return acc_win->OnPaint(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_KEYDOWN:
        {
            if (acc_win)
                return acc_win->OnKeyDown(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_KEYUP:
        {
            if (acc_win)
                return acc_win->OnKeyUp(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_LBUTTONDBLCLK:
        {
            if (acc_win)
                return acc_win->OnLButtonDblClk(hwnd, msg, wparam, lparam);
        }
        break;
    default: 
        break;
    }

    return 0;
}

gfWinInputDoubleDlg::gfWinInputDoubleDlg(void)
{
}

gfWinInputDoubleDlg::~gfWinInputDoubleDlg(void)
{
    if (m_hWnd)
        DestroyWindow(m_hWnd);
}

LRESULT gfWinInputDoubleDlg::OnInitDialog(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    m_hWnd = hwnd;
    char text[255];
    sprintf(text, "%f", (float) m_init_value);
    SetDlgItemText(m_hWnd, IDC_DOUBLE_EDIT, text);
    ::SetWindowText(GetDlgItem(m_hWnd, IDC_DOUBLE_STATIC), m_param_name.c_str());

    return DefWindowProc(hwnd, msg, wparam, lparam);
}

LRESULT gfWinInputDoubleDlg::OnCommand(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    switch (LOWORD(wparam)) 
    { 
    case IDOK: 
        EndDialog(m_hWnd, 1);

        char text[255];
        GetDlgItemText(m_hWnd, IDC_DOUBLE_EDIT, text, 255);
        m_value = atof(text);

        m_hWnd = NULL;
        return 1;
    }

    return 0;
}

LRESULT gfWinInputDoubleDlg::OnClose(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{ 
    char text[255];
    GetDlgItemText(m_hWnd, IDC_DOUBLE_EDIT, text, 255);
    m_value = atof(text);

    EndDialog(m_hWnd, 1);
    m_hWnd = NULL;

    return 0;
}

LRESULT gfWinInputDoubleDlg::OnMove(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinInputDoubleDlg::OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinInputDoubleDlg::OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinInputDoubleDlg::OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinInputDoubleDlg::OnLButtonDblClk(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

GFRESULT gfWinInputDoubleDlg::CreateDoubleDialog(HWND parent)
{
    if (m_hWnd)
        DestroyWindow(m_hWnd);

    DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_DOUBLE_DLG), parent, (DLGPROC)event, (LPARAM)this);
    m_hWnd = NULL;

    return GF_S_OK;
}

void gfWinInputDoubleDlg::SetInitValue(gfDouble value)
{
    m_init_value = value;
    return;
}

void gfWinInputDoubleDlg::SetParameterText(char* name)
{
    m_param_name.assign(name);
    return;
}

gfDouble gfWinInputDoubleDlg::GetValue(void)
{
    return m_value;
}