//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinMaterialEditor.h                                //
//   Description: Window for changing material                  //
//                                                              //
//////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "resource.h"
#include "gfWinMaterialEditor.h"
#include "gfAuxLogging.h"
#include "gfObjectFactory.h"


LRESULT _stdcall gfWinMaterialEditor::event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{	
    //return DefWindowProc(hwnd, msg, wparam, lparam);
    gfWinMaterialEditor* acc_win = (gfWinMaterialEditor*)::GetWindowLong(hwnd, GWL_USERDATA);

    switch(msg)
    {
    case WM_INITDIALOG:
        {
            gfWinMaterialEditor* acc_win = (gfWinMaterialEditor*)lparam;
            ::SetWindowLong(hwnd, GWL_USERDATA, (long) acc_win);
            if (acc_win)
            {
                return acc_win->OnInitDialog(hwnd, msg, wparam, lparam);
            }
        }
        break;
    case WM_COMMAND:
        {
            if (acc_win)
                return acc_win->OnCommand(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_MOVE:
        {
            if (acc_win)
                return acc_win->OnMove(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_PAINT:
        {
            if (acc_win)
                return acc_win->OnPaint(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_KEYDOWN:
        {
            if (acc_win)
                return acc_win->OnKeyDown(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_KEYUP:
        {
            if (acc_win)
                return acc_win->OnKeyUp(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_LBUTTONDBLCLK:
        {
            if (acc_win)
                return acc_win->OnLButtonDblClk(hwnd, msg, wparam, lparam);
        }
        break;
    default: 
        break;
    }

    return 0;
}

gfWinMaterialEditor::gfWinMaterialEditor(void)
{
    m_material = NULL;
    m_ambient_and_diffuse = true;
    m_ambient_diffuse_changing = false;
}

gfWinMaterialEditor::~gfWinMaterialEditor(void)
{
}

LRESULT gfWinMaterialEditor::OnInitDialog(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    char text[255];

    // ambient
    sprintf(text, "%f", m_material->m_ambient_r);
    SetDlgItemText(hwnd, IDC_AMBIENT_R, text);

    sprintf(text, "%f", m_material->m_ambient_g);
    SetDlgItemText(hwnd, IDC_AMBIENT_G, text);

    sprintf(text, "%f", m_material->m_ambient_b);
    SetDlgItemText(hwnd, IDC_AMBIENT_B, text);

    sprintf(text, "%f", m_material->m_ambient_a);
    SetDlgItemText(hwnd, IDC_AMBIENT_A, text);

    // diffuse
    sprintf(text, "%f", m_material->m_diffuse_r);
    SetDlgItemText(hwnd, IDC_DIFFUSE_R, text);

    sprintf(text, "%f", m_material->m_diffuse_g);
    SetDlgItemText(hwnd, IDC_DIFFUSE_G, text);

    sprintf(text, "%f", m_material->m_diffuse_b);
    SetDlgItemText(hwnd, IDC_DIFFUSE_B, text);

    sprintf(text, "%f", m_material->m_diffuse_a);
    SetDlgItemText(hwnd, IDC_DIFFUSE_A, text);

    // specular
    sprintf(text, "%f", m_material->m_specular_r);
    SetDlgItemText(hwnd, IDC_SPECULAR_R, text);

    sprintf(text, "%f", m_material->m_specular_g);
    SetDlgItemText(hwnd, IDC_SPECULAR_G, text);

    sprintf(text, "%f", m_material->m_specular_b);
    SetDlgItemText(hwnd, IDC_SPECULAR_B, text);

    sprintf(text, "%f", m_material->m_specular_a);
    SetDlgItemText(hwnd, IDC_SPECULAR_A, text);

    // reflection, refraction, shininess
    sprintf(text, "%f", m_material->m_shininess);
    SetDlgItemText(hwnd, IDC_EDIT_SHININESS, text);

    sprintf(text, "%f", m_material->m_reflection);
    SetDlgItemText(hwnd, IDC_EDIT_REFLECTION, text);

    sprintf(text, "%f", m_material->m_refraction);
    SetDlgItemText(hwnd, IDC_EDIT_REFRACTION, text);

    // check box
    if (m_ambient_and_diffuse)
        CheckDlgButton(hwnd, IDC_AMBIENT_AND_DIFFUSE, BST_CHECKED);
    else
        CheckDlgButton(hwnd, IDC_AMBIENT_AND_DIFFUSE, BST_UNCHECKED);

    return DefWindowProc(hwnd, msg, wparam, lparam);
}

LRESULT gfWinMaterialEditor::OnCommand(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    char text[255];
    gfVector4 color;

    switch (LOWORD(wparam)) 
    { 
    case IDCANCEL:
        if (m_material)
            gfDeleteMaterial(m_material);
        m_material = NULL;
        EndDialog(hwnd, 1);

        return 1; 
    case IDOK:
        EndDialog(hwnd, 1);

        return 1;

    // ambient settings
    case IDC_BUTTON_AMBIENT:
        color.Set(m_material->m_ambient_r, m_material->m_ambient_g, m_material->m_ambient_b, m_material->m_ambient_a);
        if (GF_SUCC(GetColorFromDialog(hwnd, &color)))
        {
            m_material->m_ambient_r = color.X();
            m_material->m_ambient_g = color.Y();
            m_material->m_ambient_b = color.Z();
            m_material->m_ambient_a = color.W();

            m_ambient_diffuse_changing = true;

            sprintf(text, "%f", m_material->m_ambient_r);
            SetDlgItemText(hwnd, IDC_AMBIENT_R, text);
            sprintf(text, "%f", m_material->m_ambient_g);
            SetDlgItemText(hwnd, IDC_AMBIENT_G, text);
            sprintf(text, "%f", m_material->m_ambient_b);
            SetDlgItemText(hwnd, IDC_AMBIENT_B, text);
            sprintf(text, "%f", m_material->m_ambient_a);
            SetDlgItemText(hwnd, IDC_AMBIENT_A, text);

            if (m_ambient_and_diffuse)
            {
                sprintf(text, "%f", m_material->m_ambient_r);
                SetDlgItemText(hwnd, IDC_DIFFUSE_R, text);
                sprintf(text, "%f", m_material->m_ambient_g);
                SetDlgItemText(hwnd, IDC_DIFFUSE_G, text);
                sprintf(text, "%f", m_material->m_ambient_b);
                SetDlgItemText(hwnd, IDC_DIFFUSE_B, text);
                sprintf(text, "%f", m_material->m_ambient_a);
                SetDlgItemText(hwnd, IDC_DIFFUSE_A, text);
            }

            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_AMBIENT_R:
        GetDlgItemText(hwnd, IDC_AMBIENT_R, text, 255);
        m_material->m_ambient_r = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_DIFFUSE_R, text);
            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_AMBIENT_G:
        GetDlgItemText(hwnd, IDC_AMBIENT_G, text, 255);
        m_material->m_ambient_g = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_DIFFUSE_G, text);
            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_AMBIENT_B:
        GetDlgItemText(hwnd, IDC_AMBIENT_B, text, 255);
        m_material->m_ambient_b = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_DIFFUSE_B, text);
            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_AMBIENT_A:
        GetDlgItemText(hwnd, IDC_AMBIENT_A, text, 255);
        m_material->m_ambient_a = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_DIFFUSE_A, text);
            m_ambient_diffuse_changing = false;
        }
        break;

        // diffuse settings
    case IDC_BUTTON_DIFFUSE:
        color.Set(m_material->m_diffuse_r, m_material->m_diffuse_g, m_material->m_diffuse_b, m_material->m_diffuse_a);
        if (GF_SUCC(GetColorFromDialog(hwnd, &color)))
        {
            m_material->m_diffuse_r = color.X();
            m_material->m_diffuse_g = color.Y();
            m_material->m_diffuse_b = color.Z();
            m_material->m_diffuse_a = color.W();

            m_ambient_diffuse_changing = true;

            sprintf(text, "%f", m_material->m_ambient_r);
            SetDlgItemText(hwnd, IDC_DIFFUSE_R, text);
            sprintf(text, "%f", m_material->m_ambient_g);
            SetDlgItemText(hwnd, IDC_DIFFUSE_G, text);
            sprintf(text, "%f", m_material->m_ambient_b);
            SetDlgItemText(hwnd, IDC_DIFFUSE_B, text);
            sprintf(text, "%f", m_material->m_ambient_a);
            SetDlgItemText(hwnd, IDC_DIFFUSE_A, text);

            if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
            {
                sprintf(text, "%f", m_material->m_diffuse_r);
                SetDlgItemText(hwnd, IDC_AMBIENT_R, text);
                sprintf(text, "%f", m_material->m_diffuse_g);
                SetDlgItemText(hwnd, IDC_AMBIENT_G, text);
                sprintf(text, "%f", m_material->m_diffuse_b);
                SetDlgItemText(hwnd, IDC_AMBIENT_B, text);
                sprintf(text, "%f", m_material->m_diffuse_a);
                SetDlgItemText(hwnd, IDC_AMBIENT_A, text);
            }

            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_DIFFUSE_R:
        GetDlgItemText(hwnd, IDC_DIFFUSE_R, text, 255);
        m_material->m_diffuse_r = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_AMBIENT_R, text);
            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_DIFFUSE_G:
        GetDlgItemText(hwnd, IDC_DIFFUSE_G, text, 255);
        m_material->m_diffuse_g = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_AMBIENT_G, text);
            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_DIFFUSE_B:
        GetDlgItemText(hwnd, IDC_DIFFUSE_B, text, 255);
        m_material->m_diffuse_b = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_AMBIENT_B, text);
            m_ambient_diffuse_changing = false;
        }
        break;
    case IDC_DIFFUSE_A:
        GetDlgItemText(hwnd, IDC_DIFFUSE_A, text, 255);
        m_material->m_diffuse_a = atof(text);
        if (!m_ambient_diffuse_changing && m_ambient_and_diffuse)
        {
            m_ambient_diffuse_changing = true;
            SetDlgItemText(hwnd, IDC_AMBIENT_A, text);
            m_ambient_diffuse_changing = false;
        }
        break;
        
        // specular settings
    case IDC_BUTTON_SPECULAR:
        color.Set(m_material->m_specular_r, m_material->m_specular_g, m_material->m_specular_b, m_material->m_specular_a);
        if (GF_SUCC(GetColorFromDialog(hwnd, &color)))
        {
            m_material->m_specular_r = color.X();
            m_material->m_specular_g = color.Y();
            m_material->m_specular_b = color.Z();
            m_material->m_specular_a = color.W();
        }
        break;
    case IDC_SPECULAR_R:
        GetDlgItemText(hwnd, IDC_SPECULAR_R, text, 255);
        m_material->m_specular_r = atof(text);
        break;
    case IDC_SPECULAR_G:
        GetDlgItemText(hwnd, IDC_SPECULAR_G, text, 255);
        m_material->m_specular_g = atof(text);
        break;
    case IDC_SPECULAR_B:
        GetDlgItemText(hwnd, IDC_SPECULAR_B, text, 255);
        m_material->m_specular_b = atof(text);
        break;
    case IDC_SPECULAR_A:
        GetDlgItemText(hwnd, IDC_SPECULAR_A, text, 255);
        m_material->m_specular_a = atof(text);
        break;

    case IDC_EDIT_SHININESS:
        GetDlgItemText(hwnd, IDC_EDIT_SHININESS, text, 255);
        m_material->m_shininess = atof(text);
        break;
    case IDC_EDIT_REFLECTION:
        GetDlgItemText(hwnd, IDC_EDIT_REFLECTION, text, 255);
        m_material->m_reflection = atof(text);
        break;
    case IDC_EDIT_REFRACTION:
        GetDlgItemText(hwnd, IDC_EDIT_REFRACTION, text, 255);
        m_material->m_refraction = atof(text);
        break;

    case IDC_AMBIENT_AND_DIFFUSE:
        m_ambient_and_diffuse = !m_ambient_and_diffuse;
        break;
    }

    return 0;
}

LRESULT gfWinMaterialEditor::OnMove(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinMaterialEditor::OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinMaterialEditor::OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinMaterialEditor::OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinMaterialEditor::OnLButtonDblClk(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

gfScnMaterial* gfWinMaterialEditor::CreateMaterialEditorDialog(gfScnMaterial* material, HWND parent)
{
    if (material)
    {
        m_material = material->Clone();
    }
    else
    {
        m_material = gfCreateMaterial();
        m_material->InitDefaultMaterial();
    }

    DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_MATERIAL_DLG), parent, (DLGPROC)event, (LPARAM)this);

    return m_material;
}

GFRESULT gfWinMaterialEditor::GetColorFromDialog(HWND owner, gfVector4* color)
{
    CHOOSECOLOR cc;
    static COLORREF acrCustClr[16];

    // Initialize CHOOSECOLOR 
    ZeroMemory(&cc, sizeof(cc));
    cc.lStructSize = sizeof(cc);
    cc.hwndOwner = owner;
    cc.lpCustColors = (LPDWORD) acrCustClr;
    cc.rgbResult = RGB(255*color->X(), 255*color->Y(), 255*color->Z());
    cc.Flags = CC_FULLOPEN | CC_RGBINIT;

    if (ChooseColor(&cc) == TRUE) 
    {
        gfInt r = GetRValue(cc.rgbResult);
        gfInt g = GetGValue(cc.rgbResult);
        gfInt b = GetBValue(cc.rgbResult);
        color->Set((gfDouble) (r)/255.0, (gfDouble) (g)/255.0, (gfDouble) (b)/255.0, color->W());
        return GF_S_OK;
    }

    return GF_E_FAIL;

}