//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinOGLExtensions.h                                 //
//   Description: Definitions for working with OGL extensions   //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_OGL_EXT_H_
#define _GF_OGL_EXT_H_

#include "gfDefines.h"
#include <string>
#include <stdio.h>
#include <sys/stat.h>

#include "OGL/glut.h"
#include "OGL/glext.h"
#include "OGL/wglext.h"


// shader info
struct gfOGLShaderInfo
{
    GLhandleARB program_obj;
    GLhandleARB vertex_shader;
    GLhandleARB fragment_shader;
};


//-------------------------------definitions-------------------------------------
// shader objects
extern PFNGLCREATEPROGRAMOBJECTARBPROC          glCreateProgramObjectARB;
extern PFNGLDELETEOBJECTARBPROC                 glDeleteObjectARB;
extern PFNGLDETACHOBJECTARBPROC                 glDetachObjectARB;
extern PFNGLUSEPROGRAMOBJECTARBPROC             glUseProgramObjectARB;
extern PFNGLCREATESHADEROBJECTARBPROC           glCreateShaderObjectARB;
extern PFNGLSHADERSOURCEARBPROC                 glShaderSourceARB;
extern PFNGLCOMPILESHADERARBPROC                glCompileShaderARB;
extern PFNGLGETOBJECTPARAMETERIVARBPROC         glGetObjectParameterivARB;
extern PFNGLATTACHOBJECTARBPROC                 glAttachObjectARB;
extern PFNGLGETINFOLOGARBPROC                   glGetInfoLogARB;
extern PFNGLLINKPROGRAMARBPROC                  glLinkProgramARB;
extern PFNGLGETUNIFORMLOCATIONARBPROC           glGetUniformLocationARB;
extern PFNGLUNIFORM4FARBPROC                    glUniform4fARB;
extern PFNGLUNIFORM3FARBPROC                    glUniform3fARB;
extern PFNGLUNIFORM2FARBPROC                    glUniform2fARB;
extern PFNGLUNIFORM1FARBPROC                    glUniform1fARB;
extern PFNGLUNIFORM4IARBPROC                    glUniform4iARB;
extern PFNGLUNIFORM3IARBPROC                    glUniform3iARB;
extern PFNGLUNIFORM2IARBPROC                    glUniform2iARB;
extern PFNGLUNIFORM1IARBPROC                    glUniform1iARB;

// vertex shader
extern PFNGLGETATTRIBLOCATIONARBPROC            glGetAttribLocationARB;

// WGL
extern PFNWGLGETEXTENSIONSSTRINGARBPROC         wglGetExtensionsStringARB;
extern PFNWGLGETPIXELFORMATATTRIBIVARBPROC      wglGetPixelFormatAttribivARB;
extern PFNWGLGETPIXELFORMATATTRIBFVARBPROC      wglGetPixelFormatAttribfvARB;
extern PFNWGLCHOOSEPIXELFORMATARBPROC           wglChoosePixelFormatARB;
extern PFNWGLCREATEPBUFFERARBPROC               wglCreatePbufferARB;
extern PFNWGLGETPBUFFERDCARBPROC                wglGetPbufferDCARB;
extern PFNWGLRELEASEPBUFFERDCARBPROC            wglReleasePbufferDCARB;
extern PFNWGLDESTROYPBUFFERARBPROC              wglDestroyPbufferARB;
extern PFNWGLQUERYPBUFFERARBPROC                wglQueryPbufferARB;
extern PFNWGLBINDTEXIMAGEARBPROC			    wglBindTexImageARB;
extern PFNWGLRELEASETEXIMAGEARBPROC			    wglReleaseTexImageARB;
extern PFNWGLSETPBUFFERATTRIBARBPROC		    wglSetPbufferAttribARB;

// vertex buffer objects
extern PFNGLGENBUFFERSARBPROC                   glGenBuffersARB;
extern PFNGLBINDBUFFERARBPROC                   glBindBufferARB;
extern PFNGLBUFFERDATAARBPROC                   glBufferDataARB;
extern PFNGLDELETEBUFFERSARBPROC                glDeleteBuffersARB;

// compiled vertex arrays
extern PFNGLLOCKARRAYSEXTPROC                   glLockArraysEXT;
extern PFNGLUNLOCKARRAYSEXTPROC                 glUnlockArraysEXT;


// ----------------------------- utility functions --------------------------------

// work with shaders
gfOGLShaderInfo* gfCreateShader(const char *vertex_shader_file, const char *fragment_shader_file);
void gfReleaseShader(gfOGLShaderInfo* shader);
void gfSetActiveShader(gfOGLShaderInfo* shader);
void gfSetShaderUniform4f(gfOGLShaderInfo* shader, const char* uniform, GLfloat f1, GLfloat f2, GLfloat f3, GLfloat f4);
void gfSetShaderUniform3f(gfOGLShaderInfo* shader, const char* uniform, GLfloat f1, GLfloat f2, GLfloat f3);
void gfSetShaderUniform2f(gfOGLShaderInfo* shader, const char* uniform, GLfloat f1, GLfloat f2);
void gfSetShaderUniform1f(gfOGLShaderInfo* shader, const char* uniform, GLfloat f1);
void gfSetShaderUniform4i(gfOGLShaderInfo* shader, const char* uniform, GLint i1, GLint i2, GLint i3, GLint i4);
void gfSetShaderUniform3i(gfOGLShaderInfo* shader, const char* uniform, GLint i1, GLint i2, GLint i3);
void gfSetShaderUniform2i(gfOGLShaderInfo* shader, const char* uniform, GLint i1, GLint i2);
void gfSetShaderUniform1i(gfOGLShaderInfo* shader, const char* uniform, GLint i1);

// functions for setting up extensions
GFRESULT gfSetup_GL_ARB_shading_language_100(void);
GFRESULT gfSetup_GL_ARB_shader_objects(void);
GFRESULT gfSetup_GL_ARB_vertex_shader(void);
GFRESULT gfSetup_GL_ARB_fragment_shader(void);
GFRESULT gfSetup_WGL_ARB_extensions_string(void);
GFRESULT gfSetup_WGL_ARB_pixel_format(void);
GFRESULT gfSetup_WGL_ARB_pbuffer(void);
GFRESULT gfSetup_WGL_ARB_render_texture(void);
GFRESULT gfSetup_GL_ARB_vertex_buffer_object(void);
GFRESULT gfSetup_GL_EXT_compiled_vertex_array(void);


unsigned char *gfReadFile(const char *file_name);
GFRESULT gfCheckExtension(const char *ext_name);
GFRESULT gfCheckWGLExtension(const char *ext_name);

#endif

