//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinOGLWindow.cpp                                   //
//   Description: Wrapper for working with OpenGL window        //
//                                                              //
//////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gfWinOGLWindow.h"
#include "gfAuxLogging.h"
#include "gfAuxNumUtils.h"


gfWinOGLWindow* gfWinOGLWindow::m_wnd_creating;

LRESULT _stdcall gfWinOGLWindow::event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{	
    gfWinOGLWindow* acc_win = (gfWinOGLWindow*)::GetWindowLong(hwnd, GWL_USERDATA);
    if (!acc_win)
    {
        acc_win = m_wnd_creating;
        ::SetWindowLong(hwnd, GWL_USERDATA, (long) m_wnd_creating);
    }

    switch(msg)
	{
	case WM_CLOSE:
        //PostQuitMessage(0);
        return 0;
		break;
	case WM_SIZE:
		{
			if (acc_win)
                return acc_win->OnSize(hwnd, msg, wparam, lparam);
		}
		break;
    case WM_PAINT:
		{
			if (acc_win)
                return acc_win->OnPaint(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYDOWN:
		{
			if (acc_win)
                return acc_win->OnKeyDown(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYUP:
		{
			if (acc_win)
                return acc_win->OnKeyUp(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_MOUSEMOVE:
		{
			if (acc_win)
                return acc_win->OnMouseMove(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_LBUTTONDOWN:
		{
			if (acc_win)
                return acc_win->OnLButtonDown(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_RBUTTONDOWN:
		{
			if (acc_win)
                return acc_win->OnRButtonDown(hwnd, msg, wparam, lparam);
		}
		break;
    case WM_LBUTTONUP:
		{
			if (acc_win)
                return acc_win->OnLButtonUp(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_RBUTTONUP:
		{
			if (acc_win)
                return acc_win->OnRButtonUp(hwnd, msg, wparam, lparam);
		}
		break;
    case WM_SYSCOMMAND:	
		{
			if (wparam == SC_SCREENSAVE || wparam == SC_MONITORPOWER)						
			{	
				return 0;						
			}
			break;								
		}
	default: 
        break;
	}
	
	return DefWindowProc(hwnd, msg, wparam, lparam);
}


gfWinOGLWindow::gfWinOGLWindow(void)
{
    m_hWnd = NULL;;
    m_hRC = NULL;;
    m_hDC = NULL;;
    m_hInstance = NULL;
    m_current_view = NULL;

    m_keyboardfunc = NULL;
    m_mousemovefunc = NULL;
    m_mousedownfunc = NULL;
    m_mouseupfunc = NULL;
    m_renderfunc = NULL;
    m_reshapefunc = NULL;
}

gfWinOGLWindow::~gfWinOGLWindow(void)
{
    ClearViews();
}

LRESULT gfWinOGLWindow::OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    if(m_keyboardfunc)
	    m_keyboardfunc(wparam, GF_KEY_DOWN);

    return 0;
}

LRESULT gfWinOGLWindow::OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    if(m_keyboardfunc)
	    m_keyboardfunc(wparam, GF_KEY_UP);

    return 0;
}

LRESULT gfWinOGLWindow::OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return DefWindowProc(hwnd, msg, wparam, lparam);
}

LRESULT gfWinOGLWindow::OnSize(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt width = LOWORD(lparam);
    gfInt height = HIWORD(lparam);
    if (width * height == 0)
        return 0;

    gfInt num_views = (gfInt)m_views.size();
    // now fit this rectangle to new window
    gfDouble rx = width / (gfDouble)(m_width);
    gfDouble ry = height / (gfDouble)(m_height);

    for (gfInt i = 0;i < num_views;i++)
    {
        gfWinView* view = m_views.at(i);

        gfInt nleft, nwidth, nbottom, nheight;
        nleft = (view->m_left * width) / (gfDouble) m_width;
        nwidth = (view->m_width + view->m_left) * width / (gfDouble) m_width - nleft;
        nbottom = view->m_bottom * height / (gfDouble) m_height;
        nheight = (view->m_bottom + view->m_height) * height / (gfDouble) m_height - nbottom;

        view->m_left = nleft;
        view->m_width = nwidth;
        view->m_bottom = nbottom;
        view->m_height = nheight;
    }

    m_width = width;
    m_height = height;

    if (m_reshapefunc)
        m_reshapefunc(width, height);

    return 0;
}

LRESULT gfWinOGLWindow::OnMouseMove(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt x = LOWORD(lparam);
    gfInt y = m_height - HIWORD(lparam);

    if(m_mousemovefunc)
        m_mousemovefunc(x, y, wparam);

    return 0;
}

LRESULT gfWinOGLWindow::OnLButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt x = LOWORD(lparam);
    gfInt y = m_height - HIWORD(lparam);

    gfInt num_views = GetViewsCount();
    for (gfInt i = 0;i < num_views;i++)
    {
        gfWinView* view = GetView(i);
        if (!view)
            continue;
        if (view->IsPointInside(x, y))
        {
            SetCurrentView(view);
            break;
        }
    }

    if(m_mousedownfunc)
        m_mousedownfunc(x, y, GF_MOUSE_LBUTTON, wparam);

    return 0;
}

LRESULT gfWinOGLWindow::OnLButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt x = LOWORD(lparam);
    gfInt y = m_height - HIWORD(lparam);

    if(m_mouseupfunc)
        m_mouseupfunc(x, y, GF_MOUSE_LBUTTON, wparam);

    return 0;
}

LRESULT gfWinOGLWindow::OnRButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt x = LOWORD(lparam);
    gfInt y = m_height - HIWORD(lparam);

    gfInt num_views = GetViewsCount();
    for (gfInt i = 0;i < num_views;i++)
    {
        gfWinView* view = GetView(i);
        if (!view)
            continue;
        if (view->IsPointInside(x, y))
        {
            SetCurrentView(view);
            break;
        }
    }

    if(m_mousedownfunc)
        m_mousedownfunc(x, y, GF_MOUSE_RBUTTON, wparam);

    return 0;
}

LRESULT gfWinOGLWindow::OnRButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt x = LOWORD(lparam);
    gfInt y = m_height - HIWORD(lparam);

    if(m_mouseupfunc)
        m_mouseupfunc(x, y, GF_MOUSE_RBUTTON, wparam);

    return 0;
}

GFRESULT gfWinOGLWindow::CreateGLWindow(gfChar* title, int width, int height, int bits, HWND parent, HICON icon, HCURSOR cursor)
{
    gfInt		PixelFormat;
    WNDCLASS	wc;	
    DWORD		dwExStyle;
    DWORD		dwStyle;
    RECT		WindowRect;	
    WindowRect.left = (long)0;
    WindowRect.right = (long)width;
    WindowRect.top = (long)0;
    WindowRect.bottom = (long)height;

    // prepare window class
    m_hInstance			= GetModuleHandle(NULL);
    wc.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    wc.lpfnWndProc		= (WNDPROC) event;	
    wc.cbClsExtra		= 0;
    wc.cbWndExtra		= 0;
    wc.hInstance		= m_hInstance;
    wc.hIcon			= LoadIcon(NULL, IDI_WINLOGO);
    wc.hCursor			= LoadCursor(NULL, IDC_ARROW);
    wc.hbrBackground	= NULL;
    wc.lpszMenuName		= NULL;	
    wc.lpszClassName	= "OpenGL";	

    if (!RegisterClass(&wc))
    {
        gfLog("Failed to register window class\n.");
        return GF_E_FAIL;
    }

    if (parent)
    {
        dwExStyle = WS_EX_WINDOWEDGE;
        dwStyle = WS_CHILD | WS_DLGFRAME;
    }
    else
    {
        dwExStyle = WS_EX_APPWINDOW | WS_EX_WINDOWEDGE;
        dwStyle = WS_OVERLAPPEDWINDOW;
    }

    // adjust window to true requested size
    AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);

    // this is for catching this class in static window proc function
    m_wnd_creating = this;

    // create window
    if (!(m_hWnd = CreateWindowEx(dwExStyle,
        "OpenGL",
        title,
        dwStyle | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        0, 0,
        WindowRect.right-WindowRect.left,
        WindowRect.bottom-WindowRect.top,
        parent,			
        NULL,			
        m_hInstance,		
        NULL)))					
    {
        KillGLWindow();	
        gfLog("Window creation error.\n");
        return GF_E_FAIL;		
    }

    m_width = width;
    m_height = height;

    // pixel format descriptor
    static PIXELFORMATDESCRIPTOR pfd =
    {
        sizeof(PIXELFORMATDESCRIPTOR),
            1,
            PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
            PFD_TYPE_RGBA,
            bits,
            0, 0, 0, 0, 0, 0,
            0,
            0,
            0,
            0, 0, 0, 0,	
            16,	 
            0,	
            0,	
            PFD_MAIN_PLANE,	
            0,	
            0, 0, 0	
    };

    if (!(m_hDC = GetDC(m_hWnd)))
    {
        KillGLWindow();
        gfLog("Can't create OGL device context.\n");
        return GF_E_FAIL;
    }

    if (!(PixelFormat = ChoosePixelFormat(m_hDC, &pfd)))
    {
        KillGLWindow();
        gfLog("Can't find suitable PixelFormat for OGL window.\n");
        return GF_E_FAIL;
    }

    if(!SetPixelFormat(m_hDC, PixelFormat, &pfd))
    {
        KillGLWindow();
        gfLog("Can't set PixelFormat.\n");
        return GF_E_FAIL;
    }

    if (!(m_hRC = wglCreateContext(m_hDC)))
    {
        KillGLWindow();
        gfLog("Can't create OGL rendering context.\n");
        return GF_E_FAIL;
    }

    if(!wglMakeCurrent(m_hDC, m_hRC))
    {
        KillGLWindow();
        gfLog("Can't activate OGL rendering context.\n");
        return GF_E_FAIL;
    }

    ShowWindow(m_hWnd, SW_SHOW);				

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::KillGLWindow(void)
{
    if (m_hRC)
    {
        if (!wglMakeCurrent(NULL, NULL))
        {
            gfLog(NULL,"Release of DC and RC failed.");
        }
        // delete rendering context
        if (!wglDeleteContext(m_hRC))
        {
            gfLog("Release Rendering Context Failed.\n");
        }
        m_hRC = NULL;
    }

    if (m_hDC && !ReleaseDC(m_hWnd, m_hDC))
    {
        gfLog(NULL,"Release of device context failed.\n");
        m_hDC = NULL;
    }

    if (m_hWnd && !DestroyWindow(m_hWnd))
    {
        gfLog("Couldn't release OpenGL window handle.\n");
        m_hWnd = NULL;
    }

    if (!UnregisterClass("OpenGL", m_hInstance))
    {
        gfLog(NULL,"Could Not unregister OpenGL window class.\n");
        m_hInstance = NULL;
    }

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::CreateGLAreaByDC(HDC hdc, int color_bits, int depth_bits)
{
    gfInt PixelFormat;

    static PIXELFORMATDESCRIPTOR pfd =
    {
        sizeof(PIXELFORMATDESCRIPTOR),
            1 ,	
            PFD_DRAW_TO_WINDOW | PFD_SUPPORT_OPENGL | PFD_DOUBLEBUFFER,
            PFD_TYPE_RGBA,
            color_bits,
            0, 0, 0, 0, 0, 0,
            0,
            0,
            0,
            0, 0, 0, 0,
            depth_bits,
            0,
            0,
            PFD_MAIN_PLANE,
            0,
            0, 0, 0	
    };

    m_hDC = hdc;

    if (!(PixelFormat = ChoosePixelFormat(m_hDC, &pfd)))
    {
        KillGLWindow();	
        gfLog("Can't find suitable PixelFormat for OGL window.\n");
        return GF_E_FAIL;
    }

    if(!SetPixelFormat(m_hDC, PixelFormat, &pfd))
    {
        KillGLWindow();
        gfLog(NULL,"Can't set pixel format.\n");
        return GF_E_FAIL;
    }

    if (!(m_hRC = wglCreateContext(m_hDC)))
    {
        KillGLWindow();
        gfLog("Can't Create A GL Rendering Context.\n");
        return GF_E_FAIL;
    }

    // try to activate rendering context
    if(!wglMakeCurrent(m_hDC, m_hRC))
    {
        KillGLWindow();
        gfLog(NULL,"Can't activate GL rendering context.\n");
        return GF_E_FAIL;
    }

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::ReleaseContext(void)
{
    if (m_hRC)
    {
        if (!wglMakeCurrent(NULL, NULL))
        {
            gfLog("Release of DC and RC failed.\n");
        }

        if (!wglDeleteContext(m_hRC))
        {
            gfLog("Release rendering context failed.\n");
        }
        m_hRC = NULL;
    }

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::Redisplay(void)
{
    ::RedrawWindow(m_hWnd, NULL, NULL, RDW_UPDATENOW);

    return GF_S_OK;
}

HWND gfWinOGLWindow::GetHWND(void)
{
    return m_hWnd;
}

HDC gfWinOGLWindow::GetHDC(void)
{
    return m_hDC;
}

GFRESULT gfWinOGLWindow::MakeCurrent(void)
{
    // check if window is already current
    if (m_hDC == wglGetCurrentDC() && m_hRC == wglGetCurrentContext())
        return GF_S_OK;

    if(!wglMakeCurrent(m_hDC, m_hRC))
	{
		gfLog("Unable to change current context.\n");
		return GF_E_FAIL;
	}

	return GF_S_OK;
}

gfInt gfWinOGLWindow::GetWidth(void)
{
    return m_width;
}

gfInt gfWinOGLWindow::GetHeight(void)
{
    return m_height;
}

GFRESULT gfWinOGLWindow::Resize(gfInt width, gfInt height)
{
    SetWindowPos(m_hWnd, NULL, 0, 0, width, height, SWP_NOMOVE | SWP_SHOWWINDOW);

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::AddView(gfInt left, gfInt bottom, gfInt width, gfInt height, gfScnCamera* camera)
{
    if (!camera)
        return GF_E_BADARG;

    gfWinView* view = new gfWinView;

    view->m_bottom = bottom;
    view->m_left = left;
    view->m_width = width;
    view->m_height = height;
    view->m_camera = camera;

    m_views.push_back(view);
    m_current_view = view;

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::CreateDefault1View(gfScnCamera* camera)
{
    ClearViews();

    if (!camera)
        return GF_E_BADARG;

    gfWinView* view = new gfWinView;

    RECT rect;
    GetClientRect(m_hWnd, &rect);

    view->m_bottom = rect.top + GF_BORDER;
    view->m_left = rect.left + GF_BORDER;
    view->m_width = rect.right - rect.left - 2 * GF_BORDER;
    view->m_height = rect.bottom - rect.top  - 2 * GF_BORDER;
    view->m_camera = camera;

    camera->SetAspectRatio((gfDouble) view->m_width / (gfDouble) view->m_height);

    m_views.push_back(view);

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::CreateDefault4View(gfScnCamera* cam_front, gfScnCamera* cam_right, gfScnCamera* cam_top, gfScnCamera* cam_persp)
{
    ClearViews();

    if (!cam_front || !cam_right || !cam_top || !cam_persp)
        return GF_E_BADARG;

    gfWinView* view_front = new gfWinView;
    gfWinView* view_right = new gfWinView;
    gfWinView* view_top = new gfWinView;
    gfWinView* view_persp = new gfWinView;

    view_front->m_bottom = GF_BORDER;
    view_front->m_left = GF_BORDER;
    view_front->m_width = (m_width - 3 * GF_BORDER) / 2.0;
    view_front->m_height = (m_height - 3 * GF_BORDER) / 2.0;
    view_front->m_camera = cam_front;
    cam_front->SetAspectRatio((gfDouble) view_front->m_width / (gfDouble) view_front->m_height);

    view_right->m_bottom = GF_BORDER;
    view_right->m_left = (m_width + GF_BORDER) / 2.0;
    view_right->m_width = (m_width - 3 * GF_BORDER) / 2.0;
    view_right->m_height = (m_height - 3 * GF_BORDER) / 2.0;
    view_right->m_camera = cam_right;
    cam_right->SetAspectRatio((gfDouble) view_right->m_width / (gfDouble) view_right->m_height);

    view_top->m_bottom = (m_height + GF_BORDER) / 2.0;
    view_top->m_left = GF_BORDER;
    view_top->m_width = (m_width - 3 * GF_BORDER) / 2.0 ;
    view_top->m_height = (m_height - 3 * GF_BORDER) / 2.0;
    view_top->m_camera = cam_top;
    cam_top->SetAspectRatio((gfDouble) view_top->m_width / (gfDouble) view_top->m_height);

    view_persp->m_bottom = (m_height + GF_BORDER) / 2.0;
    view_persp->m_left = (m_width + GF_BORDER) / 2.0;
    view_persp->m_width = (m_width - 3 * GF_BORDER) / 2.0;
    view_persp->m_height = (m_height - 3 * GF_BORDER) / 2.0;
    view_persp->m_camera = cam_persp;
    cam_persp->SetAspectRatio((gfDouble) view_persp->m_width / (gfDouble) view_persp->m_height);

    m_views.push_back(view_front);
    m_views.push_back(view_right);
    m_views.push_back(view_top);
    m_views.push_back(view_persp);

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::ClearViews(void)
{
    gfInt num_views = (gfInt)m_views.size();
    for (gfInt i = 0;i < num_views;i++)
        delete m_views.at(i);

    m_views.clear();
    m_current_view = NULL;

    return GF_S_OK;
}

gfWinView* gfWinOGLWindow::GetViewByPoint(gfInt px, gfInt py)
{
    gfInt num_views = (gfInt)m_views.size();
    for (gfInt i = 0;i < num_views;i++)
    {
        gfWinView* view = m_views.at(i);

        if (view->IsPointInside(px, py))
            return view;
    }

    return NULL;
}

gfInt gfWinOGLWindow::GetViewsCount(void)
{
    return (gfInt)m_views.size();
}

gfWinView* gfWinOGLWindow::GetView(gfInt index)
{
    if (index < 0 || index >= (gfInt)m_views.size())
        return NULL;

    return m_views.at(index);
}

gfWinView* gfWinOGLWindow::GetCurrentView(void)
{
    return m_current_view;
}

GFRESULT gfWinOGLWindow::SetCurrentView(gfWinView* view)
{
    m_current_view = view;

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::DrawViewFrames(void)
{
    glViewport(0, 0, m_width, m_height);

    glMatrixMode(GL_PROJECTION);
    glLoadIdentity();
    gluOrtho2D(0.0, m_width, 0.0, m_height);

    glMatrixMode(GL_MODELVIEW);
    glLoadIdentity();

    gfInt num_views = (gfInt)m_views.size();
    gfWinView* view = NULL;
    for (gfInt i = 0;i < num_views;i++)
    {
        view = m_views.at(i);
        if (!view)
            continue;

        if (view == m_current_view)
        {
            glColor3f(1.0, 1.0, 0.0);
            glLineWidth(2.0);
        }
        else
        {
            glColor3f(0.0, 0.0, 0.0);
            glLineWidth(1.0);
        }
        
        glBegin(GL_LINE_LOOP);
        glVertex2d(view->m_left, view->m_bottom);
        glVertex2d(view->m_left + view->m_width, view->m_bottom);
        glVertex2d(view->m_left + view->m_width, view->m_bottom + view->m_height);
        glVertex2d(view->m_left, view->m_bottom + view->m_height);
        glEnd();

        //glRecti(view->m_left, view->m_bottom, view->m_left + view->m_width, view->m_bottom + view->m_height);
        glLineWidth(1.0);

        view = NULL;
    }

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::Get3DRayFromPoint(gfDouble px, gfDouble py, gfPoint3* ray_begin, gfVector3* ray_dir)
{
    // find view under point
    gfInt num_views = (gfInt)m_views.size();
    gfWinView* view = NULL;
    for (gfInt i = 0;i < num_views;i++)
    {
        view = m_views.at(i);
        if (view->IsPointInside(px, py))
            break;

        view = NULL;
    }

    if (!view)
        return GF_E_FAIL;

    view->SetOGLViewport();
    view->m_camera->SetOGLProjectionMatrix();
    view->m_camera->SetOGLViewMatrix();

    // retrieve these matrices and unproject point
    GLint viewport[4];
    GLdouble mvmatrix[16], projmatrix[16];
    glGetIntegerv(GL_VIEWPORT, viewport);
    glGetDoublev(GL_MODELVIEW_MATRIX, mvmatrix);
    glGetDoublev(GL_PROJECTION_MATRIX, projmatrix);

    //GLfloat winz;
    //glReadPixels(px, py, 1, 1, GL_DEPTH_COMPONENT, GL_FLOAT, &winz);

    GLdouble wx0, wy0, wz0;
    GLdouble wx1, wy1, wz1;
    gluUnProject((GLdouble) px, (GLdouble) py, 0.0, mvmatrix, projmatrix, viewport, &wx0, &wy0, &wz0);
    gluUnProject((GLdouble) px, (GLdouble) py, 1.0, mvmatrix, projmatrix, viewport, &wx1, &wy1, &wz1);

    ray_begin->Set(wx0, wy0, wz0);
    ray_dir->Set(wx1 - wx0, wy1 - wy0, wz1 - wz0);
    ray_dir->Normalize();

    return GF_S_OK;
}

GFRESULT gfWinOGLWindow::SaveToBmp(std::string str)
{
    BITMAPINFOHEADER BMIH;
    GLubyte* bits;

    gfInt width = 4*(m_width/4);
    gfInt height = 4*(m_height/4);

    memset(&BMIH, 0, sizeof(BITMAPINFOHEADER));
    BMIH.biSize = sizeof(BITMAPINFOHEADER);
    BMIH.biBitCount = 24;
    BMIH.biPlanes = 1;
    BMIH.biCompression = BI_RGB;
    BMIH.biWidth = width;
    BMIH.biHeight = height;
    BMIH.biSizeImage = ((((BMIH.biWidth * BMIH.biBitCount) + 31) & ~31) >> 3) * BMIH.biHeight;
    HBITMAP h_surface = CreateDIBSection(m_hDC, (CONST BITMAPINFO*)&BMIH, DIB_RGB_COLORS, (void**)&bits, NULL, 0);

    //bits = new GLubyte[3 * (m_width + 1) * (m_height + 1)];
    glReadPixels((m_width-width)/2, (m_height-height)/2, width, height, GL_RGB, GL_UNSIGNED_BYTE, bits);
    for (gfInt i = 0;i < width;i++)
        for (gfInt j = 0;j < height;j++)
        {
            GLubyte aux = bits[3*(i+j*width)];
            bits[3*(i+j*width)] = bits[3*(i+j*width)+2];
            bits[3*(i+j*width)+2] = aux;
        }

    //Create a new file for writing
    FILE *file = fopen(str.c_str(), "wb");
    if(file == NULL)
    {
        return GF_E_FAIL;
    }

    BITMAPFILEHEADER bmfh;
    memset(&bmfh, 0, sizeof(BITMAPFILEHEADER));
    int nBitsOffset = sizeof(BITMAPFILEHEADER) + BMIH.biSize; 
    LONG lImageSize = BMIH.biSizeImage;
    LONG lFileSize = nBitsOffset + lImageSize;
    bmfh.bfType = 0x4D42;
    bmfh.bfOffBits = nBitsOffset;
    bmfh.bfSize = lFileSize;
    bmfh.bfReserved1 = bmfh.bfReserved2 = 0;

    //Write the bitmap file header
    fwrite(&bmfh, 1, sizeof(BITMAPFILEHEADER), file);
    //And then the bitmap info header
    fwrite(&BMIH, 1, sizeof(BITMAPINFOHEADER), file);
    //Finally, write the image data itself 
    fwrite(bits, 1, lImageSize, file);

    fclose(file);
    DeleteObject(h_surface);

    return GF_S_OK;
}

void gfWinOGLWindow::KeyboardFunc(void(*keyboardx)(gfInt key, gfInt dir))
{
    m_keyboardfunc = keyboardx;
}

void gfWinOGLWindow::MouseDownFunc(void(*mousedownx)(gfInt x, gfInt y, gfInt button, gfInt special))
{
    m_mousedownfunc = mousedownx;
}

void gfWinOGLWindow::MouseUpFunc(void(*mouseupx)(gfInt x, gfInt y, gfInt button, gfInt special))
{
    m_mouseupfunc = mouseupx;
}

void gfWinOGLWindow::MouseMoveFunc(void(*mousemovex)(gfInt x, gfInt y, gfInt special))
{
    m_mousemovefunc = mousemovex;
}

void gfWinOGLWindow::RenderFunc(void(*renderx)(void))
{
    m_renderfunc = renderx;
}

void gfWinOGLWindow::ReshapeFunc(void(*reshapex)(gfInt width, gfInt height))
{
    m_reshapefunc = reshapex;
}