//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinOGLWindow.h                                     //
//   Description: Wrapper for working with OpenGL window        //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_OGL_WIN_H_
#define _GF_OGL_WIN_H_

#include <windows.h>
#include <stdio.h>
#include "gfPoint.h"
#include "gfVector.h"
#include "gfWinOGLExtensions.h"
#include "gfDefines.h"
#include <string>
#include <vector>
#include "gfWinView.h"


//LRESULT _stdcall event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);


class gfWinOGLWindow
{
private:
    HWND m_hWnd;
    HGLRC m_hRC;
    HDC m_hDC;
    HINSTANCE  m_hInstance;
    gfInt m_width, m_height;

    // views
    std::vector<gfWinView*> m_views;
    gfWinView* m_current_view;

    // window proc
    static LRESULT CALLBACK event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    static gfWinOGLWindow* m_wnd_creating;

    // callback functions
    void (*m_keyboardfunc)(gfInt key, gfInt dir);
    void (*m_mousemovefunc)(gfInt x, gfInt y, gfInt special);
    void (*m_mousedownfunc)(gfInt x, gfInt y, gfInt button, gfInt special);
    void (*m_mouseupfunc)(gfInt x, gfInt y, gfInt button, gfInt special);
    void (*m_renderfunc)(void);
    void (*m_reshapefunc)(gfInt width, gfInt height);

    // messages to window
    LRESULT OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnSize(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnMouseMove(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnLButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnLButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnRButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnRButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);

public:
    gfWinOGLWindow(void);
    ~gfWinOGLWindow(void);

	GFRESULT CreateGLWindow(gfChar* title, int width, int height, int bits, HWND parent, HICON icon, HCURSOR cursor);
    GFRESULT KillGLWindow(void);
    GFRESULT CreateGLAreaByDC(HDC hdc, int color_bits, int depth_bits);
    GFRESULT ReleaseContext(void);
    GFRESULT Redisplay(void);
    GFRESULT Resize(gfInt width, gfInt height);

    HWND GetHWND(void);
    HDC GetHDC(void);
    GFRESULT MakeCurrent(void);
    gfInt GetWidth(void);
    gfInt GetHeight(void);

    // view management
    GFRESULT AddView(gfInt left, gfInt bottom, gfInt width, gfInt height, gfScnCamera* camera);
    GFRESULT CreateDefault1View(gfScnCamera* camera);
    GFRESULT CreateDefault4View(gfScnCamera* cam_front, gfScnCamera* cam_right, gfScnCamera* cam_top, gfScnCamera* cam_persp);
    GFRESULT ClearViews(void);
    gfWinView* GetViewByPoint(gfInt px, gfInt py);
    gfInt GetViewsCount(void);
    gfWinView* GetView(gfInt index);
    gfWinView* GetCurrentView(void);
    GFRESULT SetCurrentView(gfWinView* view);
    GFRESULT DrawViewFrames(void);

    // 
    GFRESULT Get3DRayFromPoint(gfDouble px, gfDouble py, gfPoint3* ray_begin, gfVector3* ray_dir);

    // save to file
    GFRESULT SaveToBmp(std::string str);

    // init callbacks
    void KeyboardFunc(void(*keyboardx)(gfInt key, gfInt dir));
    void MouseDownFunc(void(*mousedownx)(gfInt x, gfInt y, gfInt button, gfInt special));
    void MouseUpFunc(void(*mouseupx)(gfInt x, gfInt y, gfInt button, gfInt special));
    void MouseMoveFunc(void(*mousemovex)(gfInt x, gfInt y, gfInt special));
    void RenderFunc(void(*renderx)(void));
    void ReshapeFunc(void(*reshapex)(gfInt width, gfInt height));
};

#endif