//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinRaytrace.h                                      //
//   Description: Window for showing raytraced scene            //
//                                                              //
//////////////////////////////////////////////////////////////////


#include "stdafx.h"
#include "resource.h"
#include "gfWinRaytrace.h"
#include "gfAuxLogging.h"

#include "OGL/glut.h"


LRESULT _stdcall gfWinRaytrace::event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{	
    //return DefWindowProc(hwnd, msg, wparam, lparam);
    gfWinRaytrace* acc_win = (gfWinRaytrace*)::GetWindowLong(hwnd, GWL_USERDATA);

    switch(msg)
	{
    case WM_INITDIALOG:
        {
            gfWinRaytrace* acc_win = (gfWinRaytrace*)lparam;
            ::SetWindowLong(hwnd, GWL_USERDATA, (long) acc_win);
            if (acc_win)
            {
                acc_win->m_hWnd = hwnd;
                return acc_win->OnInitDialog(hwnd, msg, wparam, lparam);
            }
        }
        break;
    case WM_COMMAND:
        {
            if (acc_win)
                return acc_win->OnCommand(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_MOVE:
        {
            if (acc_win)
                return acc_win->OnMove(hwnd, msg, wparam, lparam);
        }
        break;
    case WM_PAINT:
		{
			if (acc_win)
                return acc_win->OnPaint(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYDOWN:
		{
			if (acc_win)
                return acc_win->OnKeyDown(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYUP:
		{
			if (acc_win)
                return acc_win->OnKeyUp(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_LBUTTONDBLCLK:
		{
			if (acc_win)
                return acc_win->OnLButtonDblClk(hwnd, msg, wparam, lparam);
		}
		break;
	default: 
        break;
	}
	
	return 0;
}

gfWinRaytrace::gfWinRaytrace(void)
{
    m_raytraced = false;
    m_width = m_height = 200;
    m_ratio_progress = false;
    m_shadows = true;
    m_interleaved = true;
    m_bezier_clipping = true;
    m_aa = GF_AA_ADAPTIVE;
}

gfWinRaytrace::~gfWinRaytrace(void)
{
    if (m_hWnd)
        DestroyWindow(m_hWnd);
}

LRESULT gfWinRaytrace::OnInitDialog(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    m_hWnd = hwnd;

    // set edit boxes
    SetDlgItemInt(m_hWnd, IDC_EDIT_WIDTH, m_width, false);
    SetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, m_height, false);
    SetDlgItemInt(m_hWnd, IDC_EDIT_DEPTH, 1, false);
    

    if (m_shadows)
        CheckDlgButton(m_hWnd, IDC_CHECK_SHADOWS, BST_CHECKED);
    else
        CheckDlgButton(m_hWnd, IDC_CHECK_SHADOWS, BST_UNCHECKED);

    if (m_interleaved)
        CheckDlgButton(m_hWnd, IDC_CHECK_INTERLEAVED, BST_CHECKED);
    else
        CheckDlgButton(m_hWnd, IDC_CHECK_INTERLEAVED, BST_UNCHECKED);

    if (m_aa == GF_AA_ADAPTIVE)
        CheckDlgButton(m_hWnd, IDC_CHECK_ADAPTIVE_AA, BST_CHECKED);
    else
        CheckDlgButton(m_hWnd, IDC_CHECK_ADAPTIVE_AA, BST_UNCHECKED);

    if (m_bezier_clipping)
        CheckDlgButton(m_hWnd, IDC_CHECK_BEZIER_CLIPPING, BST_CHECKED);
    else
        CheckDlgButton(m_hWnd, IDC_CHECK_BEZIER_CLIPPING, BST_UNCHECKED);

    if (m_raytracer)
    {
        CheckDlgButton(m_hWnd, IDC_CHECK_RATIO, BST_CHECKED);
        m_ratio_progress = true;
        SetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, m_width / m_raytracer->GetCameraRatio(), false);
        m_ratio_progress = false;
    }

    // set resolution of window
    gfInt win_height;
    if (14 + m_height + 30 < 340)
        win_height = 340;
    else
        win_height = 14 + m_height + 30;
    SetWindowPos(m_hWnd, NULL, 0, 0, 230 + m_width + 14, win_height, SWP_SHOWWINDOW);
    //HWND disp_button = GetDlgItem(m_hWnd, IDC_BUTTON_WND);
    //MoveWindow(disp_button, 230, 7, 200, 200, true);

    return DefWindowProc(hwnd, msg, wparam, lparam);
}
LRESULT gfWinRaytrace::OnCommand(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    switch (LOWORD(wparam)) 
    { 
    case IDCLOSE: 
        //DestroyWindow(m_hWnd);
        m_width = GetDlgItemInt(m_hWnd, IDC_EDIT_WIDTH, NULL, false);
        m_height = GetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, NULL, false);
        EndDialog(m_hWnd, 1);
        m_hWnd = NULL;
        return 1; 
    case IDC_RENDER_BUTTON:
        {
            gfInt width = GetDlgItemInt(m_hWnd, IDC_EDIT_WIDTH, NULL, false);
            gfInt height = GetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, NULL, false);
            gfInt depth = GetDlgItemInt(m_hWnd, IDC_EDIT_DEPTH, NULL, false);

            // set resolution of dialog
            gfInt win_height;
            if (14 + height + 30 < 340)
                win_height = 340;
            else
                win_height = 14 + height + 30;
            SetWindowPos(m_hWnd, NULL, 0, 0, 230 + width + 14, win_height, SWP_NOMOVE);

            m_raytracer->SetResolution(width, height);
            m_raytracer->SetShadowsRendering(m_shadows);
            m_raytracer->SetInterleaved(m_interleaved);
            m_raytracer->SetAAType(m_aa);
            m_raytracer->SetBezierClipping(m_bezier_clipping);
            m_raytracer->Raytrace(depth);

            m_raytraced = true;
            RedrawWindow(m_hWnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT);

            return 0;
        }
    case IDC_SAVE_BUTTON:
        {
            if (!m_raytraced)
                return 0;

            OPENFILENAME ofn;       // common dialog box structure
            char szFile[260];       // buffer for file name

            // Initialize OPENFILENAME
            ZeroMemory(&ofn, sizeof(ofn));
            ofn.lStructSize = sizeof(ofn);
            ofn.hwndOwner = m_hWnd;
            ofn.lpstrFile = szFile;
            ofn.lpstrFile[0] = '\0';
            ofn.nMaxFile = sizeof(szFile);
            ofn.lpstrFilter = "All\0*.*\0Bmp\0*.bmp\0";
            ofn.nFilterIndex = 1;
            ofn.lpstrFileTitle = NULL;
            ofn.nMaxFileTitle = 0;
            ofn.lpstrInitialDir = NULL;
            ofn.Flags = OFN_SHOWHELP | OFN_OVERWRITEPROMPT;

            // Display the Open dialog box. 
            if (GetSaveFileName(&ofn)) 
            {
                std::string str;
                str.append(ofn.lpstrFile);
                SaveToBmp(str);
            }

            return 0;
        }
    case IDC_EDIT_WIDTH:
        {
            if (m_ratio_progress)
                return 0;
            m_ratio_progress = true;
            gfInt width = GetDlgItemInt(m_hWnd, IDC_EDIT_WIDTH, NULL, false);
            if (IsDlgButtonChecked(m_hWnd, IDC_CHECK_RATIO))
                SetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, width / m_raytracer->GetCameraRatio(), false);
            m_ratio_progress = false;
            return 0;
        }
    case IDC_EDIT_HEIGHT:
        {
            if (m_ratio_progress)
                return 0;
            m_ratio_progress = true;
            gfInt height = GetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, NULL, false);
            if (IsDlgButtonChecked(m_hWnd, IDC_CHECK_RATIO))
                SetDlgItemInt(m_hWnd, IDC_EDIT_WIDTH, height * m_raytracer->GetCameraRatio(), false);
            m_ratio_progress = false;
            return 0;
        }
    case IDC_CHECK_SHADOWS:
        {
            m_shadows = !m_shadows;
            return 0;
        }
    case IDC_CHECK_INTERLEAVED:
        {
            m_interleaved = !m_interleaved;
            return 0;
        }
    case IDC_CHECK_ADAPTIVE_AA:
        {
            if (m_aa == GF_AA_NONE)
                m_aa = GF_AA_ADAPTIVE;
            else
                m_aa = GF_AA_NONE;
            return 0;
        }
    case IDC_CHECK_BEZIER_CLIPPING:
        {
            m_bezier_clipping = !m_bezier_clipping;
            return 0;
        }
    }

    return 0;
}

LRESULT gfWinRaytrace::OnMove(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinRaytrace::OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinRaytrace::OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinRaytrace::OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    if (m_raytraced)
    {
        gfInt width = GetDlgItemInt(m_hWnd, IDC_EDIT_WIDTH, NULL, false);
        gfInt height = GetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, NULL, false);
        HWND disp_button = GetDlgItem(m_hWnd, IDC_BUTTON_WND);
        HDC dc = GetDC(m_hWnd);

        for (gfInt i = 0;i < width;i++)
            for (gfInt j = 0;j < height;j++)
            {
                gfVector3 color;
                m_raytracer->GetPixel(i, j, &color);
                ::SetPixel(dc, i + 230, height - j + 14, RGB(255*color.X(), 255*color.Y(), 255*color.Z()));
            }
    }

    return 0;
}

LRESULT gfWinRaytrace::OnLButtonDblClk(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

GFRESULT gfWinRaytrace::CreateRaytraceDialog(gfChar* title, gfAuxRaytracer* raytracer, HWND parent)
{
    if (m_hWnd)
        DestroyWindow(m_hWnd);
    m_raytracer = raytracer;

    DialogBoxParam(GetModuleHandle(NULL), MAKEINTRESOURCE(IDD_RAYTRACE_DLG), parent, (DLGPROC)event, (LPARAM)this);
    m_hWnd = NULL;

    return GF_S_OK;
}

GFRESULT gfWinRaytrace::SaveToBmp(std::string str)
{
    if (!m_raytraced)
        return GF_S_OK;

    HDC dc = GetDC(GetDlgItem(m_hWnd, IDC_BUTTON_WND));
    gfInt width = GetDlgItemInt(m_hWnd, IDC_EDIT_WIDTH, NULL, false);
    gfInt height = GetDlgItemInt(m_hWnd, IDC_EDIT_HEIGHT, NULL, false);

    BITMAPINFOHEADER BMIH;
    GLubyte* bits;

    memset(&BMIH, 0, sizeof(BITMAPINFOHEADER));
    BMIH.biSize = sizeof(BITMAPINFOHEADER);
    BMIH.biBitCount = 24;
    BMIH.biPlanes = 1;
    BMIH.biCompression = BI_RGB;
    BMIH.biWidth = width;
    BMIH.biHeight = height;
    BMIH.biSizeImage = ((((BMIH.biWidth * BMIH.biBitCount) + 31) & ~31) >> 3) * BMIH.biHeight;
    HBITMAP h_surface = CreateDIBSection(dc, (CONST BITMAPINFO*)&BMIH, DIB_RGB_COLORS, (void**)&bits, NULL, 0);

    for (gfInt i = 0;i < width;i++)
        for (gfInt j = 0;j < height;j++)
        {
            gfVector3 color;
            m_raytracer->GetPixel(i, j, &color);
            bits[3*(i+j*width)] = 255*color.Z();
            bits[3*(i+j*width)+1] = 255*color.Y();
            bits[3*(i+j*width)+2] = 255*color.X();
        }

    //Create a new file for writing
    FILE *file = fopen(str.c_str(), "wb");
    if(file == NULL)
    {
        return GF_E_FAIL;
    }

    BITMAPFILEHEADER bmfh;
    memset(&bmfh, 0, sizeof(BITMAPFILEHEADER));
    int nBitsOffset = sizeof(BITMAPFILEHEADER) + BMIH.biSize; 
    LONG lImageSize = BMIH.biSizeImage;
    LONG lFileSize = nBitsOffset + lImageSize;
    bmfh.bfType = 0x4D42;
    bmfh.bfOffBits = nBitsOffset;
    bmfh.bfSize = lFileSize;
    bmfh.bfReserved1 = bmfh.bfReserved2 = 0;

    //Write the bitmap file header
    fwrite(&bmfh, 1, sizeof(BITMAPFILEHEADER), file);
    //And then the bitmap info header
    fwrite(&BMIH, 1, sizeof(BITMAPINFOHEADER), file);
    //Finally, write the image data itself 
    fwrite(bits, 1, lImageSize, file);

    fclose(file);
    DeleteObject(h_surface);

    return GF_S_OK;
}
