//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinToolbar.cpp                                     //
//   Description: Wrapper for working toolbar                   //
//                                                              //
//////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "gfWinToolbar.h"
#include "gfAuxLogging.h"
#include "gfAuxNumUtils.h"
#include "gfObjectFactory.h"


WNDCLASS gfWinToolbar::m_wnd_class;
gfWinToolbar* gfWinToolbar::m_wnd_creating;

LRESULT _stdcall gfWinToolbar::event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{	
    gfWinToolbar* acc_win = (gfWinToolbar*)::GetWindowLong(hwnd, GWL_USERDATA);
    if (!acc_win)
    {
        acc_win = m_wnd_creating;
        ::SetWindowLong(hwnd, GWL_USERDATA, (long) m_wnd_creating);
    }

    switch(msg)
	{
	case WM_CLOSE:
        //PostQuitMessage(0);
        return 0;
		break;
	case WM_SIZE:
		{
			if (acc_win)
                return acc_win->OnSize(hwnd, msg, wparam, lparam);
		}
		break;
    case WM_PAINT:
		{
			if (acc_win)
                return acc_win->OnPaint(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYDOWN:
		{
			if (acc_win)
                return acc_win->OnKeyDown(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_KEYUP:
		{
			if (acc_win)
                return acc_win->OnKeyUp(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_MOUSEMOVE:
		{
			if (acc_win)
                return acc_win->OnMouseMove(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_LBUTTONDOWN:
		{
			if (acc_win)
                return acc_win->OnLButtonDown(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_RBUTTONDOWN:
		{
			if (acc_win)
                return acc_win->OnRButtonDown(hwnd, msg, wparam, lparam);
		}
		break;
    case WM_LBUTTONUP:
		{
			if (acc_win)
                return acc_win->OnLButtonUp(hwnd, msg, wparam, lparam);
		}
		break;
	case WM_RBUTTONUP:
		{
			if (acc_win)
                return acc_win->OnRButtonUp(hwnd, msg, wparam, lparam);
		}
		break;
	default: 
        break;
	}
	
	return DefWindowProc(hwnd, msg, wparam, lparam);
}


gfWinToolbar::gfWinToolbar(void)
{
    m_hwnd = NULL;
    m_hinstance = NULL;
    m_radio = true;
    m_horizontal = true;
    m_current_tool = -1;
    m_lmdown = false;

    m_notify_function = NULL;
}

gfWinToolbar::~gfWinToolbar(void)
{
    DestroyToolbar();
}

LRESULT gfWinToolbar::OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinToolbar::OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinToolbar::OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    gfInt num_tools = (gfInt)m_tool_bitmaps.size();
    RECT rc;
    GetClientRect(m_hwnd, &rc);
    gfInt width = rc.right - rc.left;
    gfInt height = rc.bottom - rc.top;
    if (num_tools <= 0 || width*height == 0)
        return DefWindowProc(hwnd, msg, wparam, lparam);

    PAINTSTRUCT ps;
    BeginPaint(m_hwnd, &ps);
    COLORREF button_color = RGB(200, 200, 200);

    if (m_horizontal)
    {
        for (gfInt i = 0;i < num_tools;i++)
        {
            gfDouble item_size = width / (gfDouble)num_tools;
            RECT rc;
            rc.left = i * item_size;
            rc.top = 0;
            rc.right = (i + 1) * item_size;
            rc.bottom = item_size;

            // draw rectangle around bitmap
            if (i == m_current_tool && m_radio)
                DrawButtonBorder(ps.hdc, rc, button_color, 2, false);
            else
                DrawButtonBorder(ps.hdc, rc, button_color, 2, true);

            // draw bitmap
            DrawState(ps.hdc, NULL, NULL, (LPARAM)m_tool_bitmaps.at(i), 0, rc.left+2, rc.top+2, rc.right-2, rc.bottom-2, DSS_NORMAL|DST_BITMAP);
        }
    }
    else
    {
        for (gfInt i = 0;i < num_tools;i++)
        {
            gfDouble item_size = height / (gfDouble)num_tools;
            RECT rc;
            rc.left = 0;
            rc.top = i * item_size;
            rc.right = item_size;
            rc.bottom = (i + 1) * item_size;

            // draw rectangle around bitmap
            if (i == m_current_tool && m_radio)
                DrawButtonBorder(ps.hdc, rc, button_color, 2, false);
            else
                DrawButtonBorder(ps.hdc, rc, button_color, 2, true);

            // draw bitmap
            DrawState(ps.hdc, NULL, NULL, (LPARAM)m_tool_bitmaps.at(i), 0, rc.left+2, rc.top+2, rc.right-2, rc.bottom-2, DSS_NORMAL|DST_BITMAP);
        }
    }

    EndPaint(m_hwnd, &ps);

    return 0;
}

LRESULT gfWinToolbar::OnSize(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinToolbar::OnMouseMove(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    if (!m_lmdown)
        return DefWindowProc(hwnd, msg, wparam, lparam);
    gfInt num_tools = (gfInt)m_tool_bitmaps.size();
    RECT rc;
    GetClientRect(m_hwnd, &rc);
    gfInt width = rc.right - rc.left;
    gfInt height = rc.bottom - rc.top;
    if (num_tools <= 0 || width*height == 0)
        return DefWindowProc(hwnd, msg, wparam, lparam);

    gfInt x = LOWORD(lparam);
    gfInt y = HIWORD(lparam);
    m_current_tool = -1;
    if (m_horizontal)
    {
        gfDouble item_size = width / (gfDouble)num_tools;
        if (y >= 0 && y <= height)
            m_current_tool = x / item_size;
        RedrawWindow(m_hwnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    }
    else
    {
        gfDouble item_size = height / (gfDouble)num_tools;
        if (x >= 0 && x <= width)
            m_current_tool = y / item_size;
        RedrawWindow(m_hwnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    }

    return 0;
}

LRESULT gfWinToolbar::OnLButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    m_lmdown = true;
    m_old_capture = SetCapture(m_hwnd);

    gfInt num_tools = (gfInt)m_tool_bitmaps.size();
    RECT rc;
    GetClientRect(m_hwnd, &rc);
    gfInt width = rc.right - rc.left;
    gfInt height = rc.bottom - rc.top;
    if (num_tools <= 0 || width*height == 0)
        return DefWindowProc(hwnd, msg, wparam, lparam);

    gfInt x = LOWORD(lparam);
    gfInt y = HIWORD(lparam);
    m_current_tool = -1;
    if (m_horizontal)
    {
        gfDouble item_size = width / (gfDouble)num_tools;
        if (y >= 0 && y <= height)
            m_current_tool = x / item_size;
        RedrawWindow(m_hwnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    }
    else
    {
        gfDouble item_size = height / (gfDouble)num_tools;
        if (x >= 0 && x <= width)
            m_current_tool = y / item_size;
        RedrawWindow(m_hwnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    }

    return 0;
}

LRESULT gfWinToolbar::OnLButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    m_lmdown = false;
    SetCapture(m_old_capture);

    gfInt num_tools = (gfInt)m_tool_bitmaps.size();
    RECT rc;
    GetClientRect(m_hwnd, &rc);
    gfInt width = rc.right - rc.left;
    gfInt height = rc.bottom - rc.top;
    if (num_tools <= 0 || width*height == 0)
        return DefWindowProc(hwnd, msg, wparam, lparam);

    gfInt x = LOWORD(lparam);
    gfInt y = HIWORD(lparam);
    m_current_tool = -1;
    if (m_horizontal)
    {
        gfDouble item_size = width / (gfDouble)num_tools;
        gfInt tool = x / item_size;

        if (y >= 0 && y <= height)
        {
            if (m_notify_function)
                m_notify_function(tool);

            if (m_radio)
                m_current_tool = tool;
        }
        RedrawWindow(m_hwnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    }
    else
    {
        gfDouble item_size = height / (gfDouble)num_tools;
        gfInt tool = y / item_size;

        if (x >= 0 && x <= width)
        {
            if (m_notify_function)
                m_notify_function(tool);

            if (m_radio)
                m_current_tool = tool;
        }
        RedrawWindow(m_hwnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    }

    return 0;
}

LRESULT gfWinToolbar::OnRButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

LRESULT gfWinToolbar::OnRButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam)
{
    return 0;
}

GFRESULT gfWinToolbar::RegisterWindowClass(HINSTANCE instance)
{
    // prepare window class
    m_hinstance = instance;
    m_wnd_class.style			= CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
    m_wnd_class.lpfnWndProc		= (WNDPROC) event;	
    m_wnd_class.cbClsExtra		= 0;
    m_wnd_class.cbWndExtra		= 0;
    m_wnd_class.hInstance		= instance;
    m_wnd_class.hIcon			= LoadIcon(NULL, IDI_WINLOGO);
    m_wnd_class.hCursor			= LoadCursor(NULL, IDC_ARROW);
    m_wnd_class.hbrBackground	= NULL;
    m_wnd_class.lpszMenuName		= NULL;	
    m_wnd_class.lpszClassName	= "GFToolbar";	

    if (!RegisterClass(&m_wnd_class))
    {
        gfLog("Failed to register toolbar window class\n.");
        return GF_E_FAIL;
    }

    return GF_S_OK;
}

GFRESULT gfWinToolbar::UnregisterWindowClass(void)
{
    if (!UnregisterClass("GFToolbar", m_hinstance))
    {
        gfLog(NULL,"Could not unregister toolbar window class.\n");
        m_hinstance = NULL;
        return GF_E_FAIL;
    }
    return GF_S_OK;
}

GFRESULT gfWinToolbar::CreateToolbar(HINSTANCE instance, gfBool horizontal, gfBool radio, gfChar* title, HWND parent)
{
    DWORD		dwExStyle;
    DWORD		dwStyle;
    RECT		WindowRect;	
    WindowRect.left = (long)0;
    WindowRect.right = (long)0;
    WindowRect.top = (long)0;
    WindowRect.bottom = (long)0;

    if (parent)
    {
        dwExStyle = NULL;//WS_EX_WINDOWEDGE;
        dwStyle = WS_CHILD;// | WS_DLGFRAME;
    }

    // adjust window to true requested size
    AdjustWindowRectEx(&WindowRect, dwStyle, FALSE, dwExStyle);

    // create window
    // this is for catching this class in static window proc function
    m_wnd_creating = this;
    if (!(m_hwnd = CreateWindowEx(dwExStyle,
        "GFToolbar",
        title,
        dwStyle | WS_CLIPSIBLINGS | WS_CLIPCHILDREN,
        0, 0,
        WindowRect.right-WindowRect.left,
        WindowRect.bottom-WindowRect.top,
        parent,			
        NULL,			
        instance,		
        NULL)))					
    {
        gfLog("Toolbar window creation error.\n");
        return GF_E_FAIL;		
    }

    m_radio = radio;
    m_horizontal = horizontal;
    m_current_tool = -1;

    return GF_S_OK;
}

GFRESULT gfWinToolbar::DestroyToolbar(void)
{
    if (!m_hwnd)
        return GF_S_OK;

    // release all bitmaps
    //HDC dc = GetDC();
    gfInt num_tools = (gfInt)m_tool_bitmaps.size();
    for (gfInt i = 0;i < num_tools;i++)
    {
        DeleteObject(m_tool_bitmaps.at(i));
        //gfDeleteString(m_tool_names.at(i));
    }

    m_tool_names.clear();

    if (!DestroyWindow(m_hwnd))
    {
        gfLog("Failed to destroy toolbar window.\n");
        m_hwnd = NULL;
        return GF_E_FAIL;
    }

    m_hwnd = NULL;
    return GF_S_OK;
}

GFRESULT gfWinToolbar::AddTool(gfChar* name, gfInt resource)
{
    HBITMAP bmp = LoadBitmap(m_hinstance, MAKEINTRESOURCE(resource));
    if (!bmp)
    {
        gfLog("Failed to load tool %s bitmap from resource.\n", name);
        return GF_E_FAIL;
    }

    std::string str(name);
    
    m_tool_bitmaps.push_back(bmp);
    m_tool_names.push_back(str);

    return GF_S_OK;
}

HWND gfWinToolbar::GetHWND(void)
{
    return m_hwnd;
}

gfBool gfWinToolbar::IsHorizontal(void)
{
    return m_horizontal;
}

gfBool gfWinToolbar::IsRadio(void)
{
    return m_radio;
}

GFRESULT gfWinToolbar::SetCurrentTool(gfInt tool)
{
    if (m_radio && tool >= 0 && tool < (gfInt)m_tool_bitmaps.size())
    {
        m_current_tool = tool;
        RedrawWindow(m_hwnd, NULL, NULL, RDW_UPDATENOW | RDW_INTERNALPAINT | RDW_ERASE | RDW_INVALIDATE);
    }

    return GF_S_OK;
}

gfInt gfWinToolbar::GetNumTools()
{
    return (gfInt)m_tool_bitmaps.size();
}

void gfWinToolbar::SetNotifyFunc(void(*notifyx)(gfInt tool))
{
    m_notify_function = notifyx;
}

void gfWinToolbar::DrawButtonBorder(HDC hDC, RECT& rect, COLORREF clr_face, gfInt border, gfBool raisen)
{
    HPEN    hOldPen;
    HPEN    hPen,hPrevPen;

    // light parts - left and top edges
    hPen = (HPEN)CreatePen(PS_SOLID, 1, (raisen) ? GetLightColor(clr_face) : GetDarkColor(clr_face));
    hOldPen = (HPEN)SelectObject(hDC, hPen);
    MoveToEx(hDC, rect.left, rect.bottom, NULL);
    LineTo(hDC, rect.left, rect.top);
    LineTo(hDC, rect.right, rect.top);
    hPen = (HPEN)CreatePen(PS_SOLID, 1, (raisen) ? GetHighLightColor(clr_face) : GetShadowColor(clr_face));
    hPrevPen = (HPEN)SelectObject(hDC, hPen);
    DeleteObject(hPrevPen);
    MoveToEx(hDC, rect.left + 1, rect.bottom - 3, NULL);
    LineTo(hDC, rect.left + 1, rect.top + 1);
    LineTo(hDC, rect.right - 1, rect.top + 1);
    // dark parts - bottom and right edges
    hPen = (HPEN)CreatePen(PS_SOLID, 1, (raisen) ? GetShadowColor(clr_face) : GetHighLightColor(clr_face));
    hPrevPen = (HPEN)SelectObject(hDC, hPen);
    DeleteObject(hPrevPen);
    MoveToEx(hDC, rect.left + 1, rect.bottom - 2, NULL);
    LineTo(hDC, rect.right - 2, rect.bottom - 2);
    LineTo(hDC, rect.right - 2, rect.top + 1);
    hPen = (HPEN)CreatePen(PS_SOLID, 1, (raisen) ? GetDarkColor(clr_face) : GetLightColor(clr_face));
    hPrevPen = (HPEN)SelectObject(hDC, hPen);
    DeleteObject(hPrevPen);
    MoveToEx(hDC, rect.left, rect.bottom - 1, NULL);
    LineTo(hDC, rect.right - 1, rect.bottom - 1);
    LineTo(hDC, rect.right - 1, rect.top);
    // restore old object
    hPrevPen = (HPEN)SelectObject(hDC, hOldPen);
    // delete unused object
    DeleteObject(hPrevPen);
}

COLORREF gfWinToolbar::ModifyColor(COLORREF clrFill, short sDelta)
{
	gfInt r = GetRValue(clrFill);
	gfInt g = GetGValue(clrFill);
	gfInt b = GetBValue(clrFill);

	r = gfClamp(0, 255, r + sDelta);
	g = gfClamp(0, 255, g + sDelta);
	b = gfClamp(0, 255, b + sDelta);

	return RGB(r, g, b);
}

COLORREF gfWinToolbar::GetHighLightColor(COLORREF clrFill)
{
    return ModifyColor(clrFill, 64);
}

COLORREF gfWinToolbar::GetLightColor(COLORREF clrFill)
{
    return ModifyColor(clrFill, 32);
}

COLORREF gfWinToolbar::GetShadowColor(COLORREF clrFill)
{
    return ModifyColor(clrFill, -32);
}

COLORREF gfWinToolbar::GetDarkColor(COLORREF clrFill)
{
    return ModifyColor(clrFill, -64);
}

