//////////////////////////////////////////////////////////////////
//                                                              //
//   Project: GeomForge                                         //
//   Author: Martin Samuelcik                                   //
//   File: gfWinToolbar.h                                       //
//   Description: Wrapper for working toolbar                   //
//                                                              //
//////////////////////////////////////////////////////////////////


#ifndef _GF_WIN_TOOLBAR_H_
#define _GF_WIN_TOOLBAR_H_

#include <windows.h>
#include <stdio.h>
#include "gfPoint.h"
#include "gfVector.h"
#include "gfWinOGLExtensions.h"
#include "gfDefines.h"
#include <string>
#include <vector>
#include "gfWinView.h"


//LRESULT _stdcall event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);


class gfWinToolbar
{
private:
    HINSTANCE m_hinstance;
    HWND m_hwnd;
    std::vector<HBITMAP> m_tool_bitmaps;
    std::vector<std::string> m_tool_names;
    gfBool m_horizontal;
    gfBool m_radio;
    gfInt m_current_tool;
    gfBool m_lmdown;
    HWND m_old_capture;

    // window proc
    static WNDCLASS m_wnd_class;
    static gfWinToolbar* m_wnd_creating;
    static LRESULT CALLBACK event(struct HWND__* hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);

    // callback functions
    void (*m_notify_function)(gfInt tool);

    // messages to window
    LRESULT OnKeyDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnKeyUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnPaint(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnSize(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnMouseMove(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnLButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnLButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnRButtonDown(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);
    LRESULT OnRButtonUp(HWND hwnd, unsigned int msg, unsigned int wparam, unsigned long lparam);

public:
    gfWinToolbar(void);
    ~gfWinToolbar(void);

    GFRESULT RegisterWindowClass(HINSTANCE instance);
    GFRESULT UnregisterWindowClass(void);
	GFRESULT CreateToolbar(HINSTANCE instance, gfBool horizontal, gfBool radio, gfChar* title, HWND parent);
    GFRESULT DestroyToolbar(void);
    GFRESULT AddTool(gfChar* name, gfInt resource);

    HWND GetHWND(void);
    gfBool IsHorizontal(void);
    gfBool IsRadio(void);
    GFRESULT SetCurrentTool(gfInt tool);
    gfInt GetNumTools();

    // init callbacks
    void SetNotifyFunc(void(*notifyx)(gfInt tool));

    // auxiliary functions for drawing
    void DrawButtonBorder(HDC hDC, RECT& rect, COLORREF clr_face, gfInt border, gfBool raisen);
    COLORREF ModifyColor(COLORREF clrFill, short sDelta);
    COLORREF GetHighLightColor(COLORREF clrFill);
    COLORREF GetLightColor(COLORREF clrFill);
    COLORREF GetShadowColor(COLORREF clrFill);
    COLORREF GetDarkColor(COLORREF clrFill);
};

#endif